﻿Imports System.Collections.ObjectModel
Imports Microsoft.VisualBasic.ApplicationServices

Namespace My
    ' The following events are available for MyApplication:
    ' Startup: Raised when the application starts, before the startup form is created.
    ' Shutdown: Raised after all application forms are closed.  This event is not raised if the application terminates abnormally.
    ' UnhandledException: Raised if the application encounters an unhandled exception.
    ' StartupNextInstance: Raised when launching a single-instance application and the application is already active. 
    ' NetworkAvailabilityChanged: Raised when the network connection is connected or disconnected.
    Partial Friend Class MyApplication
        Protected Overrides Function OnInitialize(commandLineArgs As ReadOnlyCollection(Of String)) As Boolean
            ' ----- Display the splash form for at least 3 seconds.
            My.Application.MinimumSplashScreenDisplayTime = 3000
            Return MyBase.OnInitialize(commandLineArgs)
        End Function

        Private Sub MyApplication_Startup(sender As Object, e As StartupEventArgs) Handles Me.Startup
            ' ----- The application starts here.
            Dim productionDBVersion As String
			Dim testConnect As SqlConnection

            ' ----- Perform general initialization.
            InitializeSystem()

			' ----- Make sure we can connect to the database.
			testConnect = ConnectDatabase()
			If (testConnect Is Nothing) Then
				Me.HideSplashScreen()
				e.Cancel = True
				Return
			End If

            ' ----- Check the database version.
            productionDBVersion = GetSystemValue("DatabaseVersion").Trim()
            If (CInt(Val(productionDBVersion)) <> UseDBVersion) Then
                MessageBox.Show(
                    "The program cannot continue due to an incompatible database. " &
                    "The current database version is '" & productionDBVersion &
                    "'. The application version is '" & UseDBVersion & "'.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Me.HideSplashScreen()
                e.Cancel = True
                Return
            End If

            ' ----- Load some settings that reside in the database.
            LoadDatabaseSettings()
            RefreshHolidays()
        End Sub

        Private Sub MyApplication_StartupNextInstance(sender As Object, e As StartupNextInstanceEventArgs) Handles Me.StartupNextInstance
            ' ----- Force the main form to the front.
            My.Application.MainForm.Activate()
        End Sub

        Private Sub MyApplication_UnhandledException(sender As Object, e As UnhandledExceptionEventArgs) Handles Me.UnhandledException
            ' ----- Record the error, and keep running.
            e.ExitApplication = False
            GeneralError("Unhandled Exception", e.Exception)
        End Sub
    End Class
End Namespace
