﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class AuthorAddLocate
    Private ActivePanel As Integer

    Private Const PanelType As Integer = 1
    Private Const PanelCriteria As Integer = 2
    Private Const PanelResults As Integer = 3

    Private Function FindUserRecords(lastText As String, firstText As String) As Integer
        ' ----- Search for author names using user-specified criteria.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim matches As Integer

        ' ----- Clear any existing values.
        SearchResults.Items.Clear()
        Me.Cursor = Cursors.WaitCursor

        ' ----- Retrieve the values from the database.
        matches = 0
        sqlText = "SELECT * FROM Author WHERE LastName LIKE @TestLast"
        If (firstText.Length <> 0) Then sqlText &= " AND FirstName LIKE @TestFirst"
        sqlText &= " ORDER BY LastName, FirstName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestLast", lastText)
        If (firstText.Length <> 0) Then _
            sqlRun.AddText("TestFirst", firstText)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("AuthorAddLocate.FindUserRecords", ex)
            Return -1
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            SearchResults.Items.Add(New ListItemData(dbRow.FormatAuthorName(),
                DBGetLong(dbRow!ID)))
            matches += 1
            If (matches >= SearchMatchLimit) Then Exit For
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
        Me.Cursor = Cursors.Default
        Return matches
    End Function

    Public Function PromptUser(ByRef nameTypeID As Long) As Long
        ' ----- Prompt the user for a author. Return the matching ID. Return
        '       -1 if no change made.
        ActivePanel = PanelType
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then
            nameTypeID = CLng(DirectCast(NameType.SelectedItem, ListItemData))
            If (CLng(DirectCast(SearchResults.SelectedItem, ListItemData)) = -1&) Then
                ' ----- Prompt for a new author using the original
                '       typed name.
                Return (New Author).AddRecord(LastName.Text.Trim(),
                    FirstName.Text.Trim())
            Else
                ' ----- The user selected a match.
                Return CLng(DirectCast(SearchResults.SelectedItem, ListItemData))
            End If
        Else
            ' ----- Cancelled by user.
            nameTypeID = -1&
            Return -1&
        End If
    End Function

    Private Sub ActBack_Click(sender As Object, e As EventArgs) Handles ActBack.Click
        ' ----- Go back to the previous panel.
        Select Case ActivePanel
            Case PanelCriteria
                ' ----- Go back to the Name Type panel.
                ActivePanel = PanelType
                SecondPanel.Visible = False
                FirstPanel.Visible = True
                ActBack.Enabled = False
                NameType.Focus()
            Case PanelResults
                ' ----- Go back to the Last/First Name panel.
                ActivePanel = PanelCriteria
                ThirdPanel.Visible = False
                SecondPanel.Visible = True
                ActNext.Text = "&Next >"
                LastName.Focus()
        End Select
    End Sub

    Private Sub ActNext_Click(sender As Object, e As EventArgs) Handles ActNext.Click
        ' ----- Take action on the user input.
        Dim lastText As String
        Dim firstText As String
        Dim matches As Integer

        ' ----- Different code for each of the panels.
        If (ActivePanel = PanelType) Then
            ' ----- Make sure a name type is selected.
            If (CLng(DirectCast(NameType.SelectedItem, ListItemData)) = -1&) Then
                MessageBox.Show("Please select a name type from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                NameType.Focus()
                Return
            End If

            ' ----- Move to the search panel.
            ActivePanel = PanelCriteria
            SecondPanel.Visible = True
            FirstPanel.Visible = False
            ActBack.Enabled = True
            LastName.Focus()
        ElseIf (ActivePanel = PanelCriteria) Then
            ' ----- Determine the search string.
            lastText = LastName.Text.Trim()
            If (lastText.Length = 0) Then
                MessageBox.Show("Please supply a last name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                LastName.Focus()
                Return
            End If
            If (lastText.Contains("*") = False) Then lastText &= "*"
            lastText = lastText.Replace("*", "%")
            firstText = FirstName.Text.Trim()
            If (firstText.Contains("*") = False) Then firstText &= "*"
            firstText = firstText.Replace("*", "%")

            ' ----- Search for matches.
            matches = FindUserRecords(lastText, firstText)
            If (matches = -1) Then Return

            ' ----- If there are no matches, then either error out, or
            '       add a new record, depending on the user's security.
            '       If there are matches, show them.
            If (matches = 0) Then
                If (SecurityProfile(LibrarySecurity.ManageAuthors)) Then
                    ' ----- Prompt to add a new record.
                    SearchResults.ClearSelected()
                    SearchResults.Items.Add(New ListItemData(
                        "<No matches. Add new author or name.>", -1&))
                    SearchResults.SelectedIndex = 0

                    ' ----- Return to the PromptUser() routine.
                    Me.DialogResult = DialogResult.OK
                Else
                    ' ----- Give an error.
                    MessageBox.Show("No matches.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    Return
                End If
            Else
                ' ----- If the user can add new author, add that option.
                If (SecurityProfile(LibrarySecurity.ManageAuthors)) Then _
                    SearchResults.Items.Insert(0, New ListItemData(
                    "<None of these matches. Add new author or name.>", -1&))
                ActivePanel = PanelResults
                ThirdPanel.Visible = True
                SecondPanel.Visible = False
                ActNext.Text = "Finish"
                SearchResults.Focus()
            End If
        Else
            ' ----- Check for a selected result.
            If (SearchResults.SelectedIndex = -1) Then
                MessageBox.Show("Please select a match from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                SearchResults.Focus()
                Return
            End If

            ' ----- Return to the PromptUser() routine.
            Me.DialogResult = DialogResult.OK
        End If
    End Sub

    Private Sub SearchResults_DoubleClick(sender As Object, e As EventArgs) Handles SearchResults.DoubleClick
        ' ----- Same as the finish button.
        If (SearchResults.SelectedIndex = -1) Then Return
        ActNext.PerformClick()
    End Sub

    Private Sub AuthorAddLocate_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Adjust the wizard panels.
        SecondPanel.Location = New Point(FirstPanel.Location.X, FirstPanel.Location.Y)
        ThirdPanel.Location = New Point(FirstPanel.Location.X,
            FirstPanel.Location.Y)

        ' ----- Fill in the list of named items.
        NameType.Items.Add(New ListItemData("<Select a Name Type>", -1&))
        NameType.SelectedIndex = 0
        sqlText = "SELECT * FROM CodeAuthorType ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("AuthorAddLocate.AuthorAddLocate_Load", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            NameType.Items.Add(New ListItemData(DBGetText(dbRow!FullName),
                DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Private Sub LastName_Enter(sender As Object, e As EventArgs) Handles LastName.Enter
        ' ----- Highlight the entire text.
        LastName.SelectAll()
    End Sub

    Private Sub FirstName_Enter(sender As Object, e As EventArgs) Handles FirstName.Enter
        ' ----- Highlight the entire text.
        FirstName.SelectAll()
    End Sub
End Class