﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Imports System.ComponentModel

Public MustInherit Class BarcodeItemGeneric
    <Browsable(False)> Public MustOverride ReadOnly Property ItemType() As String
    Public MustOverride Overrides Function ToString() As String
End Class

Public Class BarcodeItemText
    ' ----- Includes a static text element in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of properties.
    Private StoredTextAngle As Short
    Private StoredDisplayText As String

    ' ----- The ItemType constant helps when saving the record
    '       to the database.
    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        Get
            Return "T"
        End Get
    End Property

    Public Overrides Function ToString() As String
        If (StoredDisplayText.Length = 0) Then
            Return "Text: [Undefined]"
        Else
            Return $"Text: {StoredDisplayText}"
        End If
    End Function

    ' ----- Upper-left corner of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the left-edge position of the text box.")>
    Public Property Left As Single

    ' ----- Upper-left corner of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the top-edge position of the text box.")>
    Public Property Top As Single

    ' ----- The width and height of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the width of the text box.")>
    Public Property Width As Single

    ' ----- The width and height of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the height of the text box.")>
    Public Property Height As Single

    ' ----- The text color.
    <Browsable(True), DescriptionAttribute("Sets the color of the displayed text.")>
    Public Property TextColor As Color

    ' ----- Rotation angle of the text.
    <Browsable(True),
        DescriptionAttribute("Sets the clockwise rotation angle, from 0 to 359 degrees.")>
    Public Property RotationAngle() As Short
        Get
            Return StoredTextAngle
        End Get
        Set(ByVal Value As Short)
            ' ----- Make sure the angle is in the valid range.
            If (Value < 0) Or (Value > 359) Then
                Throw New ArgumentException(
                    $"Invalid rotation angle: {Value}. " &
                    "Rotation angle must range from 0 to 359 degrees.")
                Return
            End If
            StoredTextAngle = Value
        End Set
    End Property

    ' ----- The font of the text.
    <Browsable(True), DescriptionAttribute("Sets the font of the text.")>
    Public Property Font As Font

    ' ----- The actual text.
    <Browsable(True), DescriptionAttribute("Sets the text to display.")>
    Public Property DisplayText() As String
        Get
            Return StoredDisplayText
        End Get
        Set(ByVal Value As String)
            ' ----- Make sure the angle is in the valid range.
            If (Value.Length > 100) Then
                Throw New ArgumentException(
                    "Invalid description. Text is limited to 100 characters maximum.")
                Return
            End If
            StoredDisplayText = Value
        End Set
    End Property

    ' ----- The alignment of the text.
    <Browsable(True), DescriptionAttribute("Sets the alignment of the text.")>
    Public Property Alignment As ContentAlignment
End Class

Public Class BarcodeItemLine
    ' ----- Includes a basic line element in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredLineY1 As Single

    ' ----- The ItemType constant helps when saving the record
    '       to the database.
    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        Get
            Return "L"
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return $"Line: ({Me.X1},{Me.Y1}) to ({Me.X2},{Me.Y2})"
    End Function

    ' ----- Starting point of the line.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the starting horizontal position of the line.")>
    Public Property X1 As Single

    ' ----- Starting point of the line.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the starting vertical position of the line.")>
    Public Property Y1 As Single

    ' ----- Ending point of the line.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the ending horizontal position of the line.")>
    Public Property X2 As Single

    ' ----- Ending point of the line.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the ending vertical position of the line.")>
    Public Property Y2 As Single

    ' ----- The line color.
    <Browsable(True), DescriptionAttribute("Sets the color of the line.")>
    Public Property LineColor As Color
End Class

Public Class BarcodeItemBarcodeImage
    ' ----- Includes an actual barcode element in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredBarAngle As Short
    Private StoredBarPadDigits As Byte

    ' ----- The ItemType constant helps when saving the record
    '       to the database.
    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        Get
            Return "B"
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return $"Barcode: At ({Me.Left},{Me.Top})"
    End Function

    ' ----- Upper-left corner of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the left-edge position of the barcode box.")>
    Public Property Left As Single

    ' ----- Upper-left corner of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the top-edge position of the barcode box.")>
    Public Property Top As Single

    ' ----- The width of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the width of the barcode box.")>
    Public Property Width As Single

    ' ----- The height of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the height of the barcode box.")>
    Public Property Height As Single

    ' ----- The barcode font size.
    <Browsable(True), DescriptionAttribute("Sets the size of the barcode, in points.")>
    Public Property BarcodeSize As Single

    ' ----- The barcode color.
    <Browsable(True), DescriptionAttribute("Sets the color of the displayed barcode.")>
    Public Property BarcodeColor As Color

    ' ----- Rotation angle of the text.
    <Browsable(True),
        DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")>
    Public Property RotationAngle() As Short
        Get
            Return StoredBarAngle
        End Get
        Set(ByVal Value As Short)
            ' ----- Make sure the angle is in the valid range.
            If (Value < 0) Or (Value > 359) Then
                Throw New ArgumentException(
                    $"Invalid rotation angle: {Value}. " &
                    "Rotation angle must range from 0 to 359 degrees.")
                Return
            End If
            StoredBarAngle = Value
        End Set
    End Property

    ' ----- The alignment of the text.
    <Browsable(True), DescriptionAttribute("Sets the alignment of the barcode.")>
    Public Property Alignment As ContentAlignment

    ' ----- Number of pad digits.
    <Browsable(True),
        DescriptionAttribute("The number of digits in which to pad the barcode number, or 0 to ignore.")>
    Public Property PadDigits() As Byte
        Get
            Return StoredBarPadDigits
        End Get
        Set(ByVal Value As Byte)
            ' ----- Make sure the count is in the valid range.
            If (Value < 0) Or (Value > 20) Then
                Throw New ArgumentException(
                    $"Invalid pad digits value: {Value}. " &
                    "Pad digits must range from 0 to 20.")
                Return
            End If
            StoredBarPadDigits = Value
        End Set
    End Property
End Class

Public Class BarcodeItemBarcodeNumber
    ' ----- Includes the numeric version of a barcode in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredTextAngle As Short
    Private StoredTextPadDigits As Byte

    ' ----- The ItemType constant helps when saving the record
    '       to the database.
    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        Get
            Return "N"
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return $"Barcode Text: At ({Me.Left},{Me.Top})"
    End Function

    ' ----- Upper-left corner of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the left-edge position of the text box.")>
    Public Property Left As Single

    ' ----- Upper-left corner of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the top-edge position of the text box.")>
    Public Property Top As Single

    ' ----- The width of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the width of the text box.")>
    Public Property Width As Single

    ' ----- The height of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the height of the text box.")>
    Public Property Height As Single

    ' ----- The text color.
    <Browsable(True), DescriptionAttribute("Sets the color of the displayed text.")>
    Public Property TextColor As Color

    ' ----- Rotation angle of the text.
    <Browsable(True),
        DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")>
    Public Property RotationAngle() As Short
        Get
            Return StoredTextAngle
        End Get
        Set(ByVal Value As Short)
            ' ----- Make sure the angle is in the valid range.
            If (Value < 0) Or (Value > 359) Then
                Throw New ArgumentException(
                    $"Invalid rotation angle: {Value}. " &
                    "Rotation angle must range from 0 to 359 degrees.")
                Return
            End If
            StoredTextAngle = Value
        End Set
    End Property

    ' ----- The font of the text.
    <Browsable(True), DescriptionAttribute("Sets the font of the text.")>
    Public Property Font As Font

    ' ----- The alignment of the text.
    <Browsable(True), DescriptionAttribute("Sets the alignment of the text.")>
    Public Property Alignment As ContentAlignment

    ' ----- Number of pad digits.
    <Browsable(True),
        DescriptionAttribute("The number of digits in which to pad the barcode number, or 0 to ignore.")>
    Public Property PadDigits() As Byte
        Get
            Return StoredTextPadDigits
        End Get
        Set(ByVal Value As Byte)
            ' ----- Make sure the count is in the valid range.
            If (Value < 0) Or (Value > 20) Then
                Throw New ArgumentException(
                    $"Invalid pad digits value: {Value}. " &
                    "Pad digits must range from 0 to 20.")
                Return
            End If
            StoredTextPadDigits = Value
        End Set
    End Property
End Class

Public Class BarcodeItemRect
    ' ----- Includes a basic rectangle element in a barcode label.
    Inherits BarcodeItemGeneric

    ' ----- Private store of attributes.
    Private StoredRectAngle As Short

    ' ----- The ItemType constant helps when saving the record
    '       to the database.
    <Browsable(False)> Public Overrides ReadOnly Property ItemType() As String
        Get
            Return "R"
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return $"Rectangle: At ({Me.Left},{Me.Top})"
    End Function

    ' ----- Upper-left corner of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the left-edge position of the rectangle.")>
    Public Property Left As Single

    ' ----- Upper-left corner of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the top-edge position of the rectangle.")>
    Public Property Top As Single

    ' ----- The width and height of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the width of the rectangle.")>
    Public Property Width As Single

    ' ----- The width and height of the text box.
    <Browsable(True), DefaultValueAttribute("0"),
        DescriptionAttribute("Sets the height of the rectangle.")>
    Public Property Height As Single

    ' ----- The line color.
    <Browsable(True), DescriptionAttribute("Sets the border color of the rectangle.")>
    Public Property BorderColor As Color

    ' ----- The fill color.
    <Browsable(True), DescriptionAttribute("Sets the fill color of the rectangle.")>
    Public Property FillColor As Color

    ' ----- Rotation angle of the rectangle.
    <Browsable(True),
        DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")>
    Public Property RotationAngle() As Short
        Get
            Return StoredRectAngle
        End Get
        Set(ByVal Value As Short)
            ' ----- Make sure the angle is in the valid range.
            If (Value < 0) Or (Value > 359) Then
                Throw New ArgumentException(
                    $"Invalid rotation angle: {Value}. " &
                    "Rotation angle must range from 0 to 359 degrees.")
                Return
            End If
            StoredRectAngle = Value
        End Set
    End Property
End Class
