﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class BarcodePage
    Private Const MaxPageInInches As Single = 22.0F
    Private Const TwipsPerInch As Single = 1440.0F

    Private ActiveSheetID As Long
    Private StartingOver As Boolean
    Private CurrentMeasure As Char

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveSheetID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveSheetID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the record.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete this barcode template?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question,
            MessageBoxDefaultButton.Button2) <> DialogResult.Yes) Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM BarcodeLabel WHERE BarcodeSheet = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("You cannot delete this template because it is " +
                    "in use by one or more labels templates.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        Catch ex As Exception
            GeneralError("BarcodePage.DeleteRecord.Check", ex)
            Return False
        End Try

        ' ----- Delete the record.
        sqlText = "DELETE FROM BarcodeSheet WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("BarcodePage.DeleteRecord.Delete", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveSheetID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveSheetID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("BarcodePage.FillListWithRecords", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM BarcodeSheet WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            Return DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("BarcodePage.FormatRecordName", ex)
            Return "Error"
        End Try
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return a description of this editor.
        Return "Use these codes to identify the format of pages onto wihch " &
            "barcodes are printed. You can define the layout of the page and the " &
            "number and position of barcode labels."
    End Function
    Public Overrides Function GetEditTitle() As String
        ' ----- Return the title-bar text for this editor.
        Return "Barcode Sheet Templates"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim dbRow As DataRow

        ' ----- Fill in the list of units.
        If (RecordUnits.Items.Count = 0) Then
            RecordUnits.Items.Add(New ListItemData("Inches", Asc("I")))
            RecordUnits.Items.Add(New ListItemData("Centimeters", Asc("C")))
            RecordUnits.Items.Add(New ListItemData("Points", Asc("P")))
            RecordUnits.Items.Add(New ListItemData("Twips", Asc("T")))
            RecordUnits.SelectedIndex = 0
        End If

        ' ----- Default to inches.
        CurrentMeasure = "I"c
        RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(Asc(CurrentMeasure))

        ' ----- Clear the existing data.
        StartingOver = True
        RecordName.Text = ""
        BCPageWidth.Text = ""
        BCPageHeight.Text = ""
        BCLeftMargin.Text = ""
        BCRightMargin.Text = ""
        BCTopMargin.Text = ""
        BCBottomMargin.Text = ""
        BCInterColumn.Text = ""
        BCInterRow.Text = ""
        BCColumns.Text = ""
        BCRows.Text = ""
        PreviewArea.Invalidate()
        If (ActiveSheetID = -1&) Then Return

        ' ----- Load in the record.
        sqlText = "SELECT * FROM BarcodeSheet WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveSheetID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("BarcodePage.PrepareFormFields", ex)
            Return
        End Try
        dbRow = dbInfo.Rows(0)
        RecordName.Text = DBGetText(dbRow!FullName)
        RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(Asc(CChar(DBGetText(dbRow!UnitType))))
        If (RecordUnits.SelectedIndex = -1) Then RecordUnits.SelectedIndex = 0
        CurrentMeasure = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        BCPageWidth.Text = DBGetDecimal(dbRow!PageWidth).ToString("0.####")
        BCPageHeight.Text = DBGetDecimal(dbRow!PageHeight).ToString("0.####")
        BCLeftMargin.Text = DBGetDecimal(dbRow!MarginLeft).ToString("0.####")
        BCRightMargin.Text = DBGetDecimal(dbRow!MarginRight).ToString("0.####")
        BCTopMargin.Text = DBGetDecimal(dbRow!MarginTop).ToString("0.####")
        BCBottomMargin.Text = DBGetDecimal(dbRow!MarginBottom).ToString("0.####")
        BCInterColumn.Text = DBGetDecimal(dbRow!IntraColumn).ToString("0.####")
        BCInterRow.Text = DBGetDecimal(dbRow!IntraRow).ToString("0.####")
        BCColumns.Text = DBGetInteger(dbRow!ColumnsCount).ToString()
        BCRows.Text = DBGetInteger(dbRow!RowsCount).ToString()
        dbInfo.Dispose()
        PreviewArea.Invalidate()
    End Sub

    Private Sub BarcodePage_Activated(sender As Object, e As EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordName_Enter(sender As Object, e As EventArgs) Handles RecordName.Enter
        ' ----- Highlight the entire text.
        RecordName.SelectAll()
    End Sub

    Private Sub BCPageWidth_Enter(sender As Object, e As EventArgs) Handles BCPageWidth.Enter
        ' ----- Highlight the entire text.
        BCPageWidth.SelectAll()
    End Sub

    Private Sub BCPageHeight_Enter(sender As Object, e As EventArgs) Handles BCPageHeight.Enter
        ' ----- Highlight the entire text.
        BCPageHeight.SelectAll()
    End Sub

    Private Sub BCLeftMargin_Enter(sender As Object, e As EventArgs) Handles BCLeftMargin.Enter
        ' ----- Highlight the entire text.
        BCLeftMargin.SelectAll()
    End Sub

    Private Sub BCRightMargin_Enter(sender As Object, e As EventArgs) Handles BCRightMargin.Enter
        ' ----- Highlight the entire text.
        BCRightMargin.SelectAll()
    End Sub

    Private Sub BCTopMargin_Enter(sender As Object, e As EventArgs) Handles BCTopMargin.Enter
        ' ----- Highlight the entire text.
        BCTopMargin.SelectAll()
    End Sub

    Private Sub BCBottomMargin_Enter(sender As Object, e As EventArgs) Handles BCBottomMargin.Enter
        ' ----- Highlight the entire text.
        BCBottomMargin.SelectAll()
    End Sub

    Private Sub BCInterColumn_Enter(sender As Object, e As EventArgs) Handles BCInterColumn.Enter
        ' ----- Highlight the entire text.
        BCInterColumn.SelectAll()
    End Sub

    Private Sub BCInterRow_Enter(sender As Object, e As EventArgs) Handles BCInterRow.Enter
        ' ----- Highlight the entire text.
        BCInterRow.SelectAll()
    End Sub

    Private Sub BCColumns_Enter(sender As Object, e As EventArgs) Handles BCColumns.Enter
        ' ----- Highlight the entire text.
        BCColumns.SelectAll()
    End Sub

    Private Sub BCRows_Enter(sender As Object, e As EventArgs) Handles BCRows.Enter
        ' ----- Highlight the entire text.
        BCRows.SelectAll()
    End Sub

    Private Function ValidateForm() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Name is a required field.
        If (RecordName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply a template name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM BarcodeSheet WHERE UPPER(FullName) = @TestName"
        If (ActiveSheetID <> -1&) Then sqlText &= " AND ID <> @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestName", RecordName.Text.Trim().ToUpper())
        If (ActiveSheetID <> -1&) Then _
            sqlRun.AddLong("RecordID", ActiveSheetID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                ' ----- Duplicate entry.
                MessageBox.Show("A record already exists with the name '" &
                    RecordName.Text.Trim() & "'.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordName.Focus()
                Return False
            End If
        Catch ex As Exception
            GeneralError("BarcodePage.ValidateForm", ex)
            Return False
        End Try

        ' ----- Check the other fields.
        If (VerifyPageSize(False) = False) Then Return False
        If (VerifyLayout(False) = False) Then Return False

        ' ----- Success.
        Return True
    End Function

    Private Function VerifyLayout(silent As Boolean) As Boolean
        ' ----- Check the values supplied by the user to see if they are valid.
        '       This routine assumes that VerifyPageSize() has been called.
        If (IsNumeric(BCLeftMargin.Text) = False) Or (Val(BCLeftMargin.Text) < 0) Then
            If (silent = False) Then MessageBox.Show("Left Margin is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (IsNumeric(BCRightMargin.Text) = False) Or (Val(BCRightMargin.Text) < 0) Then
            If (silent = False) Then MessageBox.Show("Right Margin is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (IsNumeric(BCTopMargin.Text) = False) Or (Val(BCTopMargin.Text) < 0) Then
            If (silent = False) Then MessageBox.Show("Top Margin is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (IsNumeric(BCBottomMargin.Text) = False) Or (Val(BCBottomMargin.Text) < 0) Then
            If (silent = False) Then MessageBox.Show("Bottom Margin is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (IsNumeric(BCInterColumn.Text) = False) Or (Val(BCInterColumn.Text) < 0) Then
            If (silent = False) Then MessageBox.Show("Intra-Column Spacing is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (IsNumeric(BCInterRow.Text) = False) Or (Val(BCInterRow.Text) < 0) Then
            If (silent = False) Then MessageBox.Show("Intra-Row Spacing is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (IsNumeric(BCColumns.Text) = False) Or (Val(BCColumns.Text) <= 0) Then
            If (silent = False) Then MessageBox.Show("Column Count is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (IsNumeric(BCRows.Text) = False) Or (Val(BCRows.Text) <= 0) Then
            If (silent = False) Then MessageBox.Show("Row Count is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If

        ' ----- Make sure there is space for the labels.
        If ((Val(BCPageWidth.Text) - ((Val(BCColumns.Text) - 1) *
                Val(BCInterColumn.Text))) <= 0) Then
            If (silent = False) Then MessageBox.Show("Insufficient horizontal space provided.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If ((Val(BCPageHeight.Text) - ((Val(BCRows.Text) - 1) *
                Val(BCInterRow.Text))) <= 0) Then
            If (silent = False) Then MessageBox.Show("Insufficient vertical space provided.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function VerifyPageSize(silent As Boolean) As Boolean
        ' ----- Check the values supplied by the user to see if they are valid.
        Dim pageWidth As Single
        Dim pageHeight As Single
        Dim fromUnit As Char

        ' ----- Check for valid width and height.
        If (IsNumeric(BCPageWidth.Text) = False) Or (Val(BCPageWidth.Text) <= 0) Then
            If (silent = False) Then MessageBox.Show("Page Width is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (IsNumeric(BCPageHeight.Text) = False) Or (Val(BCPageHeight.Text) <= 0) Then
            If (silent = False) Then MessageBox.Show("Page Height is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If

        ' ----- The width and height are limited to a maximum size.
        fromUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        pageWidth = ConvertPageUnits(fromUnit, "T"c, CSng(BCPageWidth.Text))
        pageHeight = ConvertPageUnits(fromUnit, "T"c, CSng(BCPageHeight.Text))
        If (pageWidth > (TwipsPerInch * MaxPageInInches)) Then
            If (silent = False) Then MessageBox.Show("The width of the page is limited to " &
                ConvertPageUnits("T"c, fromUnit, MaxPageInInches) & " " &
                RecordUnits.Text & ".",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If
        If (pageHeight > (TwipsPerInch * MaxPageInInches)) Then
            If (silent = False) Then MessageBox.Show("The height of the page is limited to " &
                ConvertPageUnits("T"c, fromUnit, MaxPageInInches) & " " &
                RecordUnits.Text & ".",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub PreviewArea_Paint(sender As Object, e As PaintEventArgs) Handles PreviewArea.Paint
        ' ----- Display a preview of the image.
        Dim pageWidth As Integer
        Dim pageHeight As Integer
        Dim trueWidth As Integer
        Dim trueHeight As Integer
        Dim pageLeft As Integer  ' Start of the "white" area.
        Dim pageTop As Integer   ' Start of the "white" area.
        Dim oneWidth As Single
        Dim oneHeight As Single
        Dim oneWidthTwips As Integer
        Dim oneHeightTwips As Integer
        Dim rowScan As Integer
        Dim colScan As Integer
        Dim leftOffset As Integer
        Dim topOffset As Integer
        Dim useRatio As Single
        Dim fromUnit As Char

        On Error Resume Next

        ' ----- Draw in the background color.
        e.Graphics.Clear(SystemColors.Control)

        ' ----- Make sure that a valid page size is supplied.
        If (VerifyPageSize(True) = False) Then Return

        ' ----- Determine the page area.
        fromUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        trueWidth = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(BCPageWidth.Text)))
        trueHeight = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(BCPageHeight.Text)))
        If (trueWidth = 0) Or (trueHeight = 0) Then Return
        pageWidth = PreviewArea.Width - 4
        pageHeight = PreviewArea.Height - 4
        useRatio = CSng(pageWidth) / CSng(trueWidth)
        If (CInt(trueHeight * useRatio) > pageHeight) Then
            ' ----- Scale based on the height.
            useRatio = CSng(pageHeight) / CSng(trueHeight)
            pageWidth = CInt(trueWidth * useRatio)
        Else
            ' ----- Scale based on the width.
            pageHeight = CInt(trueHeight * useRatio)
        End If
        pageLeft = CInt((PreviewArea.Width - 4 - pageWidth) / 2) + 1
        pageTop = CInt((PreviewArea.Height - 4 - pageHeight) / 2) + 1

        ' ----- Draw the page area.
        e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 1,
            pageTop + 1, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 2,
            pageTop + 2, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(Brushes.Black, pageLeft - 1, pageTop - 1,
            pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(Brushes.White, pageLeft, pageTop, pageWidth, pageHeight)

        ' ----- Check the rest of the fields.
        If (VerifyLayout(True) = False) Then Return

        ' ----- Transform a page of twips into a display page.
        e.Graphics.TranslateTransform(pageLeft, pageTop)
        e.Graphics.ScaleTransform(useRatio, useRatio)

        ' ----- Figure out the size of one label.
        oneWidth = (CSng(BCPageWidth.Text) - CSng(BCLeftMargin.Text) -
            CSng(BCRightMargin.Text) - ((CSng(BCColumns.Text) - 1.0F) *
            CSng(BCInterColumn.Text))) / CSng(BCColumns.Text)
        oneHeight = (CSng(BCPageHeight.Text) - CSng(BCTopMargin.Text) -
            CSng(BCBottomMargin.Text) - ((CSng(BCRows.Text) - 1.0F) *
            CSng(BCInterRow.Text))) / CSng(BCRows.Text)
        oneWidthTwips = CInt(ConvertPageUnits(fromUnit, "T"c, oneWidth))
        oneHeightTwips = CInt(ConvertPageUnits(fromUnit, "T"c, oneHeight))

        ' ----- Process each row and column.
        For rowScan = 1 To CInt(BCRows.Text)
            For colScan = 1 To CInt(BCColumns.Text)
                leftOffset = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(BCLeftMargin.Text) +
                    ((colScan - 1) * (CSng(BCInterColumn.Text) + oneWidth))))
                topOffset = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(BCTopMargin.Text) +
                    ((rowScan - 1) * (CSng(BCInterRow.Text) + oneHeight))))
                e.Graphics.DrawRectangle(Pens.Cyan, leftOffset, topOffset,
                    oneWidthTwips, oneHeightTwips)
            Next colScan
        Next rowScan

        ' ----- Finished.
        e.Graphics.ResetTransform()
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long

        ' ----- Confirm the data.
        If (ValidateForm() = False) Then Return
        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveSheetID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO BarcodeSheet (FullName, UnitType, PageWidth, " &
                "PageHeight, MarginLeft, MarginRight, MarginTop, MarginBottom, " &
                "IntraColumn, IntraRow, ColumnsCount, RowsCount) OUTPUT INSERTED.ID " &
                "VALUES (@NewName, @NewUnit, @NewWidth, @NewHeight, @NewLeft, @NewRight, " &
                "@NewTop, @NewBottom, @NewColumn, @NewRow, @NewColCount, @NewRowCount)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordName.Text.Trim())
            sqlRun.AddText("NewUnit", Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData)))
            sqlRun.AddDecimal("NewWidth", BCPageWidth.Text)
            sqlRun.AddDecimal("NewHeight", BCPageHeight.Text)
            sqlRun.AddDecimal("NewLeft", BCLeftMargin.Text)
            sqlRun.AddDecimal("NewRight", BCRightMargin.Text)
            sqlRun.AddDecimal("NewTop", BCTopMargin.Text)
            sqlRun.AddDecimal("NewBottom", BCBottomMargin.Text)
            sqlRun.AddDecimal("NewColumn", BCInterColumn.Text)
            sqlRun.AddDecimal("NewRow", BCInterRow.Text)
            sqlRun.AddInteger("NewColCount", BCColumns.Text)
            sqlRun.AddInteger("NewRowCount", BCRows.Text)
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                GeneralError("BarcodePage.ActOK_Click.Add", ex)
                Return
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveSheetID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE BarcodeSheet SET FullName = @NewName, " &
                "UnitType = @NewUnit, PageWidth = @NewWidth, " &
                "PageHeight = @NewHeight, MarginLeft = @NewLeft, " &
                "MarginRight = @NewRight, MarginTop = @NewTop, " &
                "MarginBottom = @NewBottom, IntraColumn = @NewColumn, " &
                "IntraRow = @NewRow, ColumnsCount = @NewColCount, " &
                "RowsCount = @NewRowCount WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordName.Text.Trim())
            sqlRun.AddText("NewUnit", Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData)))
            sqlRun.AddDecimal("NewWidth", BCPageWidth.Text)
            sqlRun.AddDecimal("NewHeight", BCPageHeight.Text)
            sqlRun.AddDecimal("NewLeft", BCLeftMargin.Text)
            sqlRun.AddDecimal("NewRight", BCRightMargin.Text)
            sqlRun.AddDecimal("NewTop", BCTopMargin.Text)
            sqlRun.AddDecimal("NewBottom", BCBottomMargin.Text)
            sqlRun.AddDecimal("NewColumn", BCInterColumn.Text)
            sqlRun.AddDecimal("NewRow", BCInterRow.Text)
            sqlRun.AddInteger("NewColCount", BCColumns.Text)
            sqlRun.AddInteger("NewRowCount", BCRows.Text)
            sqlRun.AddLong("RecordID", ActiveSheetID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("BarcodePage.ActOK_Click.Update", ex)
                Return
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Continue with AddRecord() or EditRecord()
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub BCPageWidth_TextChanged(sender As Object,
            e As EventArgs) Handles BCPageWidth.TextChanged,
            BCPageHeight.TextChanged, BCLeftMargin.TextChanged,
            BCRightMargin.TextChanged, BCTopMargin.TextChanged,
            BCBottomMargin.TextChanged, BCInterColumn.TextChanged,
            BCInterRow.TextChanged, BCRows.TextChanged, BCColumns.TextChanged
        ' ----- Refresh the preview.
        PreviewArea.Invalidate()
    End Sub

    Private Sub RecordUnits_SelectedIndexChanged(sender As Object, e As EventArgs) Handles RecordUnits.SelectedIndexChanged
        ' ----- Refresh the preview.
        Dim newMeasure As Char

        ' ----- See if the measure changed.
		If (RecordUnits.SelectedIndex = -1) Then Return
        newMeasure = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        If (CurrentMeasure <> newMeasure) Then
            ' ----- Convert the measure.
            If (IsNumeric(BCPageWidth.Text) = True) Then BCPageWidth.Text =
                (ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCPageWidth.Text))).ToString()
            If (IsNumeric(BCPageHeight.Text) = True) Then BCPageHeight.Text =
                (ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCPageHeight.Text))).ToString()
            If (IsNumeric(BCLeftMargin.Text) = True) Then BCLeftMargin.Text =
                (ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCLeftMargin.Text))).ToString()
            If (IsNumeric(BCRightMargin.Text) = True) Then BCRightMargin.Text =
                (ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCRightMargin.Text))).ToString()
            If (IsNumeric(BCTopMargin.Text) = True) Then BCTopMargin.Text =
                (ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCTopMargin.Text))).ToString()
            If (IsNumeric(BCBottomMargin.Text) = True) Then BCBottomMargin.Text =
                (ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCBottomMargin.Text))).ToString()
            If (IsNumeric(BCInterColumn.Text) = True) Then BCInterColumn.Text =
                (ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCInterColumn.Text))).ToString()
            If (IsNumeric(BCInterRow.Text) = True) Then BCInterRow.Text =
                (ConvertPageUnits(CurrentMeasure, newMeasure, CSng(BCInterRow.Text))).ToString()
            CurrentMeasure = newMeasure
        End If
        PreviewArea.Invalidate()
    End Sub

    Private Sub BarcodePage_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "BarcodePage.htm")
    End Sub
End Class
