﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Imports System.Text
Imports System.Security.Cryptography

Module General
    ' ----- Public constants.
    Public Const ProgramTitle As String = "The Library Project"
    Public Const NotAuthorizedMessage As String =
        "You are not authorized to perform this task."
    Public Const UseDBVersion As Integer = 1

    ' ----- Constants for the MatchingImages image list.
    Public Const MatchPresent As Integer = 0
    Public Const MatchNone As Integer = 1

    Public Enum LookupMethods As Integer
        ByTitle = 1
        ByAuthor = 2
        BySubject = 3
        ByKeywordAny = 4
        ByKeywordAll = 5
        ByPublisher = 6
        BySeries = 7
        BySeriesID = 8
        ByBarcode = 9
        ByDatabaseID = 10
        ByAuthorID = 11
        BySubjectID = 12
        ByPublisherID = 13
    End Enum

    ' ----- Security values.
    Public Enum LibrarySecurity As Integer
        ManageAuthors = 1
        ManageAuthorTypes = 2
        ManageCopyStatus = 3
        ManageMediaTypes = 4
        ManageSeries = 5
        ManageGroups = 6
        ManageItems = 7
        ManagePatrons = 8
        ManagePublishers = 9
        ManageValues = 10
        ManageUsers = 11
        ProcessFees = 12
        ManageLocations = 13
        CheckOutItems = 14
        CheckInItems = 15
        AdminFeatures = 16
        DailyProcessing = 17
        RunReports = 18
        PatronOverride = 19
        ManageBarcodeTemplates = 20
        ManageHolidays = 21
        ManagePatronGroups = 22
        ViewAdminPatronMessages = 23
    End Enum
    Public Const MaxLibrarySecurity As LibrarySecurity = LibrarySecurity.ViewAdminPatronMessages

    ' ----- Global variables.
    Public LoggedInUserID As Long
    Public LoggedInUserName As String
    Public LoggedInGroupID As Long
    Public SecurityProfile(MaxLibrarySecurity) As Boolean

    Public Function CenterText(origText As String, textWidth As Integer) As String
        ' ----- Center a piece of text in a field width. If the text is too
        '       wide, truncate it.
        Dim resultText As String

        resultText = origText.Trim()
        If (resultText.Length >= textWidth) Then
            ' ----- Truncate as needed.
            Return origText.Substring(0, textWidth).Trim()
        Else
            ' ----- Start with extra spaces.
            Return New String(" "c, (textWidth - origText.Length) \ 2) & resultText
        End If
    End Function

    Public Function ConnectDatabase() As SqlConnection
        ' ----- Connect to the database. Throw exception on failure.
        Static connectionString As String = ""
        Dim libraryDB As SqlConnection

        ' ----- Build the connection string.
        ' !!! WARNING: Hardcoded for now.
        If (connectionString.Length = 0) Then _
            connectionString = "Data Source=MYSYSTEM;" &
               "Initial Catalog=Library;Integrated Security=true"

        ' ----- Attempt to open the database.
        libraryDB = New SqlConnection(connectionString)
        libraryDB.Open()

        ' ----- Success.
        Return libraryDB
    End Function

    Public Function CountSubStr(mainText As String, subText As String) As Integer
        ' ----- Return a count of the number of times that a subText occurs in
        '       a string (mainText).
        Dim totalTimes As Integer
        Dim startPos As Integer
        Dim foundPos As Integer

        If (mainText Is Nothing) OrElse (mainText.Length = 0) Then Return 0
        If (subText Is Nothing) OrElse (subText.Length = 0) Then Return 0
        totalTimes = 0
        startPos = 0

        ' ----- Keep searching until we don't find it no more!
        Do
            ' ----- Search for the subText.
            foundPos = mainText.IndexOf(subText, startPos)
            If (foundPos = -1) Then Exit Do
            totalTimes += 1

            ' ----- Move to just after the occurrence.
            startPos = foundPos + subText.Length
        Loop

        ' ----- Return the count.
        Return totalTimes
    End Function

    Public Function CreateDataTable(theAction As DataCommand) As DataTable
        ' ----- Given a SQL statement with parameters, return a data table.
        Dim dbCommand As SqlCommand
        Dim dbAdapter As SqlDataAdapter
        Dim dbTable As DataTable = Nothing
        Dim localConnection As SqlConnection = Nothing

        ' ----- Connect to the database, if not yet done. Throw exception on failure.
        If (theAction.Connection Is Nothing) Then
            localConnection = ConnectDatabase()
        End If

        ' ----- Try to run the statement. Throw exception on failure.
        Try
            dbCommand = theAction.GetCommandObject()
            If (localConnection IsNot Nothing) Then _
                dbCommand.Connection = localConnection
            dbAdapter = New SqlDataAdapter(dbCommand)
            dbTable = New DataTable
            dbAdapter.Fill(dbTable)
        Catch ex As Exception
            ' ----- Reissue error after Finally clause.
			Throw ex
        Finally
            ' ----- Disconnect if needed.
            If (localConnection IsNot Nothing) Then localConnection.Close()
        End Try
        Return dbTable
    End Function

    Public Function CreateDataTable(theAction As DataCommand,
            transInfo As LibraryTransaction) As DataTable
        ' ----- Given a SQL statement with parameters, return a data table.
        Dim dbCommand As SqlCommand
        Dim dbAdapter As SqlDataAdapter
        Dim dbTable As DataTable = Nothing

        ' ----- Try to run the statement. Throw exception on failure.
        dbCommand = theAction.GetCommandObject()
        dbCommand.Connection = transInfo.Connection
        dbCommand.Transaction = transInfo.Transaction
        dbAdapter = New SqlDataAdapter(dbCommand)
        dbTable = New DataTable
        dbAdapter.Fill(dbTable)
        Return dbTable
    End Function

    Public Function CreateDataTable(sqlText As String) As DataTable
        ' ----- Given a SQL statement, return a data table.
        Dim dbCommand As SqlCommand
        Dim dbAdapter As SqlDataAdapter
        Dim dbTable As DataTable = Nothing

        ' ----- Connect to the database. Throw exception on failure.
        Using libraryDB As SqlConnection = ConnectDatabase()
            ' ----- Try to run the statement. Throw exception on failure.
            dbCommand = New SqlCommand(sqlText, libraryDB)
            dbAdapter = New SqlDataAdapter(dbCommand)
            dbTable = New DataTable
            dbAdapter.Fill(dbTable)
        End Using
        Return dbTable
    End Function

    Public Function CreateDataTable(sqlText As String,
            transInfo As LibraryTransaction) As DataTable
        ' ----- Given a SQL statement and a transaction, return a data table.
        Dim dbCommand As SqlCommand
        Dim dbAdapter As SqlDataAdapter
        Dim dbTable As DataTable

        ' ----- Try to run the statement. Throw exception on failure.
        dbCommand = New SqlCommand(sqlText, transInfo.Connection)
        dbCommand.Transaction = transInfo.Transaction
        dbAdapter = New SqlDataAdapter(dbCommand)
        dbTable = New DataTable
        dbAdapter.Fill(dbTable)
        Return dbTable
    End Function

    Public Function DBComboParam(whichField As ComboBox) As String
        ' ----- Prepare the value of a ComboBox for use in the database.
        Dim listItem As ListItemData

        ' ----- Get the stored data for this item.
        If (whichField.SelectedItem Is Nothing) Then Return ""
        listItem = DirectCast(whichField.SelectedItem, ListItemData)

        ' ----- An item value of "-1" has special meaning: unselected.
        If (listItem.ItemData = -1&) Then
            Return ""
        Else
            Return listItem.ItemData.ToString()
        End If
    End Function

    Public Function DBGetBool(ByVal dataField As Object) As Boolean
        ' ----- Return the Boolean equivalent of an optional database field.
        Return CBool(DBGetInteger(dataField))
    End Function

    Public Function DBGetDate(dataField As Object) As Date
        ' ----- Return the date equivalent of an optional database field.
        If (dataField Is Nothing) Then
            Return Date.MinValue
        ElseIf (IsDBNull(dataField) = True) Then
            Return Date.MinValue
        ElseIf (IsDate(dataField) = False) Then
            Return Date.MinValue
        Else
            Return CDate(dataField)
        End If
    End Function

    Public Function DBGetDecimal(dataField As Object) As Decimal
        ' ----- Return the decimal equivalent of an optional database field.
        If (IsDBNull(dataField) = True) Then
            Return 0@
        Else
            Return CDec(dataField)
        End If
    End Function

    Public Function DBGetInteger(dataField As Object) As Integer
        ' ----- Return the integer equivalent of an optional database field.
        If (IsDBNull(dataField) = True) Then
            Return 0
        Else
            Return CInt(dataField)
        End If
    End Function

    Public Function DBGetLong(dataField As Object,
            Optional useDefault As Long = 0&) As Long
        ' ----- Return the long equivalent of an optional database field.
        If (dataField Is Nothing) Then
            Return useDefault
        ElseIf (IsDBNull(dataField) = True) Then
            Return useDefault
        Else
            Return CLng(dataField)
        End If
    End Function

    Public Function DBGetText(dataField As Object) As String
        ' ----- Return the text equivalent of an optional database field.
        If (IsDBNull(dataField) = True) Then
            Return ""
        Else
            Return CStr(dataField)
        End If
    End Function

    Public Function DigitsOnly(origText As String) As String
        ' ----- Return only the digits found in a string.
        Dim destText As String
        Dim counter As Integer

        ' ----- Examine each character.
        destText = ""
        For counter = 0 To origText.Length - 1
            If (IsNumeric(origText.Substring(counter, 1))) Then _
                destText &= origText.Substring(counter, 1)
        Next counter
        Return destText
    End Function

    Public Function EncryptPassword(loginID As String,
            passwordText As String) As String
        ' ----- Given a username and a password, encrypt the password so that it
        '       is not easy to decrypt. There is no limit on password length since
        '       it is going to be hashed anyway.
        Dim hashingFunction As HMACSHA1
        Dim secretKey() As Byte
        Dim hashValue() As Byte
        Dim counter As Integer
        Dim result As String = ""

        ' ----- Prepare the secret key. Force it to uppercase for consistency,
        '       and then stuff it in a byte array.
        secretKey = (New UnicodeEncoding).GetBytes(loginID.ToUpper())

        ' ----- Create the hashing component using Managed SHA-1.
        hashingFunction = New HMACSHA1(secretKey, True)

        ' ----- Calculate the hash value. One simple line of code.
        hashValue = hashingFunction.ComputeHash(
            (New UnicodeEncoding).GetBytes(passwordText))

        ' ----- The hash value is ready, but I like things in plain text when
        '       possible. Let's convert it to a long hex string.
        For counter = 0 To hashValue.Length - 1
            result &= Hex(hashValue(counter))
        Next counter

        ' ----- Stored passwords are limited to 20 characters.
        If (result.Length > 20) Then result = result.Substring(0, 20)
        Return result
    End Function

    Public Sub ExecuteSQL(theAction As DataCommand)
        ' ----- Given a SQL statement with parameters, run it.
        Dim dbCommand As SqlCommand
        Dim localConnection As SqlConnection = Nothing

        ' ----- Connect to the database, if not yet done. Throw exception on failure.
        If (theAction.Connection Is Nothing) Then
            localConnection = ConnectDatabase()
        End If

        ' ----- Try to run the statement. Throw exception on failure.
        Try
            dbCommand = theAction.GetCommandObject()
            If (localConnection IsNot Nothing) Then _
                dbCommand.Connection = localConnection
            dbCommand.ExecuteNonQuery()
        Catch ex As Exception
            ' ----- Reissue error after Finally clause.
            Throw ex
        Finally
            ' ----- Disconnect if needed.
            If (localConnection IsNot Nothing) Then localConnection.Close()
        End Try
    End Sub

    Public Sub ExecuteSQL(theAction As DataCommand,
            transInfo As LibraryTransaction)
        ' ----- Given a SQL statement with parameters, run it.
        Dim dbCommand As SqlCommand

        ' ----- Try to run the statement. Throw exception on failure.
        dbCommand = theAction.GetCommandObject()
        dbCommand.Connection = transInfo.Connection
        dbCommand.Transaction = transInfo.Transaction
        dbCommand.ExecuteNonQuery()
    End Sub

    Public Sub ExecuteSQL(sqlText As String)
        ' ----- Given a SQL statement, run it.
        Dim dbCommand As SqlCommand

        ' ----- Connect to the database. Throw exception on failure.
        Using libraryDB As SqlConnection = ConnectDatabase()
            ' ----- Try to run the statement. Throw exception on failure.
            dbCommand = New SqlCommand(sqlText, libraryDB)
            dbCommand.ExecuteNonQuery()
        End Using
    End Sub

    Public Sub ExecuteSQL(sqlText As String,
            transInfo As LibraryTransaction)
        ' ----- Given a SQL statement, run it.
        Dim dbCommand As SqlCommand

        ' ----- Connect to the database. Throw exception on failure.
        dbCommand = New SqlCommand(sqlText, transInfo.Connection)
        dbCommand.Transaction = transInfo.Transaction
        dbCommand.ExecuteNonQuery()
    End Sub

    Public Function ExecuteSQLReturn(theAction As DataCommand) As Object
        ' ----- Given a SQL statement with parameters, run it, and return a result.
        Dim dbCommand As SqlCommand
        Dim localConnection As SqlConnection = Nothing

        ' ----- Connect to the database, if not yet done. Throw exception on failure.
        If (theAction.Connection Is Nothing) Then
            localConnection = ConnectDatabase()
        End If

        ' ----- Try to run the statement. Throw exception on failure.
        Try
            dbCommand = theAction.GetCommandObject()
            If (localConnection IsNot Nothing) Then _
                dbCommand.Connection = localConnection
            Return dbCommand.ExecuteScalar()
        Catch ex As Exception
            ' ----- Reissue error after Finally clause.
            Throw ex
        Finally
            ' ----- Disconnect if needed.
            If (localConnection IsNot Nothing) Then localConnection.Close()
        End Try
    End Function

    Public Function ExecuteSQLReturn(theAction As DataCommand,
            transInfo As LibraryTransaction) As Object
        ' ----- Given a SQL statement with parameters, run it, and return a result.
        Dim dbCommand As SqlCommand

        ' ----- Try to run the statement. Throw exception on failure.
        dbCommand = theAction.GetCommandObject()
        dbCommand.Connection = transInfo.Connection
        dbCommand.Transaction = transInfo.Transaction
        Return dbCommand.ExecuteScalar()
    End Function

    Public Function ExecuteSQLReturn(sqlText As String) As Object
        ' ----- Given a SQL statement, run it, and return a result.
        Dim dbCommand As SqlCommand

        ' ----- Connect to the database. Throw exception on failure.
        Using libraryDB As SqlConnection = ConnectDatabase()
            ' ----- Try to run the statement. Throw exception on failure.
            dbCommand = New SqlCommand(sqlText, libraryDB)
            Return dbCommand.ExecuteScalar()
        End Using
    End Function

    Public Function ExecuteSQLReturn(sqlText As String,
            transInfo As LibraryTransaction) As Object
        ' ----- Given a SQL statement, run it, and return a result.
        Dim dbCommand As SqlCommand

        ' ----- Connect to the database. Throw exception on failure.
        dbCommand = New SqlCommand(sqlText, transInfo.Connection)
        dbCommand.Transaction = transInfo.Transaction
        Return dbCommand.ExecuteScalar()
    End Function

    Public Sub GeneralError(routineName As String, theError As Exception)
        ' ----- Report an error to the user.
        On Error Resume Next

        MessageBox.Show("The following error occurred at location '" & routineName &
            "':" & vbCrLf & vbCrLf & theError.Message,
            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        My.Application.Log.WriteException(theError)
    End Sub

    Public Function GetSubStr(origString As String, delim As String,
            whichField As Integer) As String
        ' ----- Extracts a delimited string from another larger string.
        Dim stringParts() As String

        ' ----- Handle some errors.
        If (whichField < 0) Then Return ""
        If (origString Is Nothing) OrElse (origString.Length = 0) Then Return ""
        If (delim Is Nothing) OrElse (delim.Length = 0) Then Return ""

        ' ----- Break the string up into delimited parts.
        stringParts = origString.Split({delim}, StringSplitOptions.None)

        ' ----- See if the part we want exists and return it.
        If (whichField > stringParts.Length) Then Return "" _
            Else Return stringParts(whichField - 1)
    End Function

    Public Function GetSystemValue(valueName As String) As String
        ' ----- Return the data portion of a system value name-data pair.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Retrieve the value.
        sqlText = "SELECT ValueData FROM SystemValue " &
            "WHERE UPPER(ValueName) = @TestName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestName", valueName.ToUpper())
        Try
            Return DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("GetSystemValue", ex)
            Return ""
        End Try
    End Function

    Public Sub InitializeSystem()
        ' ----- Initialize global variables here.
        Dim counter As Integer

        ' ----- Clear security-related values.
        LoggedInUserID = -1&
        LoggedInUserName = ""
        LoggedInGroupID = -1&
        For counter = 1 To MaxLibrarySecurity
            SecurityProfile(counter) = False
        Next counter
    End Sub

    Public Function LeftAndRightText(origLeft As String, origRight As String,
            textWidth As Integer) As String
        ' ----- Build a string of the indicated width that has both left-justified
        '       and right-justified text.
        Dim leftText As String
        Dim rightText As String

        leftText = origLeft.Trim()
        rightText = origRight.Trim()
        If ((leftText & rightText).Length >= textWidth) Then
            Return (leftText & rightText).Substring(0, textWidth).Trim()
        Else
            Return leftText & New String(" "c,
                textWidth - (leftText & rightText).Length) & rightText
        End If
    End Function

    Public Sub ReprocessSecuritySet()
        ' ----- Reload in the security set for the current user. If no user
        '       is logged in, clear all settings.
        Dim counter As Integer
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable = Nothing

        ' ----- Clear out the existing items.
        For counter = 1 To MaxLibrarySecurity
            SecurityProfile(counter) = False
        Next counter

        ' ----- Exit if there is no user logged in.
        If (LoggedInUserID = -1&) Or (LoggedInGroupID = -1&) Then Return

        Try
            ' ----- Load in the security elements for this user.
            sqlText = "SELECT ActivityID FROM GroupActivity " &
                "WHERE GroupID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", LoggedInGroupID)
            dbInfo = CreateDataTable(sqlRun)
            For Each dbRow As DataRow In dbInfo.Rows
                SecurityProfile(DBGetLong(dbRow!ActivityID)) = True
            Next dbRow
            dbInfo.Dispose()
        Catch ex As Exception
            ' ----- Some database-related error.
            GeneralError("ReprocessSecuritySet", ex)

            ' ----- Un-login the administrator through recursion.
            LoggedInUserID = -1&
            LoggedInGroupID = -1&
            ReprocessSecuritySet()
        Finally
            If (dbInfo IsNot Nothing) Then dbInfo.Dispose()
            dbInfo = Nothing
        End Try
    End Sub

    Public Sub SetSystemValue(valueName As String, valueData As String)
        ' ----- Update a record in the SystemValue table.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        Try
            ' ----- See if the entry already exists.
            sqlText = "SELECT COUNT(*) FROM SystemValue " &
                "WHERE UPPER(ValueName) = @TestName"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("TestName", valueName.ToUpper())
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                ' ----- Value already exists.
                sqlText = "UPDATE SystemValue SET ValueData = @NewData " &
                    "WHERE UPPER(ValueName) = @TestName"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddText("NewData", valueData)
                sqlRun.AddText("TestName", valueName.ToUpper())
            Else
                ' ----- Need to create value.
                sqlText = "INSERT INTO SystemValue (ValueName, ValueData) " &
                    "VALUES (@NewName, @NewData)"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddText("NewName", valueName)
                sqlRun.AddText("NewData", valueData)
            End If

            ' ----- Update the value.
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("SetSystemValue", ex)
        End Try
    End Sub

    Public Function TransactionBegin() As LibraryTransaction
        ' ----- Create a new database transaction. Throw exception on failure.
        Dim transInfo As LibraryTransaction

        ' ----- Connect to the database.
        transInfo = New LibraryTransaction()
        transInfo.Connection = ConnectDatabase()
        transInfo.Transaction = transInfo.Connection.BeginTransaction()
        Return transInfo
    End Function

    Public Sub TransactionCommit(transInfo As LibraryTransaction)
        ' ----- Commit the transaction.
        If (transInfo Is Nothing) Then Return
        Try
            If (transInfo.Transaction IsNot Nothing) Then _
                transInfo.Transaction.Commit()
        Catch ex As Exception
            ' ----- Reissue error after Finally clause.
            Throw ex
        Finally
			transInfo.Transaction = Nothing
            If (transInfo.Connection IsNot Nothing) Then _
				transInfo.Connection.Close()
            transInfo.Connection = Nothing
        End Try
    End Sub

    Public Sub TransactionRollback(transInfo As LibraryTransaction)
        ' ----- Rollback the transaction.
        If (transInfo Is Nothing) Then Return
        Try
            If (transInfo.Transaction IsNot Nothing) Then _
                transInfo.Transaction.Rollback()
        Catch ex As Exception
            ' ----- Reissue error after Finally clause.
            Throw ex
        Finally
            transInfo.Transaction = Nothing
            If (transInfo.Connection IsNot Nothing) Then _
                transInfo.Connection.Close()
            transInfo.Connection = Nothing
        End Try
    End Sub
End Module

Public Class LibraryTransaction
    Public Connection As SqlConnection
    Public Transaction As SqlTransaction

    Public Sub Commit()
        ' ----- Manage the transaction commit.
        Dim saveError As Exception = Nothing

        ' ----- Commit the transaction.
        Try
            If (Me.Transaction IsNot Nothing) Then Me.Transaction.Commit()
        Catch ex As Exception
            ' ----- Save error for later.
            saveError = ex
        End Try

        ' ----- Close the connection.
        Try
            If (Me.Connection IsNot Nothing) Then Me.Connection.Close()
        Catch
            ' ----- Ignore any error.
        End Try

        ' ----- Return error if it exists.
        If (saveError IsNot Nothing) Then Throw saveError
    End Sub

    Public Sub Rollback()
        ' ----- Manage the transaction rollback.
        Dim saveError As Exception = Nothing

        ' ----- Rollback the transaction.
        Try
            If (Me.Transaction IsNot Nothing) Then Me.Transaction.Rollback()
        Catch ex As Exception
            ' ----- Save error for later.
            saveError = ex
        End Try

        ' ----- Close the connection.
        Try
            If (Me.Connection IsNot Nothing) Then Me.Connection.Close()
        Catch
            ' ----- Ignore any error.
        End Try

        ' ----- Return error if it exists.
        If (saveError IsNot Nothing) Then Throw saveError
    End Sub
End Class

Public Class DataCommand
    Public Enum CommandDataType
        AsBoolean
        AsDate
        AsDateTime
        AsLong
        AsInteger
        AsDecimal
        AsText
    End Enum

    Public Class CommandParameter
        Public ParameterName As String = ""
        Public SourceData As Object
        Public SourceType As CommandDataType
        Public TargetType As CommandDataType
    End Class

    Public Connection As SqlConnection = Nothing
    Public CommandText As String = ""
    Public Parameters As New Generic.Dictionary(Of String, CommandParameter)
    Public IsStoredProcedure As Boolean = False

    Public Sub New()
        ' ----- Default constructor.
        Me.Connection = Nothing
        Me.CommandText = ""
    End Sub

    Public Sub New(sqlText As String)
        ' ----- Command-only constructor.
        Me.Connection = Nothing
        Me.CommandText = sqlText
        If (sqlText Is Nothing) Then Me.CommandText = ""
    End Sub

    Public Sub New(sqlText As String, connectDB As SqlConnection)
        ' ----- Full constructor.
        Me.Connection = connectDB
        Me.CommandText = sqlText
        If (sqlText Is Nothing) Then Me.CommandText = ""
    End Sub

    Public Function GetCommandObject() As SqlCommand
        ' ----- Build a command object that includes all of the appropriate
        '       parameters.
        Dim result As New SqlCommand
        Dim scanParam As CommandParameter
        Dim oneParam As SqlParameter
        Dim holdText As String

        ' ----- Fill in the basics.
        result.Connection = Me.Connection
        result.CommandText = Me.CommandText
        If (Me.IsStoredProcedure = True) Then
            result.CommandType = CommandType.StoredProcedure
        Else
            result.CommandType = CommandType.Text
        End If

        For Each scanParam In Me.Parameters.Values
            ' ----- Build a parameter object for the command.
            oneParam = Nothing
            Select Case scanParam.TargetType
                Case CommandDataType.AsBoolean
                    ' ----- Source will always be Boolean. Convert to Integer.
                    oneParam = New SqlParameter(scanParam.ParameterName,
                        SqlDbType.Bit)
                    If (CBool(scanParam.SourceData) = True) Then
                        oneParam.Value = 1
                    Else
                        oneParam.Value = 0
                    End If
                Case CommandDataType.AsDate, CommandDataType.AsDateTime
                    ' ----- Source will be date or string.
                    oneParam = New SqlParameter(scanParam.ParameterName,
                        SqlDbType.DateTime)
                    If (scanParam.SourceType = CommandDataType.AsDateTime) Or
                            (scanParam.SourceType = CommandDataType.AsDate) Then
                        ' ----- Date source.
                        If (scanParam.TargetType = CommandDataType.AsDate) Then
                            ' ----- Exclude the time.
                            oneParam.Value = CDate(scanParam.SourceData).Date
                        Else
                            ' ----- Include the time.
                            oneParam.Value = CDate(scanParam.SourceData)
                        End If
                    Else
                        ' ----- String source.
                        holdText = CStr(scanParam.SourceData)
                        If (holdText.Trim.Length = 0) Or (IsDate(holdText) = False) Then
                            ' ----- Store NULL.
                            oneParam.Value = System.DBNull.Value
                        Else
                            ' ----- True date.
                            If (scanParam.TargetType = CommandDataType.AsDate) Then
                                ' ----- Exclude the time.
                                oneParam.Value = CDate(holdText).Date
                            Else
                                ' ----- Include the time.
                                oneParam.Value = CDate(holdText)
                            End If
                        End If
                    End If
                Case CommandDataType.AsLong
                    ' ----- String or true Long value.
                    oneParam = New SqlParameter(scanParam.ParameterName,
                        SqlDbType.BigInt)
                    If (scanParam.SourceType = CommandDataType.AsText) Then
                        holdText = CStr(scanParam.SourceData).Trim
                        If (holdText.Length = 0) Then
                            oneParam.Value = System.DBNull.Value
                        Else
                            oneParam.Value = CLng(holdText)
                        End If
                    Else
                        oneParam.Value = CLng(scanParam.SourceData)
                    End If
                Case CommandDataType.AsInteger
                    ' ----- String or true Integer value.
                    oneParam = New SqlParameter(scanParam.ParameterName,
                        SqlDbType.Int)
                    If (scanParam.SourceType = CommandDataType.AsText) Then
                        holdText = CStr(scanParam.SourceData).Trim
                        If (holdText.Length = 0) Then
                            oneParam.Value = System.DBNull.Value
                        Else
                            oneParam.Value = CInt(holdText)
                        End If
                    Else
                        oneParam.Value = CInt(scanParam.SourceData)
                    End If
                Case CommandDataType.AsDecimal
                    ' ----- String or true Decimal value.
                    oneParam = New SqlParameter(scanParam.ParameterName,
                        SqlDbType.Decimal)
                    If (scanParam.SourceType = CommandDataType.AsText) Then
                        holdText = CStr(scanParam.SourceData).Trim
                        If (holdText.Length = 0) Then
                            oneParam.Value = System.DBNull.Value
                        Else
                            oneParam.Value = CDec(holdText)
                        End If
                    Else
                        oneParam.Value = CDec(scanParam.SourceData)
                    End If
                Case CommandDataType.AsText
                    ' ----- Char, Varchar, Text, etc.
                    oneParam = New SqlParameter(scanParam.ParameterName,
                        SqlDbType.VarChar)
                    holdText = CStr(scanParam.SourceData)
                    If (holdText Is Nothing) OrElse (holdText.Trim.Length = 0) Then
                        oneParam.Value = System.DBNull.Value
                    Else
                        oneParam.Value = holdText
                    End If
            End Select

            ' ----- Store the parameter in the command.
            If (oneParam IsNot Nothing) Then result.Parameters.Add(oneParam)
        Next scanParam

        ' ----- Finished.
        Return result
    End Function

    Public Sub AddBool(paramName As String, content As Boolean)
        ' ----- Add a new Boolean parameter.
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsBoolean
        newEntry.TargetType = CommandDataType.AsBoolean
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddDate(paramName As String, Optional content As String = "")
        ' ----- Add a new date parameter. Stores NULL when content is "".
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsText
        newEntry.TargetType = CommandDataType.AsDate
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddDate(paramName As String, content As Date)
        ' ----- Add a new date parameter.
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsDate
        newEntry.TargetType = CommandDataType.AsDate
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddInteger(paramName As String, Optional content As String = "")
        ' ----- Add a new integer parameter. Stores NULL when content is "".
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsText
        newEntry.TargetType = CommandDataType.AsInteger
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddInteger(paramName As String, content As Integer)
        ' ----- Add a new integer parameter.
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsInteger
        newEntry.TargetType = CommandDataType.AsInteger
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddID(paramName As String, content As Long)
        ' ----- Same as AddLong, but a value of -1 is treated as NULL.
        If (content = -1&) Then
            Me.AddLong(paramName)
        Else
            Me.AddLong(paramName, content)
        End If
    End Sub

    Public Sub AddLong(paramName As String, Optional content As String = "")
        ' ----- Add a new long integer parameter. Stores NULL when content is "".
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsText
        newEntry.TargetType = CommandDataType.AsLong
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddLong(paramName As String, content As Long)
        ' ----- Add a new long integer parameter.
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsLong
        newEntry.TargetType = CommandDataType.AsLong
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddDecimal(paramName As String, Optional content As String = "")
        ' ----- Add a new long integer parameter. Stores NULL when content is "".
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsText
        newEntry.TargetType = CommandDataType.AsDecimal
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddDecimal(paramName As String, content As Decimal)
        ' ----- Add a new long integer parameter.
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsDecimal
        newEntry.TargetType = CommandDataType.AsDecimal
        Me.Parameters.Add(paramName, newEntry)
    End Sub

    Public Sub AddText(paramName As String, Optional content As String = "")
        ' ----- Add a new string parameter. Stores NULL when content is "".
        Dim newEntry As New CommandParameter()

        newEntry.ParameterName = paramName
        newEntry.SourceData = content
        newEntry.SourceType = CommandDataType.AsText
        newEntry.TargetType = CommandDataType.AsText
        Me.Parameters.Add(paramName, newEntry)
    End Sub
End Class
