﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class ItemCopy
    Private ActiveItem As Long
    Private ActiveCopy As Long

    ' ***** Insert Code Snippet #5 here.

    Public Function AddCopy(itemID As Long, useItemName As String) As Long
        ' ----- A default location must be defined.
        If (ConfirmDefaultLocation() = False) Then Return -1&

        ' ----- Prompt the user to add a new item copy.
        ActiveItem = itemID
        ActiveCopy = -1&
        ItemName.Text = useItemName
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveCopy Else Return -1&
    End Function

    Public Function EditCopy(itemID As Long, copyID As Long,
            useItemName As String) As Long
        ' ----- A default location must be defined.
        If (ConfirmDefaultLocation() = False) Then Return -1&

        ' ----- Prompt the user to edit an existing item copy.
        ActiveItem = itemID
        ActiveCopy = copyID
        ItemName.Text = useItemName
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveCopy Else Return -1&
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save the record to the database.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long

        Me.Cursor = Cursors.WaitCursor

        ' ----- Different code for new and existing copies.
        If (ActiveCopy = -1&) Then
            ' ----- Add a new record.
            sqlText = "INSERT INTO ItemCopy (ItemID, CopyNumber, Description, " &
                "Barcode, Available, Reference, Missing, Acquired, Cost, Status, " &
                "Condition, Location) OUTPUT INSERTED.ID VALUES (@NewItem, @NewCopy, " &
                "@NewDesc, @NewBarcode, @NewAvailable, @NewReference, @NewMissing, " &
                "@NewAcquired, @NewCost, @NewStatus, @NewCondition, @NewLocation)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("NewItem", ActiveItem)
            sqlRun.AddText("NewCopy", RecordCopy.Text)
            sqlRun.AddText("NewDesc", RecordDescription.Text.Trim())
            sqlRun.AddText("NewBarcode", RecordBarcode.Text.Trim())
            sqlRun.AddBool("NewAvailable", RecordAvailable.Checked)
            sqlRun.AddBool("NewReference", RecordReference.Checked)
            sqlRun.AddBool("NewMissing", RecordMissing.Checked)
            sqlRun.AddDate("NewAcquired", RecordAcquired.Text)
            sqlRun.AddDecimal("NewCost", RecordCost.Text)
            sqlRun.AddLong("NewStatus", DBComboParam(RecordGeneralStatus))
            sqlRun.AddText("NewCondition", RecordCondition.Text.Trim())
            sqlRun.AddLong("NewLocation", DBComboParam(RecordLocation))
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                GeneralError("ItemCopy.SaveFormData.Add", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveCopy = newID
        Else
            ' ----- Update an existing record.
            sqlText = "UPDATE ItemCopy SET CopyNumber = @NewCopy, " &
                "Description = @NewDesc, Barcode = @NewBarcode, " &
                "Available = @NewAvailable, Reference = @NewReference, " &
                "Missing = @NewMissing, Acquired = @NewAcquired, " &
                "Cost = @NewCost, Status = @NewStatus, " &
                "Condition = @NewCondition, Location = @NewLocation " &
                "WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewCopy", RecordCopy.Text)
            sqlRun.AddText("NewDesc", RecordDescription.Text.Trim())
            sqlRun.AddText("NewBarcode", RecordBarcode.Text.Trim())
            sqlRun.AddBool("NewAvailable", RecordAvailable.Checked)
            sqlRun.AddBool("NewReference", RecordReference.Checked)
            sqlRun.AddBool("NewMissing", RecordMissing.Checked)
            sqlRun.AddDate("NewAcquired", RecordAcquired.Text)
            sqlRun.AddDecimal("NewCost", RecordCost.Text)
            sqlRun.AddLong("NewStatus", DBComboParam(RecordGeneralStatus))
            sqlRun.AddText("NewCondition", RecordCondition.Text.Trim())
            sqlRun.AddLong("NewLocation", DBComboParam(RecordLocation))
            sqlRun.AddLong("RecordID", ActiveCopy)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("ItemCopy.SaveFormData.Update", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Confirm the fields on the form before saving.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Copy number is required, and it must be numeric.
        If (RecordCopy.Text.Trim().Length = 0) Then
            MessageBox.Show("Copy Number is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordCopy.Focus()
            Return False
        End If
        If (DigitsOnly(RecordCopy.Text) <> RecordCopy.Text.Trim()) Then
            MessageBox.Show("Copy Number must be numeric.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordCopy.Focus()
            Return False
        End If

        ' ----- Make sure the copy number is unique.
        sqlText = "SELECT COUNT(*) FROM ItemCopy " &
            "WHERE CopyNumber = @WhichCopy AND ItemID = @WhichItem"
        If (ActiveCopy <> -1&) Then sqlText &= " AND ID <> @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddInteger("WhichCopy", RecordCopy.Text)
        sqlRun.AddLong("WhichItem", ActiveItem)
        If (ActiveCopy <> -1&) Then _
            sqlRun.AddLong("RecordID", ActiveCopy)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                ' ----- Duplicate entry.
                MessageBox.Show("A record already exists with the copy number '" &
                    RecordCopy.Text.Trim() & "'.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordCopy.Focus()
                Return False
            End If
        Catch ex As Exception
            GeneralError("ItemCopy.ValidateFormData.Copy", ex)
            Return False
        End Try

        ' ----- Barcode, if supplied, must be numeric.
        If (RecordBarcode.Text.Trim().Length <> 0) Then
            If (DigitsOnly(RecordBarcode.Text) <> RecordBarcode.Text.Trim()) Then
                MessageBox.Show("Barcode, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordBarcode.Focus()
                Return False
            End If
        End If

        ' ----- Make sure the barcode is not already in use.
        If (RecordBarcode.Text.Trim().Length <> 0) Then
            sqlText = "SELECT COUNT(*) FROM ItemCopy " &
                "WHERE UPPER(Barcode) = @TestBarcode"
            If (ActiveCopy <> -1&) Then sqlText &= " AND ID <> @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("TestBarcode", RecordBarcode.Text.Trim().ToUpper())
            sqlRun.AddLong("RecordID", ActiveCopy)
            Try
                If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                    ' ----- Duplicate entry.
                    MessageBox.Show("A record already exists with the barcode'" &
                        RecordBarcode.Text.Trim() & "'.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    RecordBarcode.Focus()
                    Return False
                End If
            Catch ex As Exception
                GeneralError("ItemCopy.ValidateFormData.Barcode", ex)
                Return False
            End Try
        End If

        ' ----- Date acquired, if supplied, must be a valid date.
        If (RecordAcquired.Text.Trim().Length <> 0) Then
            If (IsDate(RecordAcquired.Text) = False) Then
                MessageBox.Show("Date Acquired, if supplied, must be a valid date.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordAcquired.Focus()
                Return False
            End If
        End If

        ' ----- Item cost, if supplied, must be numeric.
        If (RecordCost.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordCost.Text) = False) Then
                MessageBox.Show("Item Cost, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordCost.Focus()
                Return False
            End If
        End If

        ' ----- General status is required.
        If (CLng(DirectCast(RecordGeneralStatus.SelectedItem, ListItemData)) = -1&) Then
            MessageBox.Show("General Status is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordGeneralStatus.Focus()
            Return False
        End If

        ' ----- Location is required.
        If (CLng(DirectCast(RecordLocation.SelectedItem, ListItemData)) = -1&) Then
            MessageBox.Show("Location is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordLocation.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save the record.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub ItemCopy_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow

        ' ----- Load in the list of status codes.
        RecordGeneralStatus.Items.Add(New ListItemData("<Not Selected>", -1&))
        RecordGeneralStatus.SelectedIndex = 0
        sqlText = "SELECT * FROM CodeCopyStatus ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("ItemCopy.ItemCopy_Load.Status", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            RecordGeneralStatus.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Load in the list of locations.
        RecordLocation.Items.Add(New ListItemData("<Not Selected>", -1&))
        RecordLocation.SelectedIndex = 0
        sqlText = "SELECT * FROM CodeLocation ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("ItemCopy.ItemCopy_Load.Location", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            RecordLocation.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
            If (DBGetLong(dbRow!ID) = DefaultItemLocation) Then _
                RecordLocation.SelectedIndex = RecordLocation.Items.Count - 1
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- For new items, that's it.
        If (ActiveCopy = -1&) Then Return

        ' ----- Load in the existing item.
        sqlText = "SELECT * FROM ItemCopy WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveCopy)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("ItemCopy.ItemCopy_Load.Item", ex)
            Return
        End Try
        oneRow = dbInfo.Rows(0)
        RecordCopy.Text = DBGetText(oneRow!CopyNumber)
        RecordDescription.Text = DBGetText(oneRow!Description)
        RecordBarcode.Text = DBGetText(oneRow!Barcode)
        RecordAvailable.Checked = DBGetBool(oneRow!Available)
        RecordReference.Checked = DBGetBool(oneRow!Reference)
        RecordMissing.Checked = DBGetBool(oneRow!Missing)
        If (IsDBNull(oneRow!Acquired) = False) Then _
            RecordAcquired.Text = DBGetDate(oneRow!Acquired).ToString("M/d/yyyy")
        If (IsDBNull(oneRow!Cost) = False) Then _
            RecordCost.Text = DBGetDecimal(oneRow!Cost).ToString("0.00")
        RecordGeneralStatus.SelectedIndex =
            RecordGeneralStatus.Items.IndexOf(CLng(DBGetInteger(oneRow!Status)))
        If (RecordGeneralStatus.SelectedIndex = -1) Then _
            RecordGeneralStatus.SelectedIndex =
            RecordGeneralStatus.Items.IndexOf(-1&)
        RecordCondition.Text = DBGetText(oneRow!Condition)
        If (IsDBNull(oneRow!Location) = False) Then _
            RecordLocation.SelectedIndex =
            RecordLocation.Items.IndexOf(DBGetLong(oneRow!Location))
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Display the disposition of the item.
        RecordStatus.Text = GetCopyDisposition(ActiveCopy)
    End Sub

    Private Sub RecordCopy_Enter(sender As Object, e As EventArgs) Handles RecordCopy.Enter
        ' ----- Highlight the entire text.
        RecordCopy.SelectAll()
    End Sub

    Private Sub RecordAcquired_Enter(sender As Object, e As EventArgs) Handles RecordAcquired.Enter
        ' ----- Highlight the entire text.
        RecordAcquired.SelectAll()
    End Sub

    Private Sub RecordCost_Enter(sender As Object, e As EventArgs) Handles RecordCost.Enter
        ' ----- Highlight the entire text.
        RecordCost.SelectAll()
    End Sub

    Private Sub RecordCondition_Enter(sender As Object, e As EventArgs) Handles RecordCondition.Enter
        ' ----- Highlight the entire text.
        RecordCondition.SelectAll()
    End Sub

    Private Sub StatusDetails_Click(sender As Object, e As EventArgs) Handles StatusDetails.Click
        ' ----- Show details about the disposition of the item.
        Dim sqlText As String
        Dim dbInfo As DataTable
        Dim sqlRun As DataCommand
        Dim response As String
        Dim amountDue As Decimal
        Dim dbRow As DataRow

        ' ----- Do a quick check for new items.
        If (ActiveCopy = -1&) Then
            MessageBox.Show("This is a new item.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
            Return
        End If
        Me.Cursor = Cursors.WaitCursor

        ' ----- Retrieve information about the book.
        sqlText = "SELECT IC.Reference, IC.Missing, PC.Missing AS PatronMissing, " &
            "PC.DueDate, PC.Patron, PT.LastName, PT.FirstName, PC.Fine, PC.Paid, " &
            "PC.CheckOut FROM ItemCopy AS IC LEFT JOIN PatronCopy AS PC " &
            "ON IC.ID = PC.ItemCopy LEFT JOIN Patron AS PT " &
            "ON PC.Patron = PT.ID WHERE IC.ID = @RecordID " &
            "AND PC.Returned = 0"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveCopy)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("ItemCopy.StatusDetails_Click.Retrieve", ex)
            Return
        End Try
        If (dbInfo.Rows.Count = 0) Then
            ' ----- Perhaps the database, due to the JOIN clause, could not return
            '       the base record of a checked-in book.
            dbInfo.Dispose()
            sqlText = "SELECT Reference, Missing, NULL AS PatronMissing, " &
                "NULL AS DueDate, NULL AS Patron, NULL AS LastName, " &
                "NULL AS FirstName, NULL AS Fine, NULL AS Paid, " &
                "NULL AS CheckOut FROM ItemCopy WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", ActiveCopy)
            Try
                dbInfo = CreateDataTable(sqlRun)
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                GeneralError("ItemCopy.StatusDetails_Click.Retrieve2", ex)
                Return
            End Try
            If (dbInfo.Rows.Count = 0) Then
                ' ----- What happened to the record?
                response = "An error occurred while trying to determine the " &
                    "disposition of the item."
                GoTo ShowResponse
            End If
        End If
        dbRow = dbInfo.Rows(0)

        ' ----- We found the record. Show its details.
        If (IsDBNull(dbRow!Patron) = False) Then
            ' ----- This item is checked out.
            If (DBGetBool(dbRow!Missing) = True) Or (DBGetBool(dbRow!PatronMissing) = True) Then
                response = "This item was reported missing by patron "
            ElseIf (DBGetDate(dbRow!DueDate) < Today) Then
                response = "This item is overdue. It was last due on " &
                    DBGetDate(dbRow!DueDate).ToString("MMMM d, yyyy") & " by patron "
            Else
                response = "This item is currently checked out. It is due on " &
                    DBGetDate(dbRow!DueDate).ToString("MMMM d, yyyy") & " by patron "
            End If
            response &= DBGetText(dbRow!FirstName) & " " & DBGetText(dbRow!LastName) &
                ". It was originally checked out on " &
                DBGetDate(dbRow!CheckOut).ToString("MMMM d, yyyy") & "."
            amountDue = 0@
            If (IsDBNull(dbRow!Fine) = False) Then amountDue += DBGetDecimal(dbRow!Fine)
            If (IsDBNull(dbRow!Paid) = False) Then amountDue -= DBGetDecimal(dbRow!Paid)
            If (amountDue > 0@) Then response &= " The patron owes " &
                amountDue.ToString("$0.00") & " in fines on this item."
        Else
            ' ----- This book is currently checked in.
            If (DBGetBool(dbRow!Missing) = True) Then
                response = "This item has been reported as missing."
            ElseIf (DBGetBool(dbRow!Reference) = True) Then
                response = "This item is in the reference collection."
            Else
                response = "This item is currently checked in."
            End If
        End If
        dbInfo.Dispose()
        dbInfo = Nothing

ShowResponse:
        ' ----- Display the disposition.
        Me.Cursor = Cursors.Default
        MessageBox.Show(response,
            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub
End Class