﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class UserName
    Private ActiveID As Long
    Private StartingOver As Boolean
    Private OrigLoginID As String

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageUsers)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageUsers)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageUsers)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the record.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim transInfo As LibraryTransaction = Nothing

        ' ----- Do not delete the current user.
        If (LoggedInUserID = recordID) Then
            MessageBox.Show("You cannot delete the active user record.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the selected user record?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Delete the record.
        Try
            transInfo = TransactionBegin()

            sqlText = "UPDATE PatronPayment SET UserID = NULL WHERE UserID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            sqlText = "DELETE FROM UserName WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            GeneralError("UserName.DeleteRecord", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As DataTable
        Dim itemText As String

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, FullName, Active FROM UserName ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("UserName.FillListWithRecords", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            itemText = DBGetText(dbRow!FullName)
            If (DBGetBool(dbRow!Active) = False) Then itemText &= " (Inactive)"
            destList.Items.Add(New ListItemData(itemText, DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim itemText As String

        ' ----- Initialize.
        itemText = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName, Active FROM UserName WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("UserName.FormatRecordName", ex)
            Return "Error"
        End Try
        If (dbInfo.Rows.Count > 0) Then
            oneRow = dbInfo.Rows(0)
            itemText = DBGetText(oneRow!FullName)
            If (DBGetBool(oneRow!Active) = False) Then _
                itemText &= " (Inactive)"
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
        Return itemText
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to identify the administrative uses who have " &
            "access to the system."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Administrative Users"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow

        ' ----- Load in the list of security groups.
        If (RecordGroup.Items.Count = 0) Then
            ' ----- This is the first time. Load from the database.
            RecordGroup.Items.Clear()
            RecordGroup.Items.Add(New ListItemData("<Not Selected>", -1&))
            sqlText = "SELECT * FROM GroupName ORDER BY FullName"
            Try
                dbInfo = CreateDataTable(sqlText)
            Catch ex As Exception
                GeneralError("UserName.PrepareFormFields.Groups", ex)
                Return
            End Try

            For Each dbRow As DataRow In dbInfo.Rows
                RecordGroup.Items.Add(New ListItemData(
                    DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
            Next dbRow
            dbInfo.Dispose()
            dbInfo = Nothing
        End If

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        RecordLoginID.Text = ""
        RecordPassword.Text = ""
        RecordConfirm.Text = ""
        RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1&)
        RecordActive.Checked = True
        OrigLoginID = ""
        If (ActiveID = -1&) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM UserName WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("UserName.PrepareFormFields.Retrieve", ex)
            Return
        End Try
        If (dbInfo.Rows.Count > 0) Then
            ' ----- We never reload the password.
            oneRow = dbInfo.Rows(0)
            RecordFullName.Text = DBGetText(oneRow!FullName)
            RecordLoginID.Text = DBGetText(oneRow!LoginID)
            OrigLoginID = DBGetText(oneRow!LoginID)
            RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(DBGetLong(oneRow!GroupID))
            RecordActive.Checked = DBGetBool(oneRow!Active)
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- The user wants to save changes. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim passwordResult As String = ""
        Dim savePassword As Boolean
        Dim newID As Long = -1&

        ' ----- Prepare to save the data.
        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Encrypt the password text.
            passwordResult = EncryptPassword(RecordLoginID.Text.Trim(),
                RecordPassword.Text.Trim())

            ' ----- Create a new entry.
            sqlText = "INSERT INTO UserName(FullName, LoginID, [Password], " &
                "Active, GroupID) OUTPUT INSERTED.ID VALUES (@NewName, " &
                "@NewLogin, @NewPassword, @NewActive, @NewGroup)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            sqlRun.AddText("NewLogin", RecordLoginID.Text.Trim())
            sqlRun.AddText("NewPassword", passwordResult)
            sqlRun.AddBool("NewActive", RecordActive.Checked)
            sqlRun.AddLong("NewGroup", DBComboParam(RecordGroup))
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                GeneralError("UserName.SaveFormData.Add", ex)
                Return False
            End Try
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE UserName SET FullName = @NewName" &
                ", LoginID = @NewLogin, Active = @NewActive" &
                ", GroupID = @NewGroup"

            ' ----- See if the password should be saved.
            savePassword = False
            If (OrigLoginID.Length = 0) Then savePassword = True
            If (OrigLoginID.Trim().ToUpper() <> RecordLoginID.Text.Trim().ToUpper()) Then _
                savePassword = True
            If (RecordPassword.Text.Trim().Length <> 0) Then savePassword = True
            If (savePassword = True) Then
                If (RecordPassword.Text.Trim().ToUpper() = "NP") Then
                    ' ----- NP removes a password.
                    passwordResult = EncryptPassword(RecordLoginID.Text.Trim(), "")
                Else
                    passwordResult = EncryptPassword(RecordLoginID.Text.Trim(),
                        RecordPassword.Text.Trim())
                End If
                sqlText &= ", [Password] = @NewPassword"
            End If
            sqlText &= " WHERE ID = @RecordID"

            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            sqlRun.AddText("NewLogin", RecordLoginID.Text.Trim())
            sqlRun.AddBool("NewActive", RecordActive.Checked)
            sqlRun.AddLong("NewGroup", DBComboParam(RecordGroup))
            If (savePassword = True) Then _
                sqlRun.AddText("NewPassword", passwordResult)
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                GeneralError("UserName.SaveFormData.Update", ex)
                Return False
            End Try
        End If

        ' ----- This change may affect this user.
        If (LoggedInGroupID = ActiveID) Then ReprocessSecuritySet()

        ' ----- Success.
        Me.Cursor = Cursors.Default
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Name is a required field.
        If (RecordFullName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the User Name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Login ID is a required field.
        If (RecordLoginID.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the Login ID.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordLoginID.Focus()
            Return False
        End If

        ' ----- Password and confirm must match.
        If (RecordPassword.Text <> RecordConfirm.Text) Then
            MessageBox.Show("The Password and Confirm fields must match.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordPassword.Focus()
            Return False
        End If

        ' ----- If the login ID changed and there is no password, warn the user that
        '       the new login ID will have no password.
        If (OrigLoginID.Trim().ToUpper() <> RecordLoginID.Text.Trim().ToUpper()) And
                (OrigLoginID.Length <> 0) And (RecordPassword.Text.Trim().Length = 0) Then
            If (MessageBox.Show("When changing the Login ID, the password must also be " &
                "supplied. If you leave the Password field blank, no password will be " &
                "assigned to this user.  Do you wish to continue with a blank password?",
                ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
                DialogResult.Yes) Then Return False
        End If

        ' ----- Make sure the login ID is not already in use.
        sqlText = "SELECT COUNT(*) FROM UserName WHERE UPPER(LoginID) = @TestLogin"
        If (ActiveID <> -1&) Then sqlText &= " AND ID <> @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestLogin", RecordLoginID.Text.Trim().ToUpper())
        If (ActiveID <> -1&) Then _
            sqlRun.AddLong("RecordID", ActiveID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("A record already exists with the Login ID '" &
                    RecordLoginID.Text.Trim() & "'.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordLoginID.Focus()
                Return False
            End If
        Catch ex As Exception
            GeneralError("UserName.ValidateFormData", ex)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- Save changes and return to the summary form.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub RecordConfirm_Enter(sender As Object, e As EventArgs) Handles RecordConfirm.Enter
        ' ----- Highlight the entire text.
        RecordConfirm.SelectAll()
    End Sub

    Private Sub RecordFullName_Enter(sender As Object, e As EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub RecordLoginID_Enter(sender As Object, e As EventArgs) Handles RecordLoginID.Enter
        ' ----- Highlight the entire text.
        RecordLoginID.SelectAll()
    End Sub

    Private Sub RecordPassword_Enter(sender As Object, e As EventArgs) Handles RecordPassword.Enter
        ' ----- Highlight the entire text.
        RecordPassword.SelectAll()
    End Sub

    Private Sub UserName_Activated(sender As Object, e As EventArgs) Handles Me.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub
End Class