﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Imports System.Drawing.Printing

Public Class Maintenance
    ' ----- Store the connection string off-screen.
    Private LibraryConnection As String

    Private Const ClassifyDewey As Long = 1
    Private Const ClassifyLC As Long = 2

    Private Const DefaultReceiptPrinterWidth As Integer = 40

    Private Sub Maintenance_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim allFonts As New Text.InstalledFontCollection
        Dim counter As Integer
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Fill in the list of available system fonts.
        RecordFontName.Items.Add(New ListItemData("<Not Selected>", -1&))
        For counter = 0 To allFonts.Families.Length - 1
            RecordFontName.Items.Add(New ListItemData(
                allFonts.Families(counter).Name, counter))
        Next counter

        ' ----- Fill in the list of locations.
        RecordDefaultLocation.Items.Add(New ListItemData("<Not Selected>", -1&))
        sqlText = "SELECT * FROM CodeLocation ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("Maintenance.Maintenance_Load", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            RecordDefaultLocation.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Fill in the list of classifications.
        RecordClassification.Items.Add(New ListItemData("Dewey Decimal", ClassifyDewey))
        RecordClassification.Items.Add(New ListItemData("Library of Congress", ClassifyLC))

        ' ----- Fill in the list of local printers.
        For Each installedPrinter As String In PrinterSettings.InstalledPrinters
            RecordPrinterLocation.Items.Add(installedPrinter)
        Next installedPrinter

        ' ----- Fill in the current values.
        PopulateCurrentValues()
    End Sub

    Private Function GetDBDisplayText(connectionString As String) As String
        ' ----- Given a connection string, return just an informational portion
        '       for display purposes.
        Dim displayText As String = ""
        Dim counter As Integer
        Dim oneKey As String
        Dim oneValue As String

        For counter = 1 To CountSubStr(connectionString, ";") + 1
            ' ----- Each comma-delimited part has the format "key=value".
            oneKey = connectionString.GetSubStr(";", counter)
            oneValue = oneKey.GetSubStr("=", 2).Trim()
            oneKey = oneKey.GetSubStr("=", 1).Trim().ToUpper().Replace(" ", "")

            If (oneKey = "DATASOURCE") Then
                If (displayText.Length <> 0) Then displayText = "\" & displayText
                displayText = oneValue & displayText
            ElseIf (oneKey = "INITIALCATALOG") Then
                If (displayText.Length <> 0) Then displayText &= "\"
                displayText &= oneValue
            End If
        Next counter

        ' ----- Check for no configuration string.
        If (displayText.Length = 0) Or (displayText = "\") Then displayText = "Not Specified"
        Return displayText
    End Function

    Private Sub PopulateCurrentValues()
        ' ----- Prepare the form.
        Dim holdText As String

        ' ===== System-wide fields.

        ' ----- Fill in the current default location.
        holdText = GetSystemValue("DefaultLocation")
        RecordDefaultLocation.SelectedIndex =
            RecordDefaultLocation.Items.IndexOf(CLng(Val(holdText)))
        If (RecordDefaultLocation.SelectedIndex = -1) Then _
            RecordDefaultLocation.SelectedIndex = RecordDefaultLocation.Items.IndexOf(-1&)

        RecordGraceDays.Text = GetSystemValue("FineGrace")
        RecordSearchLimit.Text = GetSystemValue("SearchLimit")

        ' ----- Fill in the classification type.
        If (CBool(Val(GetSystemValue("UseLC"))) = True) Then
            RecordClassification.SelectedIndex =
                RecordClassification.Items.IndexOf(ClassifyLC)
        Else
            RecordClassification.SelectedIndex =
                RecordClassification.Items.IndexOf(ClassifyDewey)
        End If

        RecordSelfCheckout.Checked = CBool(Val(GetSystemValue("PatronCheckOut")))
        RecordBarcodeItem.Text = GetSystemValue("NextBarcodeItem")
        RecordBarcodePatron.Text = GetSystemValue("NextBarcodePatron")
        RecordBarcodeMisc.Text = GetSystemValue("NextBarcodeMisc")

        ' ----- Fill in the current font name.
        holdText = GetSystemValue("BarcodeFont")
        RecordFontName.SelectedIndex = RecordFontName.FindStringExact(holdText)
        If (RecordFontName.SelectedIndex = -1) Then _
            RecordFontName.SelectedIndex = RecordFontName.Items.IndexOf(-1&)

        RecordBarcode39.Checked = CBool(Val(GetSystemValue("BarcodeCode39")))
        RecordTicketTop.Text = GetSystemValue("TicketHeading")
        RecordTicketBottom.Text = GetSystemValue("TicketFooting")

        ' ===== Workstation-specific fields.
        LibraryConnection = My.Settings.DBConnection & ""
        RecordDBLocation.Text = GetDBDisplayText(LibraryConnection)
        RecordConfigLocation.Text = My.Settings.ReportConfig & ""
        RecordBasicHelp.Text = My.Settings.HelpFile & ""
        RecordAdminHelp.Text = My.Settings.HelpFileAdmin & ""
        EnableReceipts.Checked = My.Settings.UseReceipts
        RecordPrinterLocation.Text = My.Settings.ReceiptPrinter & ""
        RecordPrinterWidth.Text = My.Settings.ReceiptWidth.ToString()
        RecordPostlude.Text = My.Settings.ReceiptPostlude & ""
        HideLogin.Checked = My.Settings.HideLogin

    End Sub

    Private Sub ActAllValues_Click(sender As Object, e As EventArgs) Handles ActAllValues.Click
        ' ----- Let the user edit the list of system values.
        Dim RecordsForm As Library.ListEditRecords

        ' ----- Edit the records.
        RecordsForm = New Library.ListEditRecords
        RecordsForm.ManageRecords(New Library.SystemValue)
        RecordsForm = Nothing

        ' ----- Refresh the display elements.
        PopulateCurrentValues()
    End Sub

    Private Sub RecordGraceDays_Enter(sender As Object, e As EventArgs) Handles RecordGraceDays.Enter
        ' ----- Highlight the entire text.
        RecordGraceDays.SelectAll()
    End Sub

    Private Sub RecordSearchLimit_Enter(sender As Object, e As EventArgs) Handles RecordSearchLimit.Enter
        ' ----- Highlight the entire text.
        RecordSearchLimit.SelectAll()
    End Sub

    Private Sub RecordBarcodeItem_Enter(sender As Object, e As EventArgs) Handles RecordBarcodeItem.Enter
        ' ----- Highlight the entire text.
        RecordBarcodeItem.SelectAll()
    End Sub

    Private Sub RecordBarcodePatron_Enter(sender As Object, e As EventArgs) Handles RecordBarcodePatron.Enter
        ' ----- Highlight the entire text.
        RecordBarcodePatron.SelectAll()
    End Sub

    Private Sub RecordBarcodeMisc_Enter(sender As Object, e As EventArgs) Handles RecordBarcodeMisc.Enter
        ' ----- Highlight the entire text.
        RecordBarcodeMisc.SelectAll()
    End Sub

    Private Sub RecordTicketTop_Enter(sender As Object, e As EventArgs) Handles RecordTicketTop.Enter
        ' ----- Highlight the entire text.
        RecordTicketTop.SelectAll()
    End Sub

    Private Sub RecordTicketBottom_Enter(sender As Object, e As EventArgs) Handles RecordTicketBottom.Enter
        ' ----- Highlight the entire text.
        RecordTicketBottom.SelectAll()
    End Sub

    Private Sub RecordConfigLocation_Enter(sender As Object, e As EventArgs) Handles RecordConfigLocation.Enter
        ' ----- Highlight the entire text.
        RecordConfigLocation.SelectAll()
    End Sub

    Private Sub RecordBasicHelp_Enter(sender As Object, e As EventArgs) Handles RecordBasicHelp.Enter
        ' ----- Highlight the entire text.
        RecordBasicHelp.SelectAll()
    End Sub

    Private Sub RecordAdminHelp_Enter(sender As Object, e As EventArgs) Handles RecordAdminHelp.Enter
        ' ----- Highlight the entire text.
        RecordAdminHelp.SelectAll()
    End Sub

    Private Sub RecordPrinterLocation_Enter(sender As Object, e As EventArgs) Handles RecordPrinterLocation.Enter
        ' ----- Highlight the entire text.
        RecordPrinterLocation.SelectAll()
    End Sub

    Private Sub RecordPrinterWidth_Enter(sender As Object, e As EventArgs) Handles RecordPrinterWidth.Enter
        ' ----- Highlight the entire text.
        RecordPrinterWidth.SelectAll()
    End Sub

    Private Sub RecordPostlude_Enter(sender As Object, e As EventArgs) Handles RecordPostlude.Enter
        ' ----- Highlight the entire text.
        RecordPostlude.SelectAll()
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- Check the user-supplied data.
        If (ValidateFormData() = False) Then Return

        ' ----- Save the updated values.
        If (SaveFormData() = False) Then Return

        ' ----- Update some internal values immediately.
        MainHelpFile = RecordBasicHelp.Text
        MainAdminHelpFile = RecordAdminHelp.Text
        DefaultItemLocation = CLng(DirectCast(RecordDefaultLocation.SelectedItem,
            ListItemData))
        FineGraceDays = CInt(Val(RecordGraceDays.Text))

        ' ----- Close the form.
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Function ValidateFormData() As Boolean
        ' ----- Check the selections. Although the program will not run very
        '       well if any of these values are missing, there are other
        '       places in the program that impose the limits.

        ' ----- If grace days is supplied, it must be numeric.
        If (RecordGraceDays.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordGraceDays.Text) = False) Then
                MessageBox.Show("Fine Grace Days, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
                TabMaintenance.SelectedTab = TabSystem
                RecordGraceDays.Focus()
                Return False
            End If
        End If

        ' ----- If search limit is supplied, it must be numeric.
        If (RecordSearchLimit.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordSearchLimit.Text) = False) Then
                MessageBox.Show("Search Result Limit, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
                TabMaintenance.SelectedTab = TabSystem
                RecordSearchLimit.Focus()
                Return False
            End If
        End If

        ' ----- If barcode item number is supplied, it must be numeric.
        If (RecordBarcodeItem.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordBarcodeItem.Text) = False) Then
                MessageBox.Show("Barcode Item Number, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
                TabMaintenance.SelectedTab = TabSystem
                RecordBarcodeItem.Focus()
                Return False
            End If
        End If

        ' ----- If barcode patron number is supplied, it must be numeric.
        If (RecordBarcodePatron.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordBarcodePatron.Text) = False) Then
                MessageBox.Show("Barcode Patron Number, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
                TabMaintenance.SelectedTab = TabSystem
                RecordBarcodePatron.Focus()
                Return False
            End If
        End If

        ' ----- If barcode misc number is supplied, it must be numeric.
        If (RecordBarcodeMisc.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordBarcodeMisc.Text) = False) Then
                MessageBox.Show("Barcode Miscellaneous Number, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
                TabMaintenance.SelectedTab = TabSystem
                RecordBarcodeMisc.Focus()
                Return False
            End If
        End If

        ' ----- If printer width is supplied, it must be numeric.
        If (EnableReceipts.Checked = True) And (RecordPrinterWidth.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordPrinterWidth.Text) = False) Then
                MessageBox.Show("Receipt Width, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
                TabMaintenance.SelectedTab = TabWorkstation
                RecordPrinterWidth.Focus()
                Return False
            End If
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save all of the field values.
        Dim holdText As String

        On Error Resume Next

        ' ===== System-wide fields.
        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the default location.
        If (CLng(DirectCast(RecordDefaultLocation.SelectedItem, ListItemData)) = -1&) Then _
            holdText = "" Else holdText = CStr(CLng(DirectCast(
            RecordDefaultLocation.SelectedItem, ListItemData)))
        SetSystemValue("DefaultLocation", holdText)

        SetSystemValue("FineGrace", RecordGraceDays.Text.Trim())
        SetSystemValue("SearchLimit", RecordSearchLimit.Text.Trim())

        ' ----- Save the classification type.
        If (CLng(DirectCast(RecordClassification.SelectedItem, ListItemData)) =
            ClassifyLC) Then holdText = "-1" Else holdText = "0"
        SetSystemValue("UseLC", holdText)

        ' ----- Save the self-checkout flag.
        If (RecordSelfCheckout.Checked = True) Then holdText = "-1" Else holdText = "0"
        SetSystemValue("PatronCheckOut", holdText)

        SetSystemValue("NextBarcodeItem", RecordBarcodeItem.Text.Trim())
        SetSystemValue("NextBarcodePatron", RecordBarcodePatron.Text.Trim())
        SetSystemValue("NextBarcodeMisc", RecordBarcodeMisc.Text.Trim())

        ' ----- Save the barcode font.
        If (CLng(DirectCast(RecordFontName.SelectedItem, ListItemData)) = -1&) Then _
            holdText = "" Else holdText = RecordFontName.Text
        SetSystemValue("BarcodeFont", holdText)

        ' ----- Save the code-39 barcode flag.
        If (RecordBarcode39.Checked = True) Then holdText = "-1" Else holdText = "0"
        SetSystemValue("BarcodeCode39", holdText)

        SetSystemValue("TicketHeading", RecordTicketTop.Text.Trim())
        SetSystemValue("TicketFooting", RecordTicketBottom.Text.Trim())

        ' ===== Workstation-specific fields.
        My.Settings.DBConnection = LibraryConnection
        My.Settings.ReportConfig = RecordConfigLocation.Text.Trim()
        My.Settings.HelpFile = RecordBasicHelp.Text.Trim()
        My.Settings.HelpFileAdmin = RecordAdminHelp.Text.Trim()
        My.Settings.HideLogin = HideLogin.Checked
        My.Settings.UseReceipts = EnableReceipts.Checked
        My.Settings.ReceiptPrinter = RecordPrinterLocation.Text.Trim()
        My.Settings.ReceiptPostlude = RecordPostlude.Text

        ' ----- Save the receipt printer width.
        If (RecordPrinterWidth.Text.Trim().Length = 0) Then
            My.Settings.ReceiptWidth = DefaultReceiptPrinterWidth
        Else
            My.Settings.ReceiptWidth = CInt(RecordPrinterWidth.Text)
        End If

        ' ----- Success.
        Me.Cursor = Cursors.Default
        Return True
    End Function

    Private Sub ActDBLocation_Click(sender As Object, e As EventArgs) Handles ActDBLocation.Click
        ' ----- Prompt for the database connection details.
        Dim newConnection As String

        ' ----- Prompt the user for the new setting.
        newConnection = LocateDatabase.PromptUser()
        If (newConnection.Length = 0) Then Return

        ' ----- Store the new value.
        LibraryConnection = newConnection
        RecordDBLocation.Text = GetDBDisplayText(LibraryConnection)
    End Sub

    Private Sub ActConfigLocation_Click(sender As Object, e As EventArgs) Handles ActConfigLocation.Click
        ' ----- Let the user browse for the report configuration file.
        '       Set up the file structure.
        LocateFile.DefaultExt = "xml"
        LocateFile.FileName = RecordConfigLocation.Text
        LocateFile.Filter = "XML Files (*.xml)|*.xml|" &
            "All Files (*.*)|*.*"
        LocateFile.FilterIndex = 1
        LocateFile.Title = "Locate Configuration File"

        ' ----- Prompt the user.
        If (LocateFile.ShowDialog() <> DialogResult.OK) Then Return

        ' ----- Save the file path.
        RecordConfigLocation.Text = LocateFile.FileName
    End Sub

    Private Sub ActBasicHelp_Click(sender As Object, e As EventArgs) Handles ActBasicHelp.Click
        ' ----- Let the user browse for the basic help file.
        '       Set up the file structure.
        LocateFile.DefaultExt = "chm"
        LocateFile.FileName = RecordBasicHelp.Text
        LocateFile.Filter = "Help Files (*.chm)|*.chm|" &
            "All Files (*.*)|*.*"
        LocateFile.FilterIndex = 1
        LocateFile.Title = "Locate Help"

        ' ----- Prompt the user.
        If (LocateFile.ShowDialog() <> DialogResult.OK) Then Return

        ' ----- Save the file path.
        RecordBasicHelp.Text = LocateFile.FileName
    End Sub

    Private Sub ActAdminHelp_Click(sender As Object, e As EventArgs) Handles ActAdminHelp.Click
        ' ----- Let the user browse for the administrative help file.
        '       Set up the file structure.
        LocateFile.DefaultExt = "chm"
        LocateFile.FileName = RecordAdminHelp.Text
        LocateFile.Filter = "Help Files (*.chm)|*.chm|" &
            "All Files (*.*)|*.*"
        LocateFile.FilterIndex = 1
        LocateFile.Title = "Locate Help"

        ' ----- Prompt the user.
        If (LocateFile.ShowDialog() <> DialogResult.OK) Then Return

        ' ----- Save the file path.
        RecordAdminHelp.Text = LocateFile.FileName
    End Sub

    Private Sub EnableReceipts_CheckedChanged(sender As Object, e As EventArgs) Handles EnableReceipts.CheckedChanged
        ' ----- Toggle the availability of receipt-specific fields.
        LabelPrinterLocation.Enabled = EnableReceipts.Checked
        RecordPrinterLocation.Enabled = EnableReceipts.Checked
        LabelPrinterWidth.Enabled = EnableReceipts.Checked
        RecordPrinterWidth.Enabled = EnableReceipts.Checked
        LabelPostlude.Enabled = EnableReceipts.Checked
        RecordPostlude.Enabled = EnableReceipts.Checked
        LabelPostludeInfo.Enabled = EnableReceipts.Checked
    End Sub
End Class