﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class LocateDatabase
    Private Const AuthenticationTypeWindows As Long = 1&
    Private Const AuthenticationTypeSQL As Long = 2&

    Public Function PromptUser() As String
        ' ----- Prompt the user for database connection details. Return a
        '       valid ADO.NET connection string for SQL Server, or a blank
        '       string if the user clicks Cancel.
        Dim newConnection As String

        ' ----- Prompt the user.
        Me.ShowDialog()

        ' ----- Build the new connection string.
        If (Me.DialogResult = DialogResult.OK) Then
            newConnection = "Data Source=" & RecordServer.Text.Trim() &
                ";Initial Catalog=" & RecordDatabase.Text.Trim()
            If (CLng(DirectCast(RecordAuthentication.SelectedItem, ListItemData)) =
                    AuthenticationTypeWindows) Then
                ' ----- Use Windows security.
                newConnection &= ";Integrated Security=true"
            Else
                ' ----- Use SQL Server security.
                newConnection &= ";User ID=" & RecordUser.Text.Trim() &
                    ";Password=" & RecordPassword.Text.Trim()
            End If
            Return newConnection
        Else
            Return ""
        End If
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Check the supplied data, and return True if it is all valid.
        '       Server name is required.
        If (RecordServer.Text.Trim().Length = 0) Then
            MessageBox.Show("The Server or Host name is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordServer.Focus()
            Return False
        End If

        ' ----- Database name is required.
        If (RecordDatabase.Text.Trim().Length = 0) Then
            MessageBox.Show("The Database Name is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordDatabase.Focus()
            Return False
        End If

        ' ----- For SQL Server authentication, the User ID is required.
        If (CLng(DirectCast(RecordAuthentication.SelectedItem, ListItemData)) =
                AuthenticationTypeSQL) Then
            If (RecordUser.Text.Trim().Length = 0) Then
                MessageBox.Show("The User Name is required for SQL Server authentication.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordUser.Focus()
                Return False
            End If
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub LocateDatabase_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim counter As Integer
        Dim connectionString As String
        Dim oneKey As String
        Dim oneValue As String

        ' ----- Add the available authentication schemes.
        RecordAuthentication.Items.Add(New ListItemData("Microsoft Windows",
            AuthenticationTypeWindows))
        RecordAuthentication.SelectedIndex = 0
        RecordAuthentication.Items.Add(New ListItemData("SQL Server",
            AuthenticationTypeSQL))

        ' ----- Load in the existing data.
        connectionString = My.Settings.DBConnection & ""
        For counter = 1 To CountSubStr(connectionString, ";") + 1
            ' ----- Each comma-delimited part has the format "key=value".
            oneKey = connectionString.GetSubStr(";", counter)
            oneValue = GetSubStr(oneKey, "=", 2).Trim()
            oneKey = oneKey.GetSubStr("=", 1).Trim().ToUpper().Replace(" ", "")

            ' ----- Process each part.
            Select Case oneKey
                Case "DATASOURCE"
                    ' ----- Show the server host.
                    RecordServer.Text = oneValue
                Case "INITIALCATALOG"
                    ' ----- Show the default database name.
                    RecordDatabase.Text = oneValue
                Case "INTEGRATEDSECURITY"
                    ' ----- Only check for "true". False is assumed.
                    If (oneValue.ToUpper() = "TRUE") Then _
                        RecordAuthentication.SelectedIndex =
                        RecordAuthentication.Items.IndexOf(AuthenticationTypeWindows)
                Case "USERID"
                    ' ----- A user ID forces SQL authentication.
                    RecordAuthentication.SelectedIndex =
                        RecordAuthentication.Items.IndexOf(AuthenticationTypeSQL)
                    RecordUser.Text = oneValue
                Case "PASSWORD"
                    ' ----- A password forces SQL authentication.
                    RecordAuthentication.SelectedIndex =
                        RecordAuthentication.Items.IndexOf(AuthenticationTypeSQL)
                    RecordPassword.Text = oneValue
            End Select
        Next counter
    End Sub

    Private Sub RecordDatabase_Enter(sender As Object, e As EventArgs) Handles RecordDatabase.Enter
        ' ----- Highlight the entire text.
        RecordDatabase.SelectAll()
    End Sub

    Private Sub RecordPassword_Enter(sender As Object, e As EventArgs) Handles RecordPassword.Enter
        ' ----- Highlight the entire text.
        RecordPassword.SelectAll()
    End Sub

    Private Sub RecordServer_Enter(sender As Object, e As EventArgs) Handles RecordServer.Enter
        ' ----- Highlight the entire text.
        RecordServer.SelectAll()
    End Sub

    Private Sub RecordUser_Enter(sender As Object, e As EventArgs) Handles RecordUser.Enter
        ' ----- Highlight the entire text.
        RecordUser.SelectAll()
    End Sub

    Private Sub RecordAuthentication_SelectedIndexChanged(sender As Object, e As EventArgs) Handles RecordAuthentication.SelectedIndexChanged
        ' ----- Enable dependent fields as needed.
        If (CLng(DirectCast(RecordAuthentication.SelectedItem, ListItemData)) =
                AuthenticationTypeWindows) Then
            ' ----- Windows authentication requires no user/password.
            LabelUser.Enabled = False
            RecordUser.Enabled = False
            LabelPassword.Enabled = False
            RecordPassword.Enabled = False
        Else
            ' ----- Requires specific SQL Server user and password.
            LabelUser.Enabled = True
            RecordUser.Enabled = True
            LabelPassword.Enabled = True
            RecordPassword.Enabled = True
        End If
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub
End Class