﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class PatronPayment
    Private ActivePatronCopyID As Long
    Private OneLineHeight As Integer
    Private TwoLineHeight As Integer
    Private SecondItemRow As Integer
    Private PaymentsOnly As Generic.List(Of PaymentItem)

    Private Enum EventEntryType
        NotDefined
        PatronPayment
        FineAdded
        FineDismissal
        RefundToPatron
        OverdueFines
    End Enum

    Private Class EventHistoryItem
        Public PaymentID As Long
        Public EntryDate As Date
        Public PaymentAmount As Decimal
        Public Comments As String
        Public EntryType As EventEntryType
    End Class

    ' ***** Insert Code Snippet #3 here.

    Private Sub PatronPayment_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim balanceDue As Decimal
        Dim originalFine As Decimal
        Dim canvas As Drawing.Graphics
        Dim historyItem As EventHistoryItem

        ' ----- Get the height of the history list lines.
        canvas = EventHistory.CreateGraphics()
        OneLineHeight = EventHistory.ItemHeight
        TwoLineHeight = CInt(canvas.MeasureString("A" & vbCrLf & "g",
            EventHistory.Font).Height()) + 3
        SecondItemRow = CInt(canvas.MeasureString("Ag",
            EventHistory.Font).Height()) + 1
        canvas.Dispose()

        ' ----- Retrieve the summary details for this patron/item entry.
        sqlText = "SELECT PA.FirstName, PA.LastName, NI.Title, PC.Fine, " &
            "PC.Paid FROM PatronCopy AS PC INNER JOIN Patron AS PA ON " &
            "PC.Patron = PA.ID INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " &
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " &
            "WHERE PC.ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActivePatronCopyID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("PatronPayment.PatronPayment_Load.Summary", ex)
            Return
        End Try
        oneRow = dbInfo.Rows(0)
        RecordPatron.Text = DBGetText(oneRow!FirstName) & " " &
            DBGetText(oneRow!LastName)
        RecordItem.Text = DBGetText(oneRow!Title)

        ' ***** Insert Code Snippet #4 here.

        dbInfo.Dispose()

        ' ----- Load in the history for this item.
        EventHistory.Items.Clear()
        sqlText = "SELECT * FROM PatronPayment " &
            "WHERE PatronCopy = @RecordID ORDER BY EntryDate"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActivePatronCopyID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("PatronPayment.PatronPayment_Load.History", ex)
            dbInfo = Nothing
        End Try
        If (dbInfo IsNot Nothing) Then
            For Each dbRow As DataRow In dbInfo.Rows
                ' ----- Build the history detail entry.
                historyItem = New EventHistoryItem
                historyItem.PaymentID = DBGetLong(dbRow!ID)
                historyItem.EntryDate = DBGetDate(dbRow!EntryDate)
                historyItem.PaymentAmount = DBGetDecimal(dbRow!Amount)
                historyItem.Comments = DBGetText(dbRow!Comment)
                Select Case DBGetText(dbRow!EntryType).ToUpper()
                    ' ----- The database stores letter codes.
                    Case "D"
                        historyItem.EntryType = EventEntryType.FineDismissal
                        originalFine += DBGetDecimal(dbRow!Amount)
                    Case "F"
                        historyItem.EntryType = EventEntryType.FineAdded
                        originalFine -= DBGetDecimal(dbRow!Amount)
                    Case "P"
                        historyItem.EntryType = EventEntryType.PatronPayment
                    Case "R"
                        historyItem.EntryType = EventEntryType.RefundToPatron
                End Select
                EventHistory.Items.Add(historyItem)
            Next dbRow
            dbInfo.Dispose()
            dbInfo = Nothing
        End If

        ' ----- Add in the original fine if needed.
        If (originalFine > 0@) Then
            historyItem = New EventHistoryItem
            historyItem.PaymentID = -1&
            historyItem.EntryDate = Date.MinValue
            historyItem.PaymentAmount = originalFine
            historyItem.Comments = ""
            historyItem.EntryType = EventEntryType.OverdueFines
            EventHistory.Items.Insert(0, historyItem)
        End If
    End Sub

    Private Function SaveEventData() As Boolean
        ' ----- Add a new payment entry. Return True on success.
        Dim entryType As EventEntryType
        Dim entryCode As String
        Dim fineAmount As Decimal
        Dim paidAmount As Decimal
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long
        Dim scanPayment As PaymentItem
        Dim foundPayment As Boolean
        Dim historyItem As EventHistoryItem

        ' ----- Determine the type of entry.
        entryType = EventEntryType.NotDefined
        If (OptionPayment.Checked = True) Then
            entryType = EventEntryType.PatronPayment
            entryCode = "P"
        ElseIf (OptionFine.Checked = True) Then
            entryType = EventEntryType.FineAdded
            entryCode = "F"
        ElseIf (OptionReduce.Checked = True) Then
            entryType = EventEntryType.FineDismissal
            entryCode = "D"
        Else
            ' ----- Refund entry (OptionRefund.Checked = True).
            entryType = EventEntryType.RefundToPatron
            entryCode = "R"
        End If

        ' ----- Determine the new totals.
        fineAmount = CDec(RecordFine.Text)
        paidAmount = CDec(RecordPayments.Text)
        Select Case entryType
            Case EventEntryType.PatronPayment
                paidAmount += CDec(RecordAmount.Text)
            Case EventEntryType.FineAdded
                fineAmount += CDec(RecordAmount.Text)
            Case EventEntryType.FineDismissal
                fineAmount -= CDec(RecordAmount.Text)
            Case EventEntryType.RefundToPatron
                paidAmount -= CDec(RecordAmount.Text)
        End Select
        Me.Cursor = Cursors.WaitCursor

        ' ***** Insert Code Snippet #5 here.

        ' ***** Insert Code Snippet #6 here.

        ' ----- Update the payment in the payments list.
        foundPayment = False
        For Each scanPayment In PaymentsOnly
            If (scanPayment.PatronCopyID = ActivePatronCopyID) Then
                ' ----- Update an existing payment.
                foundPayment = True
                If (entryType = EventEntryType.PatronPayment) Then _
                    scanPayment.FeesPaid += paidAmount
                scanPayment.BalanceDue = fineAmount - paidAmount
                Exit For
            End If
        Next scanPayment
        If (foundPayment = False) And (entryType = EventEntryType.PatronPayment) Then
            ' ----- Add a new payment.
            scanPayment = New PaymentItem
            scanPayment.PatronCopyID = ActivePatronCopyID
            scanPayment.ItemTitle = RecordItem.Text
            scanPayment.FeesPaid = paidAmount
            scanPayment.BalanceDue = fineAmount - paidAmount
            PaymentsOnly.Add(scanPayment)
        End If

        ' ***** Insert Code Snippet #7 here.

        ' ----- Success.
        Me.Cursor = Cursors.Default
        Return True
    End Function

    Private Function VerifyEventData() As Boolean
        ' ----- The user is trying to add a new event. Make sure all of the
        '       fields are valid. Return True on success.

        ' ----- An entry type is required.
        If (OptionPayment.Checked = False) And (OptionFine.Checked = False) And
                (OptionReduce.Checked = False) And (OptionRefund.Checked = False) Then
            MessageBox.Show("Please indicate the type of payment entry.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If

        ' ----- Some entry types require a comment.
        If (RecordComment.Text.Trim().Length = 0) Then
            If (OptionFine.Checked = True) Or (OptionRefund.Checked = True) Then
                MessageBox.Show("Comment is required for this type of payment entry.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        End If

        ' ----- Check for a valid amount.
        If (IsNumeric(RecordAmount.Text) = False) Then
            MessageBox.Show("Please supply a valid amount.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordAmount.Focus()
            Return False
        End If
        If (CDec(RecordAmount.Text) <= 0@) Or (CDec(RecordAmount.Text) >= 1000@) Then
            MessageBox.Show("Amount must be between $0.01 and $999.99.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordAmount.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActAdd_Click(sender As Object, e As EventArgs) Handles ActAdd.Click
        ' ----- Add a new payment entry.
        If (VerifyEventData() = False) Then Return
        If (SaveEventData() = False) Then Return

        ' ----- Prepare for a new entry.
        RecordAmount.Text = ""
        RecordComment.Text = ""
        OptionPayment.Checked = False
        OptionFine.Checked = False
        OptionReduce.Checked = False
        OptionRefund.Checked = False
        Return
    End Sub

    Private Sub EventHistory_MeasureItem(sender As Object, e As MeasureItemEventArgs) Handles EventHistory.MeasureItem
        ' ----- Is the active item one or two lines high?
        Dim historyItem As EventHistoryItem

        historyItem = DirectCast(EventHistory.Items(e.Index), EventHistoryItem)
        If (historyItem.Comments.Length = 0) Then
            ' ----- One line.
            e.ItemHeight = OneLineHeight
        Else
            ' ----- Two lines.
            e.ItemHeight = TwoLineHeight
        End If
    End Sub

    Private Sub EventHistory_DrawItem(sender As Object, e As DrawItemEventArgs) Handles EventHistory.DrawItem
        ' ----- Draw the matching items on two lines.
        Dim useBrush As Drawing.Brush
        Dim historyItem As EventHistoryItem
        Dim dateText As String
        Dim amountText As String
        Dim reasonText As String
        Dim commentText As String
        Dim italicFont As Drawing.Font

        ' ----- Draw the background of the item.
        If ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Then
            useBrush = SystemBrushes.HighlightText
        Else
            useBrush = SystemBrushes.WindowText
        End If
        e.DrawBackground()

        ' ----- Make an italic version of the font.
        italicFont = New Font(e.Font, FontStyle.Italic)

        ' ----- Get the record that contains the display details.
        historyItem = DirectCast(EventHistory.Items(e.Index), EventHistoryItem)

        ' ----- Format the display components.
        If (historyItem.EntryDate = Date.MinValue) Then
            dateText = "--/--/----"
        Else
            dateText = historyItem.EntryDate.ToString("M/d/yyyy")
        End If
        amountText = historyItem.PaymentAmount.ToString("C")
        Select Case historyItem.EntryType
            Case EventEntryType.PatronPayment
                reasonText = "Payment by patron"
            Case EventEntryType.FineAdded
                reasonText = "Fine added"
            Case EventEntryType.FineDismissal
                reasonText = "Dismissal of fine"
            Case EventEntryType.RefundToPatron
                reasonText = "Refund to patron"
            Case EventEntryType.OverdueFines
                reasonText = "Overdue fines"
            Case Else
                reasonText = "Undefined entry"
        End Select
        If (historyItem.Comments.Length = 0) Then commentText = "" Else _
            commentText = ">> " & historyItem.Comments

        ' ----- Draw the text of the item.
        e.Graphics.DrawString(dateText, e.Font, useBrush, 0, e.Bounds.Top)
        e.Graphics.DrawString(amountText, e.Font, useBrush,
            LabelHistoryCol2.Left - EventHistory.Left + 2, e.Bounds.Top)
        e.Graphics.DrawString(reasonText, e.Font, useBrush,
            LabelHistoryCol3.Left - EventHistory.Left + 2, e.Bounds.Top)
        If (commentText.Length <> 0) Then
            e.Graphics.DrawString(commentText, italicFont, useBrush,
                LabelHistoryCol3.Left - EventHistory.Left + 2,
                e.Bounds.Top + SecondItemRow)
        End If

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
        italicFont.Dispose()
    End Sub

    Private Sub RecordAmount_Enter(sender As Object, e As EventArgs) Handles RecordAmount.Enter
        ' ----- Highlight the entire text.
        RecordAmount.SelectAll()
    End Sub

    Private Sub RecordComment_Enter(sender As Object, e As EventArgs) Handles RecordComment.Enter
        ' ----- Highlight the entire text.
        RecordComment.SelectAll()
    End Sub
End Class
