﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class PatronAccess
    Private SelectedPatron As Long
    Private InListMode As Boolean

    Public Function SelectPatron() As Long
        ' ----- Let the user connect as a patron. Return the patron ID,
        '       or -1 if not selected.
        InListMode = False
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.Cancel) Then _
            Return -1& Else Return SelectedPatron
    End Function

    Private Sub RecordBarcode_Enter(sender As Object, e As EventArgs) Handles RecordBarcode.Enter
        ' ----- Highlight the entire text.
        RecordBarcode.SelectAll()
    End Sub

    Private Sub RecordPassword_Enter(sender As Object, e As EventArgs) Handles RecordPassword.Enter
        ' ----- Highlight the entire text.
        RecordPassword.SelectAll()
    End Sub

    Private Sub PatronAccess_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.

        ' ----- Move the list box to a reasonable position.
        PatronList.Location = New Point(LabelBarcode.Left, RecordBarcode.Top)
        PatronList.Size = New Size(RecordBarcode.Width + (RecordBarcode.Left -
            LabelBarcode.Left), PatronList.Height)

        ' ----- Reposition the password field.
        LabelNotRequired.Location = New Point(RecordPassword.Left,
            LabelPassword.Top)

        ' ----- Display the right password field.
        RecordPassword.Visible =
                Not SecurityProfile(LibrarySecurity.PatronOverride)
        LabelNotRequired.Visible =
            SecurityProfile(LibrarySecurity.PatronOverride)
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to access a patron.
        Dim barcodeText As String
        Dim lastName As String
        Dim firstName As String
        Dim passwordText As String
        Dim holdText As String
        Dim sqlText As String
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim sqlRun As DataCommand
        Dim patronID As Long

        ' ----- Special code for list mode.
        If (InListMode = True) Then
            ' ----- A patron selection is required.
            If (PatronList.SelectedIndex = -1) Then
                MessageBox.Show("Please select a patron from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                PatronList.Focus()
                Return
            End If

            ' ----- Save the selection and close the form.
            SelectedPatron = CLng(DirectCast(PatronList.SelectedItem, ListItemData))
            Me.DialogResult = DialogResult.OK
            Return
        End If

        ' ----- A patron name or barcode is required.
        If (RecordBarcode.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply a barcode or patron name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordBarcode.Focus()
            Return
        End If

        ' ----- Get the name or barcode text components.
        If (IsNumeric(RecordBarcode.Text)) Then
            ' ----- Perform a barcode search.
            barcodeText = DigitsOnly(RecordBarcode.Text)
            lastName = ""
            firstName = ""
        Else
            ' ----- Perform a name search.
            barcodeText = ""
            lastName = RecordBarcode.Text.GetSubStr(",", 1).Trim()
            firstName = RecordBarcode.Text.GetSubStr(",", 2).Trim()

            ' ----- A last name is required.
            If (lastName.Length = 0) Then
                MessageBox.Show("Please supply a patron name in 'Last, First' format.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordBarcode.Focus()
                Return
            End If

            ' ----- No wildcards allowed in the last name.
            If (lastName.Contains("*") = True) Then
                MessageBox.Show("The last name may not include wildcard characters.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordBarcode.Focus()
                Return
            End If

            ' ----- First name may contain wildcards.
            If (firstName.Contains("*") = False) Then firstName &= "*"
        End If

        ' ----- Check the password, if required.
        If (SecurityProfile(LibrarySecurity.PatronOverride) = True) Then
            passwordText = ""
        Else
            passwordText = EncryptPassword("patron", RecordPassword.Text)
        End If

        ' ----- Try to access the record.
        Me.Cursor = Cursors.WaitCursor
        sqlText = "SELECT ID, LastName, FirstName, BarCode, AdminMessage " &
            "FROM Patron WHERE "
        If (barcodeText.Length <> 0) Then
            ' ----- Barcode search.
            sqlText &= "Barcode = @TestBarcode"
        Else
            ' ----- Name search.
            sqlText &= "LastName LIKE @TestLast " &
                "AND FirstName LIKE @TestFirst"
        End If
        If (passwordText.Length <> 0) Then _
            sqlText &= " AND [Password] = @TestPassword"
        sqlRun = New DataCommand(sqlText)
        If (barcodeText.Length <> 0) Then
            sqlRun.AddText("TestBarcode", barcodeText)
        Else
            sqlRun.AddText("TestLast", lastName)
            sqlRun.AddText("TestFirst", firstName.Replace("*", "%"))
        End If
        If (passwordText.Length <> 0) Then _
            sqlRun.AddText("TestPassword", passwordText)

        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("PatronAccess.ActOK_Click.Retrieve", ex)
            Return
        End Try

        If (dbInfo.Rows.Count = 0) Then
            Me.Cursor = Cursors.Default
            dbInfo.Dispose()
            dbInfo = Nothing

            ' ----- Perhaps a barcode was being scanned, and the enter key added to
            '       the end caused the OK button to trigger. If so, silently move
            '       to the password field.
            If (barcodeText.Length <> 0) And (RecordPassword.Text.Length = 0) And
                    (Me.ActiveControl.Name = RecordBarcode.Name) Then
                If (RecordPassword.CanFocus) Then RecordPassword.Focus()
            Else
                MessageBox.Show("Invalid patron or password.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End If
            Return
        End If
        Me.Cursor = Cursors.Default

        ' ----- We found at least one matching patron.
        patronID = DBGetLong(dbInfo.Rows(0) !ID)
        If (dbInfo.Rows.Count > 1) Then
            ' ----- There is a second record. If in patron mode, then the user
            '       was too vague.
            If (passwordText.Length <> 0) Then
                dbInfo.Dispose()
                dbInfo = Nothing
                MessageBox.Show("Invalid patron or password.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return
            End If

            ' ----- Allow the user to choose the patron.
            For Each dbRow As DataRow In dbInfo.Rows
                ' ----- Add one patron to the list.
                holdText = DBGetText(dbRow!LastName) & ", " & DBGetText(dbRow!FirstName)
                If (IsDBNull(dbRow!Barcode) = False) Then _
                    holdText &= " (" & DBGetText(dbRow!Barcode) & ")"
                PatronList.Items.Add(New ListItemData(holdText, DBGetLong(dbRow!ID)))
            Next dbRow
            dbInfo.Dispose()
            dbInfo = Nothing

            ' ----- Change into "list" mode.
            InfoText.Text = "Your criteria matched more than one patron. " &
                "Please select the correct patron from the list."
            LabelBarcode.Visible = False
            RecordBarcode.Visible = False
            LabelPassword.Visible = False
            RecordPassword.Visible = False
            LabelNotRequired.Visible = False
            PatronList.Visible = True
            PatronList.Focus()
            InListMode = True
            Return
        Else
            ' ----- Found a valid match.
            dbInfo.Dispose()
            dbInfo = Nothing
        End If

        ' ----- If authorized, show the administrative message for this patron.
        If (SecurityProfile(LibrarySecurity.ViewAdminPatronMessages) = True) Then
            sqlText = "SELECT FirstName, LastName, AdminMessage " &
                "FROM Patron WHERE ID = " & patronID
            sqlRun = New DataCommand(sqlText)
            Try
                dbInfo = CreateDataTable(sqlRun)
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                GeneralError("PatronAccess.ActOK_Click.Message", ex)
                Return
            End Try

            oneRow = dbInfo.Rows(0)
            If (IsDBNull(oneRow!AdminMessage) = False) Then
                MessageBox.Show("Message concerning " & DBGetText(oneRow!FirstName) & " " &
                    DBGetText(oneRow!LastName) & ":" & vbCrLf & vbCrLf &
                    DBGetText(oneRow!AdminMessage),
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
            End If
            dbInfo.Dispose()
        End If

        ' ----- Finished.
        SelectedPatron = patronID
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub PatronList_DoubleClick(sender As Object, e As EventArgs) Handles PatronList.DoubleClick
        ' ----- Same as a click on the OK button.
        If (PatronList.SelectedIndex = -1) Then Return
        ActOK.PerformClick()
    End Sub
End Class