﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Module TicketPrinting
    Public Sub PrintCheckoutTicket(patronID As Long, checkedOutItems As ListBox)
        ' ----- Print out a ticket of what the patron checked out. The supplied
        '       ListBox control contains objects of type CheckedOutItem.
        Dim ticketWidth As Integer
        Dim ticketText As System.Text.StringBuilder
        Dim counter As Integer
        Dim patronFines As Decimal
        Dim itemDetail As CheckedOutItem
        Dim holdText As String

        ' ----- Ignore if there is nothing to print.
        If (patronID = -1&) Or (checkedOutItems.Items.Count = 0) Then Return

        ' ----- Get the width of the ticket.
        ticketWidth = My.Settings.ReceiptWidth
        If (ticketWidth <= 0) Then ticketWidth = 40

        ' ----- Build the heading.
        ticketText = GetTicketHeader(patronID, ticketWidth)
        If (ticketText Is Nothing) Then Return

        ' ----- Process each checked-out item.
        For counter = 0 To checkedOutItems.Items.Count - 1
            ' ----- Extract the detail from the list.
            itemDetail = DirectCast(checkedOutItems.Items(counter), CheckedOutItem)

            ' ----- Add the item name.
            holdText = itemDetail.ItemTitle
            If (holdText.Length > ticketWidth) Then _
                holdText = holdText.Substring(0, ticketWidth).TrimEnd()
            ticketText.AppendLine(holdText)

            ' ----- Add the barcode number and due date.
            ticketText.AppendLine(LeftAndRightText(itemDetail.Barcode,
                "Due: " & itemDetail.DueDate.ToString("MMM d, yyyy"), ticketWidth))
            ticketText.AppendLine()
        Next counter

        ' ----- If there are fines due, print them here.
        patronFines = CalculatePatronFines(patronID)
        If (patronFines > 0@) Then
            ticketText.AppendLine("Fines Due: " & patronFines.ToString("C"))
            ticketText.AppendLine()
        End If

        ' ----- Add the bottom display text.
        ticketText.Append(GetTicketFooter(ticketWidth))

        ' ----- Send the ticket to the printer.
        Try
            RawPrinterHelper.SendStringToPrinter(My.Settings.ReceiptPrinter,
                ticketText.ToString(), "Checkout Receipt")
        Catch ex As Exception
            GeneralError("TicketPrinting.PrintCheckoutTicket", ex)
        End Try
    End Sub

    Public Sub PrintBalanceTicket(patronID As Long,
            finesList As ListBox)
        ' ----- Print out a ticket of unpaid balances. The supplied ListBox
        '       control contains objects of type PatronDetialItem.
        Dim ticketWidth As Integer
        Dim ticketText As System.Text.StringBuilder
        Dim holdText As String
        Dim counter As Integer
        Dim itemDetail As PatronDetailItem
        Dim included As Integer = 0

        ' ----- Ignore if there is nothing to print.
        If (patronID = -1&) Or (finesList.Items.Count = 0) Then Return

        ' ----- Get the width of the ticket.
        ticketWidth = My.Settings.ReceiptWidth
        If (ticketWidth <= 0) Then ticketWidth = 40

        ' ----- Build the heading.
        ticketText = GetTicketHeader(patronID, ticketWidth)
        If (ticketText Is Nothing) Then Return

        ' ----- Process each checked-out item.
        For counter = 0 To finesList.Items.Count - 1
            ' ----- Extract the detail from the list.
            itemDetail = DirectCast(finesList.Items(counter), PatronDetailItem)

            ' ----- Skip entries with a zero balance.
            If (itemDetail.BalanceDue = 0@) Then Continue For
            included += 1

            ' ----- Add the item name.
            holdText = itemDetail.TitleText
            If (holdText.Length > ticketWidth) Then _
                holdText = holdText.Substring(0, ticketWidth).TrimEnd()
            ticketText.AppendLine(holdText)

            ' ----- Print the remaining balance.
            ticketText.AppendLine(LeftAndRightText("Balance Due:",
                itemDetail.BalanceDue.ToString("C"), ticketWidth))
            ticketText.AppendLine()
        Next counter

        ' ----- Warn if there is nothing to print.
        If (included = 0) Then
            MessageBox.Show("There are no unpaid balances to print.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
            Return
        End If

        ' ----- If there are fines due, print them here.
        ticketText.AppendLine("Fines Due: " &
            CalculatePatronFines(patronID).ToString("C"))
        ticketText.AppendLine()

        ' ----- Add the bottom display text.
        ticketText.Append(GetTicketFooter(ticketWidth))

        ' ----- Send the ticket to the printer.
        Try
            RawPrinterHelper.SendStringToPrinter(My.Settings.ReceiptPrinter,
                ticketText.ToString(), "Balance Receipt")
        Catch ex As Exception
            GeneralError("TicketPrinting.PrintBalanceTicket", ex)
        End Try
    End Sub

    Public Sub PrintPaymentTicket(patronID As Long,
            paymentList As Generic.List(Of PaymentItem))
        ' ----- Print out a ticket of unpaid balances. The supplied ListBox
        '       control contains objects of type PatronDetialItem.
        Dim ticketWidth As Integer
        Dim ticketText As System.Text.StringBuilder
        Dim holdText As String
        Dim counter As Integer
        Dim itemDetail As PaymentItem
        Dim included As Integer = 0

        ' ----- Ignore if there is nothing to print.
        If (patronID = -1&) Or (paymentList.Count = 0) Then Return

        ' ----- Get the width of the ticket.
        ticketWidth = My.Settings.ReceiptWidth
        If (ticketWidth <= 0) Then ticketWidth = 40

        ' ----- Build the heading.
        ticketText = GetTicketHeader(patronID, ticketWidth)
        If (ticketText Is Nothing) Then Return

        ' ----- Process each checked-out item.
        For counter = 0 To paymentList.Count - 1
            ' ----- Extract the detail from the list.
            itemDetail = paymentList(counter)

            ' ----- Ignore items with no payment amount.
            If (itemDetail.FeesPaid = 0@) Then Continue For
            included += 1

            ' ----- Add the item name.
            holdText = itemDetail.ItemTitle
            If (holdText.Length > ticketWidth) Then _
                holdText = holdText.Substring(0, ticketWidth).TrimEnd()
            ticketText.AppendLine(holdText)

            ' ----- Show the payment amount.
            ticketText.AppendLine(LeftAndRightText("Payments:",
                itemDetail.FeesPaid.ToString("C"), ticketWidth))

            ' ----- Print the remaining balance.
            ticketText.AppendLine(LeftAndRightText("Balance Due:",
                itemDetail.BalanceDue.ToString("C"), ticketWidth))
            ticketText.AppendLine()
        Next counter
        If (included = 0) Then Return

        ' ----- If there are fines due, print them here.
        ticketText.AppendLine("Fines Due: " &
            CalculatePatronFines(patronID).ToString("C"))
        ticketText.AppendLine()

        ' ----- Add the bottom display text.
        ticketText.Append(GetTicketFooter(ticketWidth))

        ' ----- Send the ticket to the printer.
        Try
            RawPrinterHelper.SendStringToPrinter(My.Settings.ReceiptPrinter,
                ticketText.ToString(), "Balance Receipt")
        Catch ex As Exception
            GeneralError("TicketPrinting.PrintPaymentTicket", ex)
        End Try
    End Sub

    Private Function GetTicketHeader(patronID As Long, ticketWidth As Integer) As System.Text.StringBuilder
        ' ----- Print out a ticket of what the patron checked out. The supplied
        '       ListBox control contains objects of type CheckedOutItem.
        Dim ticketText As System.Text.StringBuilder = Nothing
        Dim holdText As String
        Dim counter As Integer
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Check for an active printer.
        If (My.Settings.UseReceipts = False) Then Return Nothing

        ' ----- Get the printer path.
        If (My.Settings.ReceiptPrinter Is Nothing) Then Return Nothing
        If (My.Settings.ReceiptPrinter.Trim().Length = 0) Then Return Nothing

        ' ----- Create the storage area for the content.
        ticketText = New System.Text.StringBuilder

        ' ----- Start with the heading.
        holdText = GetSystemValue("TicketHeading").Trim()
        If (holdText.Length <> 0) Then
            For counter = 1 To CountSubStr(holdText, "|") + 1
                ' ----- Center each line of the heading.
                ticketText.AppendLine(CenterText(
                    holdText.GetSubStr("|", counter), ticketWidth))
            Next counter
        End If

        ' ----- Add today's date.
        ticketText.AppendLine(CenterText(
            Today.ToString("MMMM d, yyyy"), ticketWidth))
        ticketText.AppendLine()

        ' ----- Add the patron name.
        sqlText = "SELECT FirstName + ' ' + LastName FROM Patron " &
            "WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", patronID)
        Try
            ticketText.AppendLine(CenterText("Patron: " &
                DBGetText(ExecuteSQLReturn(sqlRun)), ticketWidth))
        Catch ex As Exception
            GeneralError("TicketPrinting.GetTicketHeader", ex)
        End Try
        ticketText.AppendLine()

        ' ----- That's the full header.
        Return ticketText
    End Function

    Private Function GetTicketFooter(ticketWidth As Integer) As String
        ' ----- Build the content for the end of the ticket.
        Dim ticketText As String = ""
        Dim holdText As String
        Dim counter As Integer

        ' ----- Add the bottom display text.
        holdText = GetSystemValue("TicketFooting").Trim()
        If (holdText.Length <> 0) Then
            For counter = 1 To CountSubStr(holdText, "|") + 1
                ' ----- Center each line of the heading.
                ticketText &= CenterText(holdText.GetSubStr("|", counter),
                    ticketWidth) & vbCrLf
            Next counter
        End If

        ' ----- Include any final characters.
        ticketText &= GetPrinterPostlude()
        Return ticketText
    End Function

    Private Function GetPrinterPostlude() As String
        ' ----- Retrieve the postlude and process any escape sequences.
        Dim origText As String
        Dim oneChar As String
        Dim resultText As String
        Dim position As Integer
        Dim hexCode As Integer

        ' ----- Retrieve the raw postlude.
        origText = My.Settings.ReceiptPostlude
        If (origText Is Nothing) Then Return ""
        If (origText.Trim().Length = 0) Then Return ""

        ' ----- Scan through the text and process any backslash characters.
        resultText = ""
        position = 0
        Do While (position < origText.Length)
            ' ----- Peek at the next character.
            oneChar = origText.Substring(position, 1)
            If (oneChar = "\") Then
                ' ----- Handle the escape character.
                If (position = (origText.Length - 1)) Then
                    ' ----- Already at the end.
                    oneChar = ""
                Else
                    oneChar = origText.Substring(position + 1, 1)
                End If
                Select Case oneChar.ToUpper()
                    Case "E"
                        ' ----- Add the escape character.
                        resultText &= CStr(Chr(Keys.Escape))
                        position += 2
                    Case "N"
                        ' ----- Add the newline character.
                        resultText &= CStr(Chr(Keys.LineFeed))
                        position += 2
                    Case "R"
                        ' ----- Add the carriage return character.
                        resultText &= CStr(Chr(Keys.Return))
                        position += 2
                    Case "X"
                        ' ----- Add in an ASCII value identified by the next
                        '       two hex digits.
                        Try
                            oneChar = origText.Substring(position + 2, 2)
                            hexCode = CInt("&H" & oneChar)
                            resultText &= CStr(Chr(hexCode))
                        Catch
                            ' ----- Ignore this character.
                        Finally
                            position += 4
                        End Try
                    Case ""
                        ' ----- Already at the end. Just skip the backslash.
                        position += 1
                    Case Else
                        ' ----- Simply add this character to the final output.
                        '       oneChar may be the "\" character.
                        resultText &= oneChar
                        position += 2
                End Select
            Else
                ' ----- Simply add this character to the final output.
                resultText &= oneChar
                position += 1
            End If
        Loop

        ' ----- Return the final text.
        Return resultText
    End Function
End Module
