﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class CodeCopyStatus
    Private ActiveID As Long
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageCopyStatus)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageCopyStatus)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageCopyStatus)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the copy status code?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE Status = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("You cannot delete this record because it is in use.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        Catch ex As Exception
            GeneralError("CodeCopyStatus.DeleteRecord.Check", ex)
            Return False
        End Try

        ' ----- Delete the record.
        sqlText = "DELETE FROM CodeCopyStatus WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("CodeCopyStatus.DeleteRecord.Delete", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox,
            ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT * FROM CodeCopyStatus ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("CodeCopyStatus.FillListWithRecords", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(DBGetText(dbRow!FullName),
                DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM CodeCopyStatus WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            Return DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("CodeCopyStatus.FormatRecordName", ex)
            Return "Error"
        End Try
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to establish the different status conditions in " &
            "which library materials can be found."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Copy Status Codes"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        If (ActiveID = -1&) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT FullName FROM CodeCopyStatus WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            RecordFullName.Text = DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("CodeCopyStatus.PrepareFormFields", ex)
        End Try
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the record.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long

        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO CodeCopyStatus (FullName) " &
                "OUTPUT INSERTED.ID VALUES (@NewName)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                GeneralError("CodeCopyStatus.SaveFormData.Add", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE CodeCopyStatus SET FullName = @NewName " &
                "WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("CodeCopyStatus.SaveFormData.Update", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Name is a required field.
        If (RecordFullName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the copy status name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM CodeCopyStatus " &
            "WHERE UPPER(FullName) = @TestName"
        If (ActiveID <> -1&) Then sqlText &= " AND ID <> @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestName", RecordFullName.Text.Trim().ToUpper())
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                ' ----- Duplicate entry.
                MessageBox.Show(String.Format(
                    "A record already exists with the name '{0}'.",
                    RecordFullName.Text.Trim()),
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordFullName.Focus()
                Return False
            End If
        Catch ex As Exception
            GeneralError("CodeCopyStatus.ValidateFormData", ex)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Private Sub RecordFullName_Enter(sender As Object, e As EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub CodeCopyStatus_Activated(sender As Object, e As EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub
End Class