﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class GroupName
    Private ActiveID As Long
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageGroups)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageGroups)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageGroups)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the record.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim transInfo As LibraryTransaction = Nothing

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the security group?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM UserName WHERE GroupID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("You cannot delete this record because it is in use.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        Catch ex As Exception
            GeneralError("GroupName.DeleteRecord.Check", ex)
            Return False
        End Try

        ' ----- Delete the record.
        Try
            transInfo = TransactionBegin()

            sqlText = "DELETE FROM GroupActivity WHERE GroupID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            sqlText = "DELETE FROM GroupName WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            GeneralError("GroupName.DeleteRecord.Delete", ex)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT * FROM GroupName ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("GroupName.FillListWithRecords", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM GroupName WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            Return DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("GroupName.FormatRecordName", ex)
            Return "Error"
        End Try
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to establish the security groups into which " &
            "administrative users are placed. Each security group can receive " &
            "access to different parts of the system."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Security Groups"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim dbInfo As DataTable
        Dim sqlRun As DataCommand
        Dim matchPos As Integer

        ' ----- Load in the list of security groups.
        If (ActivityList.Items.Count = 0) Then
            ' ----- This is the first time. Load from the database.
            ActivityList.Items.Clear()
            sqlText = "SELECT * FROM Activity ORDER BY FullName"
            Try
                dbInfo = CreateDataTable(sqlText)
            Catch ex As Exception
                GeneralError("GroupName.PrepareFormFields.Retrieve", ex)
                Return
            End Try
            For Each dbRow As DataRow In dbInfo.Rows
                ActivityList.Items.Add(New ListItemData(
                    DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
            Next dbRow
            dbInfo.Dispose()
            dbInfo = Nothing
        Else
            ' ----- Editing a subsequent code. Clear selections. Amazingly,
            '       there's no Clear method to do this.
            Do While (ActivityList.CheckedItems.Count > 0)
                ActivityList.SetItemChecked(ActivityList.CheckedIndices(0), False)
            Loop
            If (ActivityList.Items.Count > 0) Then ActivityList.TopIndex = 0
        End If

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        If (ActiveID = -1&) Then Return

        ' ----- Get the name of this group.
        sqlText = "SELECT FullName FROM GroupName WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            RecordFullName.Text = DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("GroupName.PrepareFormFields.Name", ex)
            Return
        End Try

        ' ----- Load in the existing security settings.
        sqlText = "SELECT ActivityID FROM GroupActivity WHERE GroupID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("GroupName.PrepareFormFields.Current", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            matchPos = ActivityList.Items.IndexOf(DBGetLong(dbRow!ActivityID))
            If (matchPos <> -1) Then ActivityList.SetItemChecked(matchPos, True)
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- The user wants to save changes. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long = -1&
        Dim transInfo As LibraryTransaction

        ' ----- Prepare to save the data.
        Me.Cursor = Cursors.WaitCursor
        transInfo = TransactionBegin()

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO GroupName (FullName) " &
                "OUTPUT INSERTED.ID VALUES (@NewName)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun, transInfo))
            Catch ex As Exception
                TransactionRollback(transInfo)
                Me.Cursor = Cursors.Default
                GeneralError("GroupName.SaveFormData.Add", ex)
                Return False
            End Try
        Else
            ' ----- Update the existing entry.
            newID = ActiveID
            sqlText = "UPDATE GroupName SET FullName = @NewName " &
                "WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun, transInfo)
            Catch ex As Exception
                TransactionRollback(transInfo)
                Me.Cursor = Cursors.Default
                GeneralError("GroupName.SaveFormData.Update", ex)
                Return False
            End Try
        End If

        ' ----- Clear any existing security settings.
        sqlText = "DELETE FROM GroupActivity WHERE GroupID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", newID)
        Try
            ExecuteSQL(sqlRun, transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            Me.Cursor = Cursors.Default
            GeneralError("GroupName.SaveFormData.Clear", ex)
            Return False
        End Try

        ' ----- Save the selected security settings.
        For Each itemChecked As ListItemData In ActivityList.CheckedItems
            sqlText = "INSERT INTO GroupActivity (GroupID, ActivityID) " &
                "VALUES (@NewGroup, @NewActivity)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("NewGroup", newID)
            sqlRun.AddLong("NewActivity", itemChecked.ItemData)
            Try
                ExecuteSQL(sqlRun, transInfo)
            Catch ex As Exception
                TransactionRollback(transInfo)
                Me.Cursor = Cursors.Default
                GeneralError("GroupName.SaveFormData.Activity", ex)
                Return False
            End Try
        Next itemChecked

        ' ----- Complete all changes.
        Try
            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            Me.Cursor = Cursors.Default
            GeneralError("GroupName.SaveFormData.Commit", ex)
            Return False
        End Try

        ' ----- This change may affect this user.
        If (LoggedInGroupID = newID) Then ReprocessSecuritySet()

        ' ----- Success.
        ActiveID = newID
        Me.Cursor = Cursors.Default
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Name is a required field.
        If (RecordFullName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the security group name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM GroupName " &
            "WHERE UPPER(FullName) = @NewName"
        If (ActiveID <> -1&) Then sqlText &= " AND ID <> @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("NewName", RecordFullName.Text.Trim().ToUpper())
        If (ActiveID <> -1&) Then _
            sqlRun.AddLong("RecordID", ActiveID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("A record already exists with the name '" &
                    RecordFullName.Text.Trim() & "'.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordFullName.Focus()
                Return False
            End If
        Catch ex As Exception
            GeneralError("GroupName.ValidateFormData", ex)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Private Sub GroupName_Activated(sender As Object, e As EventArgs) Handles Me.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordFullName_Enter(sender As Object, e As EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- Save changes and return to the summary form.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub
End Class
