﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class BarcodeLabel
    Private ActiveLabelID As Long
    Private CurrentMeasure As Char
    Private StartingOver As Boolean
    Private LabelWidth As Integer
    Private LabelHeight As Integer
    Private IgnoreSizeChange As Boolean
    Private BarcodeFontName As String
    Private Barcode39 As Boolean

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        BarcodeFontName = GetBarcodeFont()
        If (BarcodeFontName.Length = 0) Then Return -1&
        ActiveLabelID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveLabelID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageBarcodeTemplates)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the record.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim transInfo As LibraryTransaction = Nothing

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete this barcode label template?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Delete the record.
        Try
            transInfo = TransactionBegin()

            sqlText = "DELETE FROM BarcodeLabelItem WHERE BarcodeLabel = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            sqlText = "DELETE FROM BarcodeLabel WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            GeneralError("BarcodeLabel.DeleteRecord", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        BarcodeFontName = GetBarcodeFont()
        If (BarcodeFontName.Length = 0) Then Return -1&
        ActiveLabelID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveLabelID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT BL.ID, BL.FullName, BS.FullName AS PageName " &
            "FROM BarcodeLabel AS BL INNER JOIN BarcodeSheet AS BS " &
            "ON BL.BarcodeSheet = BS.ID ORDER BY BL.FullName, BS.FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("BarcodeLabel.FillListWithRecords", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(DBGetText(dbRow!FullName) & " on " &
                DBGetText(dbRow!PageName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim result As String

        ' ----- Initialize.
        result = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT BL.FullName, BS.FullName AS PageName " &
            "FROM BarcodeLabel AS BL INNER JOIN BarcodeSheet AS BS " &
            "ON BL.BarcodeSheet = BS.ID WHERE BL.ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("BarcodeLabel.FormatRecordName", ex)
            Return "Error"
        End Try
        If (dbInfo.Rows.Count > 0) Then
            oneRow = dbInfo.Rows(0)
            result = DBGetText(oneRow!FullName) & " on " & DBGetText(oneRow!PageName)
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
        Return result
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return a description of this editor.
        Return "Use these codes to identify the format of printed barcode labels. " &
            "You can define the layout of a label using text, barcodes, and shapes."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the title-bar text for this editor.
        Return "Barcode Label Templates"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim newBarcodeImage As Library.BarcodeItemBarcodeImage
        Dim newBarcodeText As Library.BarcodeItemBarcodeNumber
        Dim newStaticText As Library.BarcodeItemText
        Dim newStaticLine As Library.BarcodeItemLine
        Dim newStaticRect As Library.BarcodeItemRect

        ' ----- Load in the list of templates.
        If (RecordTemplate.Items.Count = 0) Then
            RecordTemplate.Items.Add(New ListItemData("<Not Selected>", -1&))
            sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName"
            Try
                dbInfo = CreateDataTable(sqlText)
            Catch ex As Exception
                GeneralError("BarcodeLabel.PrepareFormFields.Template", ex)
                Return
            End Try
            For Each dbRow As DataRow In dbInfo.Rows
                RecordTemplate.Items.Add(New ListItemData(
                    DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
            Next dbRow
            dbInfo.Dispose()
            dbInfo = Nothing
        End If

        ' ----- Fill in the list of units.
        If (RecordUnits.Items.Count = 0) Then
            RecordUnits.Items.Add(New ListItemData("Inches", Asc("I")))
            RecordUnits.Items.Add(New ListItemData("Centimeters", Asc("C")))
            RecordUnits.Items.Add(New ListItemData("Points", Asc("P")))
            RecordUnits.Items.Add(New ListItemData("Twips", Asc("T")))
            RecordUnits.SelectedIndex = 0
        End If
        CurrentMeasure = "I"c

        ' ----- Clear the existing data.
        StartingOver = True
        IgnoreSizeChange = True
        LabelWidth = -1
        LabelHeight = -1
        RecordName.Text = ""
        RecordTemplate.SelectedIndex = RecordTemplate.Items.IndexOf(-1&)
        If (RecordTemplate.SelectedIndex = -1) Then RecordTemplate.SelectedIndex = 0
        RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(Asc(CurrentMeasure))
        If (RecordUnits.SelectedIndex = -1) Then RecordUnits.SelectedIndex = 0
        DisplayItems.Items.Clear()
        ActUp.Enabled = False
        ActDown.Enabled = False
        ActDelete.Enabled = False
        ItemProperties.SelectedObject = Nothing
        RecordSample.Text = "12345678"
        PreviewArea.Invalidate()
        IgnoreSizeChange = False
        Barcode39 = CBool(Val(GetSystemValue("BarcodeCode39")))
        If (ActiveLabelID = -1&) Then Return

        ' ----- Load in the record.
        sqlText = "SELECT * FROM BarcodeLabel WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveLabelID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("BarcodeLabel.PrepareFormFields.Retrieve", ex)
            Return
        End Try
        oneRow = dbInfo.Rows(0)
        IgnoreSizeChange = True
        RecordName.Text = DBGetText(oneRow!FullName)
        RecordTemplate.SelectedIndex =
            RecordTemplate.Items.IndexOf(DBGetLong(oneRow!BarcodeSheet))
        If (RecordTemplate.SelectedIndex = -1) Then RecordTemplate.Items.IndexOf(-1&)
        RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(Asc(CChar(DBGetText(oneRow!UnitType))))
        If (RecordUnits.SelectedIndex = -1) Then RecordUnits.SelectedIndex = 0
        CurrentMeasure = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        IgnoreSizeChange = False
        dbInfo.Dispose()

        ' ----- Determine the size of the label.
        RefreshLabelTemplate()
        RefreshSizeDisplay()

        ' ----- Load in the list of objects.
        sqlText = "SELECT * FROM BarcodeLabelItem " &
            "WHERE BarcodeLabel = @RecordID ORDER BY Priority"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveLabelID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("BarcodeLabel.PrepareFormFields.Objects", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Process this object.
            Select Case DBGetText(dbRow!ItemType).ToUpper()
                Case "B"
                    ' ----- Barcode image.
                    newBarcodeImage = New Library.BarcodeItemBarcodeImage
                    newBarcodeImage.Alignment = CType(DBGetInteger(dbRow!Alignment),
                        ContentAlignment)
                    newBarcodeImage.BarcodeColor = Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newBarcodeImage.BarcodeSize = DBGetDecimal(dbRow!FontSize)
                    newBarcodeImage.Left = DBGetDecimal(dbRow!PosLeft)
                    newBarcodeImage.Top = DBGetDecimal(dbRow!PosTop)
                    newBarcodeImage.Width = DBGetDecimal(dbRow!PosWidth)
                    newBarcodeImage.Height = DBGetDecimal(dbRow!PosHeight)
                    newBarcodeImage.RotationAngle = CShort(DBGetInteger(dbRow!Rotation))
                    newBarcodeImage.PadDigits = CByte(DBGetInteger(dbRow!PadDigits))
                    DisplayItems.Items.Add(newBarcodeImage)
                Case "L"
                    ' ----- Line.
                    newStaticLine = New Library.BarcodeItemLine
                    newStaticLine.LineColor = Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newStaticLine.X1 = DBGetDecimal(dbRow!PosLeft)
                    newStaticLine.X2 = DBGetDecimal(dbRow!PosWidth)
                    newStaticLine.Y1 = DBGetDecimal(dbRow!PosTop)
                    newStaticLine.Y2 = DBGetDecimal(dbRow!PosHeight)
                    DisplayItems.Items.Add(newStaticLine)
                Case "N"
                    ' ----- Barcode text.
                    newBarcodeText = New Library.BarcodeItemBarcodeNumber
                    newBarcodeText.Alignment = CType(dbRow!Alignment, ContentAlignment)
                    newBarcodeText.Left = DBGetDecimal(dbRow!PosLeft)
                    newBarcodeText.Top = DBGetDecimal(dbRow!PosTop)
                    newBarcodeText.Width = DBGetDecimal(dbRow!PosWidth)
                    newBarcodeText.Height = DBGetDecimal(dbRow!PosHeight)
                    newBarcodeText.RotationAngle = CShort(DBGetInteger(dbRow!Rotation))
                    newBarcodeText.TextColor = Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newBarcodeText.Font = New Font(DBGetText(dbRow!FontName),
                        CSng(DBGetDecimal(dbRow!FontSize)), BuildFontStyle(DBGetText(dbRow!FontStyle)))
                    newBarcodeText.PadDigits = CByte(DBGetInteger(dbRow!PadDigits))
                    DisplayItems.Items.Add(newBarcodeText)
                Case "R"
                    ' ----- Rectangle.
                    newStaticRect = New Library.BarcodeItemRect
                    newStaticRect.BorderColor = Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newStaticRect.FillColor = Color.FromArgb(DBGetInteger(dbRow!Color2))
                    newStaticRect.Left = DBGetDecimal(dbRow!PosLeft)
                    newStaticRect.Top = DBGetDecimal(dbRow!PosTop)
                    newStaticRect.Width = DBGetDecimal(dbRow!PosWidth)
                    newStaticRect.Height = DBGetDecimal(dbRow!PosHeight)
                    newStaticRect.RotationAngle = CShort(DBGetInteger(dbRow!Rotation))
                    DisplayItems.Items.Add(newStaticRect)
                Case "T"
                    ' ----- Static text.
                    newStaticText = New Library.BarcodeItemText
                    newStaticText.DisplayText = DBGetText(dbRow!StaticText)
                    newStaticText.Alignment = CType(dbRow!Alignment, ContentAlignment)
                    newStaticText.Left = DBGetDecimal(dbRow!PosLeft)
                    newStaticText.Top = DBGetDecimal(dbRow!PosTop)
                    newStaticText.Width = DBGetDecimal(dbRow!PosWidth)
                    newStaticText.Height = DBGetDecimal(dbRow!PosHeight)
                    newStaticText.RotationAngle = CShort(DBGetInteger(dbRow!Rotation))
                    newStaticText.TextColor = Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newStaticText.Font = New Font(DBGetText(dbRow!FontName),
                        DBGetDecimal(dbRow!FontSize), BuildFontStyle(DBGetText(dbRow!FontStyle)))
                    DisplayItems.Items.Add(newStaticText)
            End Select
        Next dbRow
        dbInfo.Dispose()
        PreviewArea.Invalidate()
    End Sub

    Private Sub RefreshSizeDisplay()
        ' ----- Show the size of the label.
        Dim toUnit As Char

        toUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        If (LabelWidth = -1) Or (LabelHeight = -1) Then
            LabelPreview.Text = "Label Preview"
        Else
            LabelPreview.Text = String.Format(
                "Label Preview: {0} (w) by {1} (h)",
                ConvertPageUnits("T"c, toUnit, LabelWidth),
                ConvertPageUnits("T"c, toUnit, LabelHeight))
        End If
    End Sub

    Private Function ValidateForm() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Name is a required field.
        If (RecordName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the label template name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordName.Focus()
            Return False
        End If

        ' ----- Make sure a template is selected.
        If (CLng(DirectCast(RecordTemplate.SelectedItem, ListItemData)) = -1&) Then
            MessageBox.Show("Please select a page template.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordTemplate.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM BarcodeLabel " &
            "WHERE UPPER(FullName) = @TestName AND BarcodeSheet = @SheetID"
        If (ActiveLabelID <> -1&) Then sqlText &= " AND ID <> @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestName", RecordName.Text.Trim().ToUpper())
        sqlRun.AddLong("SheetID", CLng(DirectCast(RecordTemplate.SelectedItem, ListItemData)))
        sqlRun.AddLong("RecordID", ActiveLabelID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                ' ----- Duplicate entry.
                MessageBox.Show("A label template using the page template '" & RecordTemplate.Text &
                    "' already exists with the name '" & RecordName.Text.Trim() & "'.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordName.Focus()
                Return False
            End If
        Catch ex As Exception
            GeneralError("BarcodeLabel.ValidateForm", ex)
            Return False
        End Try

        ' ----- At least one item is required.
        If (DisplayItems.Items.Count = 0) Then
            MessageBox.Show("The label template must include at least one item.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub RefreshLabelTemplate()
        ' ----- Given the active template, determine the width and height of a
        '       single label, in twips.
        Dim oneWidth As Single
        Dim oneHeight As Single
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim useMeasure As Char
        Dim dbInfo As DataTable
        Dim oneRow As DataRow

        ' ----- A width and height of -1 indicates an unknown size.
        LabelWidth = -1
        LabelHeight = -1
        If (RecordTemplate.SelectedIndex = -1) Then Return
        If (DirectCast(RecordTemplate.SelectedItem, ListItemData).ItemData = -1&) Then Return
        Me.Cursor = Cursors.WaitCursor

        ' ----- Retrieve the label details.
        sqlText = "SELECT * FROM BarcodeSheet WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", CLng(DirectCast(RecordTemplate.SelectedItem, ListItemData)))
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("BarcodeLabel.RefreshLabelTemplate", ex)
            Return
        End Try

        ' ----- The label size is what is left over on the page after the
        '       margins and the space between labels is removed.
        oneRow = dbInfo.Rows(0)
        oneWidth = (DBGetDecimal(oneRow!PageWidth) - DBGetDecimal(oneRow!MarginLeft) -
            DBGetDecimal(oneRow!MarginRight) - ((DBGetDecimal(oneRow!ColumnsCount) - 1.0F) *
            DBGetDecimal(oneRow!IntraColumn))) / DBGetDecimal(oneRow!ColumnsCount)
        oneHeight = (DBGetDecimal(oneRow!PageHeight) - DBGetDecimal(oneRow!MarginTop) -
            DBGetDecimal(oneRow!MarginBottom) - ((DBGetDecimal(oneRow!RowsCount) - 1.0F) *
            DBGetDecimal(oneRow!IntraRow))) / DBGetDecimal(oneRow!RowsCount)
        useMeasure = CChar(DBGetText(oneRow!UnitType))
        dbInfo.Dispose()

        ' ----- Convert the result into twips.
        LabelWidth = CInt(ConvertPageUnits(useMeasure, "T"c, oneWidth))
        LabelHeight = CInt(ConvertPageUnits(useMeasure, "T"c, oneHeight))
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub AdjustListItemButtons()
        ' ----- Adjust the display based on the selection.
        If (DisplayItems.SelectedIndex = -1) Then
            ItemProperties.SelectedObject = Nothing
            ActUp.Enabled = False
            ActDown.Enabled = False
            ActDelete.Enabled = False
        Else
            ItemProperties.SelectedObject = DisplayItems.SelectedItem
            ActUp.Enabled = CBool(DisplayItems.SelectedIndex > 0)
            ActDown.Enabled = CBool(DisplayItems.SelectedIndex <
                (DisplayItems.Items.Count - 1))
            ActDelete.Enabled = True
        End If
    End Sub

    Private Sub RecordName_Enter(sender As Object, e As EventArgs) Handles RecordName.Enter
        ' ----- Highlight the entire text.
        RecordName.SelectAll()
    End Sub

    Private Sub RecordSample_Enter(sender As Object, e As EventArgs) Handles RecordSample.Enter
        ' ----- Highlight the entire text.
        RecordSample.SelectAll()
    End Sub

    Private Sub DisplayItems_SelectedIndexChanged(sender As Object, e As EventArgs) Handles DisplayItems.SelectedIndexChanged
        ' ----- Adjust the display based on the selection.
        AdjustListItemButtons()
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActText_Click(sender As Object, e As EventArgs) Handles ActText.Click
        ' ----- Add a new static text item.
        Dim newStaticText As Library.BarcodeItemText
        Dim fromUnit As Char

        fromUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        newStaticText = New Library.BarcodeItemText
        newStaticText.DisplayText = "Text"
        newStaticText.Alignment = ContentAlignment.TopLeft
        newStaticText.RotationAngle = 0
        newStaticText.TextColor = Color.Black
        newStaticText.Font = New Font("Arial", 12)
        newStaticText.Left = 0
        newStaticText.Top = 0
        newStaticText.Width = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newStaticText.Height = ConvertPageUnits("I"c, fromUnit, 0.5F)
        DisplayItems.Items.Add(newStaticText)
        DisplayItems.SelectedItem = newStaticText
        ItemProperties.SelectedObject = newStaticText
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActBarcode_Click(sender As Object, e As EventArgs) Handles ActBarcode.Click
        ' ----- Add a new barcode item.
        Dim newBarcodeImage As Library.BarcodeItemBarcodeImage
        Dim fromUnit As Char

        fromUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        newBarcodeImage = New Library.BarcodeItemBarcodeImage
        newBarcodeImage.Alignment = ContentAlignment.TopLeft
        newBarcodeImage.BarcodeColor = Color.Black
        newBarcodeImage.BarcodeSize = 10.0F
        newBarcodeImage.Left = 0
        newBarcodeImage.Top = 0
        newBarcodeImage.Width = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newBarcodeImage.Height = ConvertPageUnits("I"c, fromUnit, 0.5F)
        newBarcodeImage.RotationAngle = 0
        newBarcodeImage.PadDigits = 0
        DisplayItems.Items.Add(newBarcodeImage)
        DisplayItems.SelectedItem = newBarcodeImage
        ItemProperties.SelectedObject = newBarcodeImage
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActBarNumber_Click(sender As Object, e As EventArgs) Handles ActBarNumber.Click
        ' ----- Add a new barcode text item.
        Dim newBarcodeText As Library.BarcodeItemBarcodeNumber
        Dim fromUnit As Char

        fromUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        newBarcodeText = New Library.BarcodeItemBarcodeNumber
        newBarcodeText.Alignment = ContentAlignment.TopLeft
        newBarcodeText.Left = 0
        newBarcodeText.Top = 0
        newBarcodeText.Width = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newBarcodeText.Height = ConvertPageUnits("I"c, fromUnit, 0.5F)
        newBarcodeText.RotationAngle = 0
        newBarcodeText.TextColor = Color.Black
        newBarcodeText.PadDigits = 0
        newBarcodeText.Font = New Font("Arial", 12)
        DisplayItems.Items.Add(newBarcodeText)
        DisplayItems.SelectedItem = newBarcodeText
        ItemProperties.SelectedObject = newBarcodeText
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActLine_Click(sender As Object, e As EventArgs) Handles ActLine.Click
        ' ----- Add a new line item.
        Dim newStaticLine As Library.BarcodeItemLine
        Dim fromUnit As Char

        fromUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        newStaticLine = New Library.BarcodeItemLine
        newStaticLine.LineColor = Color.Black
        newStaticLine.X1 = 0
        newStaticLine.X2 = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newStaticLine.Y1 = 0
        newStaticLine.Y2 = ConvertPageUnits("I"c, fromUnit, 0.5F)
        DisplayItems.Items.Add(newStaticLine)
        DisplayItems.SelectedItem = newStaticLine
        ItemProperties.SelectedObject = newStaticLine
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActRectangle_Click(sender As Object, e As EventArgs) Handles ActRectangle.Click
        ' ----- Add a new rectangle item.
        Dim newStaticRect As Library.BarcodeItemRect
        Dim fromUnit As Char

        fromUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))
        newStaticRect = New Library.BarcodeItemRect
        newStaticRect.BorderColor = Color.Black
        newStaticRect.FillColor = Color.White
        newStaticRect.Left = 0
        newStaticRect.Top = 0
        newStaticRect.Width = ConvertPageUnits("I"c, fromUnit, 1.0F)
        newStaticRect.Height = ConvertPageUnits("I"c, fromUnit, 0.5F)
        newStaticRect.RotationAngle = 0
        DisplayItems.Items.Add(newStaticRect)
        DisplayItems.SelectedItem = newStaticRect
        ItemProperties.SelectedObject = newStaticRect
        PreviewArea.Invalidate()
    End Sub

    Private Sub ItemProperties_PropertyValueChanged(sender As Object, e As PropertyValueChangedEventArgs) Handles ItemProperties.PropertyValueChanged
        ' ----- Update the preview when any property changes.
        PreviewArea.Invalidate()

        ' ----- Update the text of the list box item.
        DisplayItems.Items(DisplayItems.SelectedIndex) = DisplayItems.SelectedItem
    End Sub

    Private Sub ActUp_Click(sender As Object, e As EventArgs) Handles ActUp.Click
        ' ----- Move the selected item up one position.
        Dim holdObject As Object

        On Error Resume Next

        ' ----- Ignore if no item is selected.
        If (DisplayItems.SelectedIndex <= 0) Then Return

        ' ----- Reverse the items.
        holdObject = DisplayItems.Items(DisplayItems.SelectedIndex - 1)
        DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex - 1)
        DisplayItems.Items.Insert(DisplayItems.SelectedIndex + 1, holdObject)

        ' ----- Update the display.
        AdjustListItemButtons()
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActDown_Click(sender As Object, e As EventArgs) Handles ActDown.Click
        ' ----- Move the selected item down one position.
        Dim holdObject As Object

        On Error Resume Next

        ' ----- Ignore if no item is selected.
        If (DisplayItems.SelectedIndex = -1) Then Return
        If (DisplayItems.SelectedIndex = (DisplayItems.Items.Count - 1)) Then Return

        ' ----- Reverse the items.
        holdObject = DisplayItems.Items(DisplayItems.SelectedIndex + 1)
        DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex + 1)
        DisplayItems.Items.Insert(DisplayItems.SelectedIndex, holdObject)

        ' ----- Update the display.
        AdjustListItemButtons()
        PreviewArea.Invalidate()
    End Sub

    Private Sub ActDelete_Click(sender As Object, e As EventArgs) Handles ActDelete.Click
        ' ----- Delete the selected item.
        If (DisplayItems.SelectedIndex = -1) Then Return
        DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex)

        ' ----- Refresh the display.
        ItemProperties.SelectedObject = Nothing
        AdjustListItemButtons()
        PreviewArea.Invalidate()
    End Sub

    Private Sub PreviewArea_Paint(sender As Object, e As PaintEventArgs) Handles PreviewArea.Paint
        ' ----- Display a preview of the image.
        Dim pageWidth As Integer
        Dim pageHeight As Integer
        Dim pageLeft As Integer  ' Start of the "white" area.
        Dim pageTop As Integer   ' Start of the "white" area.
        Dim useRatio As Single
        Dim useRatioText As Single
        Dim fromUnit As Char
        Dim counter As Integer
        Dim textMessage As String
        Dim useGeneric As BarcodeItemGeneric
        Dim useBarImage As BarcodeItemBarcodeImage
        Dim useBarText As BarcodeItemBarcodeNumber
        Dim useText As BarcodeItemText
        Dim useLine As BarcodeItemLine
        Dim useRect As BarcodeItemRect
        Dim useFont As Font
        Dim holdState As Drawing2D.GraphicsState
        Dim pixelPen As Pen
        Dim pixelBrush As SolidBrush
        Dim textColor As Color
        Dim textAngle As Integer
        Dim textAlign As ContentAlignment
        Dim textFormat As New StringFormat
        Dim X1 As Single
        Dim X2 As Single
        Dim Y1 As Single
        Dim Y2 As Single

        On Error Resume Next

        ' ----- Draw in the background color.
        e.Graphics.Clear(SystemColors.Control)
        If (LabelWidth <= 0) Or (LabelHeight <= 0) Then Return

        ' ----- Determine the page area.
        pageWidth = PreviewArea.Width - 4
        pageHeight = PreviewArea.Height - 4
        useRatio = CSng(pageWidth) / CSng(LabelWidth)
        useRatioText = CSng(pageWidth) / ConvertPageUnits("T"c, "P"c, LabelWidth)
        If (CInt(LabelHeight * useRatio) > pageHeight) Then
            ' ----- Scale based on the height.
            useRatio = CSng(pageHeight) / CSng(LabelHeight)
            useRatioText = CSng(pageHeight) / ConvertPageUnits("T"c, "P"c, LabelHeight)
            pageWidth = CInt(LabelWidth * useRatio)
        Else
            ' ----- Scale based on the width.
            pageHeight = CInt(LabelHeight * useRatio)
        End If
        pageLeft = CInt((PreviewArea.Width - 4 - pageWidth) / 2) + 1
        pageTop = CInt((PreviewArea.Height - 4 - pageHeight) / 2) + 1

        ' ----- Draw the page area.
        e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 1, pageTop + 1, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 2, pageTop + 2, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(Brushes.Black, pageLeft - 1, pageTop - 1, pageWidth + 2, pageHeight + 2)
        e.Graphics.FillRectangle(Brushes.White, pageLeft, pageTop, pageWidth, pageHeight)

        ' ----- What size of units are we using for item display?
        If (RecordUnits.SelectedIndex = -1) Then Return
        fromUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))

        ' ----- Transform a page of twips into a display page.
        e.Graphics.TranslateTransform(pageLeft, pageTop)
        e.Graphics.ScaleTransform(useRatio, useRatio)

        ' ----- Process each item in the list.
        For counter = 0 To DisplayItems.Items.Count - 1
            ' ----- Save the current state of the graphics area.
            holdState = e.Graphics.Save()

            ' ----- Different code for each type of drawing object.
            useGeneric = DirectCast(DisplayItems.Items(counter), Library.BarcodeItemGeneric)
            Select Case useGeneric.ItemType
                Case "B"
                    ' ----- Barcode image.
                    useBarImage = DirectCast(useGeneric, Library.BarcodeItemBarcodeImage)
                    textMessage = RecordSample.Text.Trim()
                    If (textMessage.Length = 0) Then textMessage = "12345678"
                    If (textMessage.Length < useBarImage.PadDigits) Then
                        textMessage = New String("0"c, useBarImage.PadDigits -
                            textMessage.Length) & textMessage
                    End If
                    If (Barcode39 = True) Then textMessage = "*" & textMessage & "*"
                    textColor = useBarImage.BarcodeColor
                    textAngle = useBarImage.RotationAngle
                    textAlign = useBarImage.Alignment

                    ' ----- Get the font to use.
                    useFont = New Font(BarcodeFontName,
                        useBarImage.BarcodeSize, FontStyle.Regular)

                    ' ----- Get the coordinates in points.
                    X1 = ConvertPageUnits(fromUnit, "P"c, useBarImage.Left)
                    X2 = ConvertPageUnits(fromUnit, "P"c, useBarImage.Width)
                    Y1 = ConvertPageUnits(fromUnit, "P"c, useBarImage.Top)
                    Y2 = ConvertPageUnits(fromUnit, "P"c, useBarImage.Height)

                    ' ----- The rest is the same as drawing normal text.
                    GoTo DisplayTheText
                Case "L"
                    ' ----- Line.
                    useLine = DirectCast(useGeneric, Library.BarcodeItemLine)

                    ' ----- Create a one-pixel pen to use.
                    pixelPen = New Pen(useLine.LineColor, 1 / e.Graphics.DpiX)

                    ' ----- Get the coordinates in twips.
                    X1 = ConvertPageUnits(fromUnit, "T"c, useLine.X1)
                    X2 = ConvertPageUnits(fromUnit, "T"c, useLine.X2)
                    Y1 = ConvertPageUnits(fromUnit, "T"c, useLine.Y1)
                    Y2 = ConvertPageUnits(fromUnit, "T"c, useLine.Y2)

                    ' ----- Draw the line.
                    e.Graphics.DrawLine(pixelPen, X1, Y1, X2, Y2)
                    pixelPen.Dispose()

                    ' ----- If this is the active line object, draw a thin outline
                    '       around the object.
                    If (DisplayItems.SelectedIndex = counter) Then
                        pixelPen = New Pen(Color.LightGray, 1 / e.Graphics.DpiX)
                        pixelPen.DashStyle = Drawing2D.DashStyle.Dash
                        e.Graphics.DrawRectangle(pixelPen, X1 - 15, Y1 - 15,
                            X2 - X1 + 30, Y2 - Y1 + 30)
                        pixelPen.Dispose()
                    End If
                Case "N"
                    ' ----- Barcode text.
                    useBarText = DirectCast(useGeneric, Library.BarcodeItemBarcodeNumber)
                    textMessage = RecordSample.Text.Trim()
                    If (textMessage.Length = 0) Then textMessage = "12345678"
                    If (textMessage.Length < useBarText.PadDigits) Then
                        textMessage = New String("0"c, useBarText.PadDigits -
                            textMessage.Length) & textMessage
                    End If
                    textColor = useBarText.TextColor
                    textAngle = useBarText.RotationAngle
                    textAlign = useBarText.Alignment

                    ' ----- Get the font to use.
                    useFont = New Font(useBarText.Font.Name,
                        useBarText.Font.SizeInPoints, useBarText.Font.Style)

                    ' ----- Get the coordinates in points.
                    X1 = ConvertPageUnits(fromUnit, "P"c, useBarText.Left)
                    X2 = ConvertPageUnits(fromUnit, "P"c, useBarText.Width)
                    Y1 = ConvertPageUnits(fromUnit, "P"c, useBarText.Top)
                    Y2 = ConvertPageUnits(fromUnit, "P"c, useBarText.Height)

                    ' ----- The rest is the same as drawing normal text.
                    GoTo DisplayTheText
                Case "R"
                    ' ----- Rectangle.
                    useRect = DirectCast(useGeneric, Library.BarcodeItemRect)

                    ' ----- Create a one-pixel pen to use.
                    pixelPen = New Pen(useRect.BorderColor, 1 / e.Graphics.DpiX)
                    pixelBrush = New SolidBrush(useRect.FillColor)

                    ' ----- Get the coordinates in twips.
                    X1 = ConvertPageUnits(fromUnit, "T"c, useRect.Left)
                    X2 = ConvertPageUnits(fromUnit, "T"c, useRect.Width)
                    Y1 = ConvertPageUnits(fromUnit, "T"c, useRect.Top)
                    Y2 = ConvertPageUnits(fromUnit, "T"c, useRect.Height)

                    ' ----- Rotate the surface if requested.
                    If (useRect.RotationAngle <> 0) Then
                        e.Graphics.TranslateTransform(X1, Y1)
                        e.Graphics.RotateTransform(useRect.RotationAngle)
                        X1 = 0.0F
                        Y1 = 0.0F
                    End If

                    ' ----- Draw the rectangle.
                    e.Graphics.FillRectangle(pixelBrush, X1, Y1, X2, Y2)
                    e.Graphics.DrawRectangle(pixelPen, X1, Y1, X2, Y2)
                    pixelBrush.Dispose()
                    pixelPen.Dispose()

                    ' ----- If this is the active line object, draw a thin outline
                    '       around the object.
                    If (DisplayItems.SelectedIndex = counter) Then
                        pixelPen = New Pen(Color.LightGray, 1 / e.Graphics.DpiX)
                        pixelPen.DashStyle = Drawing2D.DashStyle.Dash
                        e.Graphics.DrawRectangle(pixelPen, X1 - 1 / e.Graphics.DpiX,
                            Y1 - 1 / e.Graphics.DpiX, X2 + 2 / e.Graphics.DpiX,
                            Y2 + 2 / e.Graphics.DpiX)
                        pixelPen.Dispose()
                    End If
                Case "T"
                    ' ----- Text.
                    useText = DirectCast(useGeneric, Library.BarcodeItemText)
                    textMessage = useText.DisplayText
                    textColor = useText.TextColor
                    textAngle = useText.RotationAngle
                    textAlign = useText.Alignment

                    ' ----- Get the font to use.
                    useFont = New Font(useText.Font.Name,
                        useText.Font.SizeInPoints, useText.Font.Style)

                    ' ----- Get the coordinates in points.
                    X1 = ConvertPageUnits(fromUnit, "P"c, useText.Left)
                    X2 = ConvertPageUnits(fromUnit, "P"c, useText.Width)
                    Y1 = ConvertPageUnits(fromUnit, "P"c, useText.Top)
                    Y2 = ConvertPageUnits(fromUnit, "P"c, useText.Height)

DisplayTheText:
                    ' ----- For text, we do a new transformation. Everything is
                    '       measured in points instead of twips.
                    e.Graphics.ResetTransform()
                    e.Graphics.TranslateTransform(pageLeft, pageTop)
                    e.Graphics.ScaleTransform(useRatioText, useRatioText)

                    ' ----- Rotate around the upper-left corner of the text box
                    '       if needed.
                    If (textAngle <> 0) Then
                        e.Graphics.TranslateTransform(X1, Y1)
                        e.Graphics.RotateTransform(textAngle)
                        X1 = 0.0F
                        Y1 = 0.0F
                    End If

                    ' ----- If this is the active text object, draw a thin outline
                    '       around the object.
                    If (DisplayItems.SelectedIndex = counter) Then
                        pixelPen = New Pen(Color.LightGray, 1 / e.Graphics.DpiX)
                        pixelPen.DashStyle = Drawing2D.DashStyle.Dash
                        e.Graphics.DrawRectangle(pixelPen, X1, Y1, X2, Y2)
                        pixelPen.Dispose()
                    End If

                    ' ----- Position the text within the bounding box.
                    Select Case textAlign
                        Case ContentAlignment.TopLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Far
                        Case ContentAlignment.TopCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Far
                        Case ContentAlignment.TopRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Far
                    End Select
                    textFormat.FormatFlags = textFormat.FormatFlags Or
                        StringFormatFlags.NoWrap

                    ' ----- Print the text.
                    e.Graphics.DrawString(textMessage, useFont,
                        New SolidBrush(textColor),
                        New RectangleF(X1, Y1, X2, Y2), textFormat)
                    useFont.Dispose()
            End Select

            ' ----- Restore the original transformed state of the graphics surface.
            e.Graphics.Restore(holdState)
        Next counter

        ' ----- Remove the page-size transformations.
        e.Graphics.ResetTransform()
    End Sub

    Private Sub RecordTemplate_SelectedIndexChanged(sender As Object, e As EventArgs) Handles RecordTemplate.SelectedIndexChanged
        ' ----- The user selected a different page template.
        If (IgnoreSizeChange = True) Then Return
        RefreshLabelTemplate()
        RefreshSizeDisplay()
        PreviewArea.Invalidate()
    End Sub

    Private Sub RecordSample_TextChanged(sender As Object, e As EventArgs) Handles RecordSample.TextChanged
        ' ----- Refresh the display on a change.
        PreviewArea.Invalidate()
    End Sub

    Private Sub RecordUnits_SelectedIndexChanged(sender As Object, e As EventArgs) Handles RecordUnits.SelectedIndexChanged
        ' ----- The user is altering the size.
        Dim toUnit As Char
        Dim counter As Integer
        Dim useGeneric As Library.BarcodeItemGeneric
        Dim useBarImage As Library.BarcodeItemBarcodeImage
        Dim useBarText As Library.BarcodeItemBarcodeNumber
        Dim useText As Library.BarcodeItemText
        Dim useLine As Library.BarcodeItemLine
        Dim useRect As Library.BarcodeItemRect

        On Error Resume Next

        RefreshSizeDisplay()
        If (IgnoreSizeChange = True) Then Return
        If (RecordUnits.SelectedIndex = -1) Then Return
        toUnit = Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData))

        ' ----- Change the units of each object to the new size.
        For counter = 0 To DisplayItems.Items.Count - 1
            useGeneric = DirectCast(DisplayItems.Items(counter), Library.BarcodeItemGeneric)
            Select Case useGeneric.ItemType
                Case "B"
                    ' ----- Barcode image.
                    useBarImage = DirectCast(useGeneric, Library.BarcodeItemBarcodeImage)
                    useBarImage.Left = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Left)
                    useBarImage.Top = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Top)
                    useBarImage.Width = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Width)
                    useBarImage.Height = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Height)
                Case "L"
                    ' ----- Line.
                    useLine = DirectCast(useGeneric, Library.BarcodeItemLine)
                    useLine.X1 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.X1)
                    useLine.X2 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.X2)
                    useLine.Y1 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.Y1)
                    useLine.Y2 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.Y2)
                Case "N"
                    ' ----- Barcode text.
                    useBarText = DirectCast(useGeneric, Library.BarcodeItemBarcodeNumber)
                    useBarText.Left = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Left)
                    useBarText.Top = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Top)
                    useBarText.Width = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Width)
                    useBarText.Height = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Height)
                Case "R"
                    ' ----- Rectangle.
                    useRect = DirectCast(useGeneric, Library.BarcodeItemRect)
                    useRect.Left = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Left)
                    useRect.Top = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Top)
                    useRect.Width = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Width)
                    useRect.Height = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Height)
                Case "T"
                    ' ----- Text.
                    useText = DirectCast(useGeneric, Library.BarcodeItemText)
                    useText.Left = ConvertPageUnits(CurrentMeasure, toUnit, useText.Left)
                    useText.Top = ConvertPageUnits(CurrentMeasure, toUnit, useText.Top)
                    useText.Width = ConvertPageUnits(CurrentMeasure, toUnit, useText.Width)
                    useText.Height = ConvertPageUnits(CurrentMeasure, toUnit, useText.Height)
            End Select

            ' ----- Update the display item in the list.
            DisplayItems.Items(counter) = DisplayItems.Items(counter)
        Next counter

        ' ----- Update the properties area.
        If (DisplayItems.SelectedIndex <> -1) Then _
            ItemProperties.SelectedObject = DisplayItems.SelectedItem
        CurrentMeasure = toUnit
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim counter As Integer
        Dim useGeneric As Library.BarcodeItemGeneric
        Dim useBarImage As Library.BarcodeItemBarcodeImage
        Dim useBarText As Library.BarcodeItemBarcodeNumber
        Dim useText As Library.BarcodeItemText
        Dim useLine As Library.BarcodeItemLine
        Dim useRect As Library.BarcodeItemRect
        Dim newID As Long
        Dim transInfo As LibraryTransaction

        ' ----- Confirm the data.
        If (ValidateForm() = False) Then Return
        Me.Cursor = Cursors.WaitCursor
        transInfo = TransactionBegin()

        ' ----- Save the parent record data.
        If (ActiveLabelID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO BarcodeLabel (FullName, BarcodeSheet, UnitType) " &
                "OUTPUT INSERTED.ID VALUES (@NewName, @NewSheet, @NewUnit)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordName.Text.Trim())
            sqlRun.AddLong("NewSheet", CLng(DirectCast(RecordTemplate.SelectedItem, ListItemData)))
            sqlRun.AddText("NewUnit", Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData)))
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun, transInfo))
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                TransactionRollback(transInfo)
                GeneralError("BarcodeLabel.ActOK_Click.AddParent", ex)
                Return
            End Try
            ActiveLabelID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE BarcodeLabel SET " &
                "FullName = @NewName, " &
                "BarcodeSheet = @NewSheet, " &
                "UnitType = @NewUnit " &
                "WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordName.Text.Trim())
            sqlRun.AddLong("NewSheet", CLng(DirectCast(RecordTemplate.SelectedItem, ListItemData)))
            sqlRun.AddText("NewUnit", Chr(CInt(DirectCast(RecordUnits.SelectedItem, ListItemData).ItemData)))
            sqlRun.AddLong("RecordID", ActiveLabelID)
            Try
                ExecuteSQL(sqlRun, transInfo)
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                TransactionRollback(transInfo)
                GeneralError("BarcodeLabel.ActOK_Click.UpdateParent", ex)
                Return
            End Try

            ' ----- Remove the detail records.
            sqlText = "DELETE FROM BarcodeLabelItem WHERE BarcodeLabel = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", ActiveLabelID)
            Try
                ExecuteSQL(sqlRun, transInfo)
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                TransactionRollback(transInfo)
                GeneralError("BarcodeLabel.ActOK_Click.ClearItem", ex)
                Return
            End Try
        End If

        ' ----- Add the detail records.
        For counter = 0 To DisplayItems.Items.Count - 1
            ' ----- Build the command to add the record.
            useGeneric = DirectCast(DisplayItems.Items(counter), Library.BarcodeItemGeneric)
            sqlText = "INSERT INTO BarcodeLabelItem (Priority, BarcodeLabel, ItemType, " &
                "PosLeft, PosTop, PosWidth, PosHeight, Rotation, FontName, FontSize, " &
                "FontStyle, StaticText, Color1, Color2, Alignment, PadDigits) VALUES (" &
                "@NewPriority, @NewBarcode, @NewType, @NewLeft, @NewTop, @NewWidth, " &
                "@NewHeight, @NewRotation, @NewFont, @NewSize, @NewStyle, @NewText, " &
                "@NewColor1, @NewColor2, @NewAlign, @NewPad)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddInteger("NewPriority", counter + 1)
            sqlRun.AddLong("NewBarcode", ActiveLabelID)
            sqlRun.AddText("NewType", useGeneric.ItemType)
            Select Case useGeneric.ItemType
                Case "B"
                    ' ----- Barcode image.
                    useBarImage = DirectCast(useGeneric, Library.BarcodeItemBarcodeImage)
                    sqlRun.AddDecimal("NewLeft", CDec(useBarImage.Left))
                    sqlRun.AddDecimal("NewTop", CDec(useBarImage.Top))
                    sqlRun.AddDecimal("NewWidth", CDec(useBarImage.Width))
                    sqlRun.AddDecimal("NewHeight", CDec(useBarImage.Height))
                    sqlRun.AddInteger("NewRotation", useBarImage.RotationAngle)
                    sqlRun.AddText("NewFont")
                    sqlRun.AddDecimal("NewSize", CDec(useBarImage.BarcodeSize))
                    sqlRun.AddText("NewStyle")
                    sqlRun.AddText("NewText")
                    sqlRun.AddLong("NewColor1", useBarImage.BarcodeColor.ToArgb())
                    sqlRun.AddLong("NewColor2")
                    sqlRun.AddInteger("NewAlign", useBarImage.Alignment)
                    sqlRun.AddInteger("NewPad", useBarImage.PadDigits)
                Case "L"
                    ' ----- Line.
                    useLine = DirectCast(useGeneric, Library.BarcodeItemLine)
                    sqlRun.AddDecimal("NewLeft", CDec(useLine.X1))
                    sqlRun.AddDecimal("NewTop", CDec(useLine.Y1))
                    sqlRun.AddDecimal("NewWidth", CDec(useLine.X2))
                    sqlRun.AddDecimal("NewHeight", CDec(useLine.Y2))
                    sqlRun.AddInteger("NewRotation", 0)
                    sqlRun.AddText("NewFont")
                    sqlRun.AddDecimal("NewSize")
                    sqlRun.AddText("NewStyle")
                    sqlRun.AddText("NewText")
                    sqlRun.AddLong("NewColor1", useLine.LineColor.ToArgb())
                    sqlRun.AddLong("NewColor2")
                    sqlRun.AddInteger("NewAlign")
                    sqlRun.AddInteger("NewPad")
                Case "N"
                    ' ----- Barcode text.
                    useBarText = DirectCast(useGeneric, Library.BarcodeItemBarcodeNumber)
                    sqlRun.AddDecimal("NewLeft", CDec(useBarText.Left))
                    sqlRun.AddDecimal("NewTop", CDec(useBarText.Top))
                    sqlRun.AddDecimal("NewWidth", CDec(useBarText.Width))
                    sqlRun.AddDecimal("NewHeight", CDec(useBarText.Height))
                    sqlRun.AddInteger("NewRotation", useBarText.RotationAngle)
                    sqlRun.AddText("NewFont", useBarText.Font.Name)
                    sqlRun.AddDecimal("NewSize", CDec(useBarText.Font.SizeInPoints))
                    sqlRun.AddText("NewStyle", DBFontStyle(useBarText.Font))
                    sqlRun.AddText("NewText")
                    sqlRun.AddLong("NewColor1", useBarText.TextColor.ToArgb())
                    sqlRun.AddLong("NewColor2")
                    sqlRun.AddInteger("NewAlign", useBarText.Alignment)
                    sqlRun.AddInteger("NewPad", useBarText.PadDigits)
                Case "R"
                    ' ----- Rectangle.
                    useRect = DirectCast(useGeneric, Library.BarcodeItemRect)
                    sqlRun.AddDecimal("NewLeft", CDec(useRect.Left))
                    sqlRun.AddDecimal("NewTop", CDec(useRect.Top))
                    sqlRun.AddDecimal("NewWidth", CDec(useRect.Width))
                    sqlRun.AddDecimal("NewHeight", CDec(useRect.Height))
                    sqlRun.AddInteger("NewRotation", useRect.RotationAngle)
                    sqlRun.AddText("NewFont")
                    sqlRun.AddDecimal("NewSize")
                    sqlRun.AddText("NewStyle")
                    sqlRun.AddText("NewText")
                    sqlRun.AddLong("NewColor1", useRect.BorderColor.ToArgb())
                    sqlRun.AddLong("NewColor2", useRect.FillColor.ToArgb())
                    sqlRun.AddInteger("NewAlign")
                    sqlRun.AddInteger("NewPad")
                Case "T"
                    ' ----- Text.
                    useText = DirectCast(useGeneric, Library.BarcodeItemText)
                    sqlRun.AddDecimal("NewLeft", CDec(useText.Left))
                    sqlRun.AddDecimal("NewTop", CDec(useText.Top))
                    sqlRun.AddDecimal("NewWidth", CDec(useText.Width))
                    sqlRun.AddDecimal("NewHeight", CDec(useText.Height))
                    sqlRun.AddInteger("NewRotation", useText.RotationAngle)
                    sqlRun.AddText("NewFont", useText.Font.Name)
                    sqlRun.AddDecimal("NewSize", CDec(useText.Font.SizeInPoints))
                    sqlRun.AddText("NewStyle", DBFontStyle(useText.Font))
                    sqlRun.AddText("NewText", useText.DisplayText)
                    sqlRun.AddLong("NewColor1", useText.TextColor.ToArgb())
                    sqlRun.AddLong("NewColor2")
                    sqlRun.AddInteger("NewAlign", useText.Alignment)
                    sqlRun.AddInteger("NewPad")
            End Select

            ' ----- Add the record.
            Try
                ExecuteSQL(sqlRun, transInfo)
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                TransactionRollback(transInfo)
                GeneralError("BarcodeLabel.ActOK_Click.Item", ex)
                Return
            End Try
        Next counter

        ' ----- Commit all changes.
        Try
            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            GeneralError("BarcodeLabel.ActOK_Click.Commit", ex)
            Return
        Finally
            Me.Cursor = Cursors.Default
        End Try

        ' ----- Continue with AddRecord() or EditRecord()
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub BarcodeLabel_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "BarcodeLabel.htm")
    End Sub
End Class
