﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class BarcodePrint
    Private Enum BarcodeOutput
        Item = 1
        Patron = 2
        Misc = 3
    End Enum

    Private PreviewMode As Boolean
    Private LabelsX As Integer
    Private LabelsY As Integer
    Private PageWidth As Integer
    Private PageHeight As Integer
    Private PageMarginTop As Integer
    Private PageMarginBottom As Integer
    Private PageMarginLeft As Integer
    Private PageMarginRight As Integer
    Private PageIntraColumn As Integer
    Private PageIntraRow As Integer
    Private PageLabelWidth As Integer
    Private PageLabelHeight As Integer
    Private PageSoFar As Integer
    Private LabelElements As Collections.ArrayList
    Private BarcodeFontName As String
    Private Barcode39 As Boolean
    Private WithEvents BarcodeDoc As Printing.PrintDocument

    Private Sub BarcodePrint_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "BarcodePrint.htm")
    End Sub

    Private Sub BarcodePrint_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Initialize.
        LabelsX = -1
        LabelsY = -1

        ' ----- Load in the list of number types.
        RecordType.Items.Add(New ListItemData("Item Barcodes", BarcodeOutput.Item))
        RecordType.Items.Add(New ListItemData("Patron Barcodes", BarcodeOutput.Patron))
        RecordType.Items.Add(New ListItemData("Miscellaneous Barcodes",
            BarcodeOutput.Misc))
        RecordType.SelectedIndex = 0

        ' ----- Load in the available page templates.
        RecordPage.Items.Add(New ListItemData("<Not Selected>", -1&))
        sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("BarcodePrint.BarcodePrint_Load", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            RecordPage.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
        RecordPage.SelectedIndex = 0

        ' ----- Don't load in the label templates at this time.
        RecordTemplate.Items.Add(New ListItemData("<Not Selected>", -1&))
        RecordTemplate.SelectedIndex = 0
    End Sub

    Private Sub RecordPage_SelectedIndexChanged(sender As Object, e As EventArgs) Handles RecordPage.SelectedIndexChanged
        ' ----- Adjust the display based on the selected page.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim whichPage As Long

        ' ----- Clear the previous details selection.
        RecordTemplate.Items.Clear()
        RecordTemplate.Items.Add(New ListItemData("<Not Selected>", -1&))
        RecordTemplate.SelectedIndex = 0
        RecordTemplate.Enabled = False
        LabelTemplate.Enabled = False
        RecordCountPerPage.Text = "No template selected."
        RefreshEndCount()

        ' ----- Check for no selection.
        If (RecordPage.SelectedIndex = -1) Then Return
        whichPage = CLng(DirectCast(RecordPage.SelectedItem, ListItemData))
        If (whichPage = -1&) Then Return
        Me.Cursor = Cursors.WaitCursor

        ' ----- Load in the count of the labels.
        sqlText = "SELECT ColumnsCount, RowsCount FROM BarcodeSheet " &
            "WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", whichPage)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("BarcodePrint.RecordPage_SelectedIndexChanged.Labels", ex)
            Return
        End Try
        If (dbInfo.Rows.Count = 0) Then
            Me.Cursor = Cursors.Default
            dbInfo.Dispose()
            Return
        End If
        oneRow = dbInfo.Rows(0)
        LabelsX = DBGetInteger(oneRow!ColumnsCount)
        LabelsY = DBGetInteger(oneRow!RowsCount)
        dbInfo.Dispose()

        ' ----- Update the count of the labels per sheet.
        If ((LabelsX * LabelsY) = 0) Then
            RecordCountPerPage.Text = "No labels per page."
        ElseIf ((LabelsX * LabelsY) = 1) Then
            RecordCountPerPage.Text = "1 label per page (1 by 1)"
        Else
            RecordCountPerPage.Text = (LabelsX * LabelsY).ToString() &
                " labels per page (" & LabelsX & " by " & LabelsY & ")"
        End If
        RefreshEndCount()

        ' ----- Load in the list of templates.
        sqlText = "SELECT ID, FullName FROM BarcodeLabel " &
            "WHERE BarcodeSheet = @RecordID ORDER BY FullName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", whichPage)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("BarcodePrint.RecordPage_SelectedIndexChanged.Templates", ex)
            Return
        End Try
        If (dbInfo.Rows.Count = 0) Then
            Me.Cursor = Cursors.Default
            dbInfo.Dispose()
            Return
        End If
        For Each dbRow As DataRow In dbInfo.Rows
            RecordTemplate.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
        RecordTemplate.Enabled = True
        LabelTemplate.Enabled = True
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub RecordType_SelectedIndexChanged(sender As Object, e As EventArgs) Handles RecordType.SelectedIndexChanged
        ' ----- Load in the starting value of the labels based on the selection.
        Dim startCount As Decimal
        Dim whichTarget As Integer

        On Error Resume Next

        ' ----- What type of counting will we use?
        If (RecordType.SelectedIndex = -1) Then Return
        whichTarget = CInt(CLng(DirectCast(RecordType.SelectedItem, ListItemData)))
        If (whichTarget = BarcodeOutput.Item) Then
            ' ----- Item labels.
            startCount = CDec(Val(GetSystemValue("NextBarcodeItem")))
        ElseIf (whichTarget = BarcodeOutput.Patron) Then
            ' ----- Patron labels.
            startCount = CDec(Val(GetSystemValue("NextBarcodePatron")))
        Else
            ' ----- Miscellaneous labels.
            startCount = CDec(Val(GetSystemValue("NextBarcodeMisc")))
        End If
        If (IsNumeric(startCount) = False) Then startCount = 1D
        If (Val(startCount) <= 0) Then startCount = 1D

        ' ----- Update the display.
        RecordStart.Text = startCount.ToString()
        RefreshEndCount()
    End Sub

    Private Sub RecordStart_TextChanged(sender As Object, e As EventArgs) Handles RecordStart.TextChanged
        ' ----- Update the ending value.
        RefreshEndCount()
    End Sub

    Private Sub RecordStart_Enter(sender As Object, e As EventArgs) Handles RecordStart.Enter
        ' ----- Highlight the entire text.
        RecordStart.SelectAll()
    End Sub

    Private Sub RecordPages_TextChanged(sender As Object, e As EventArgs) Handles RecordPages.TextChanged
        ' ----- Update the ending value.
        RefreshEndCount()
    End Sub

    Private Sub RecordPages_Enter(sender As Object, e As EventArgs) Handles RecordPages.Enter
        ' ----- Highlight the entire text.
        RecordPages.SelectAll()
    End Sub

    Private Sub RefreshEndCount()
        ' ----- Update the end count based on the start count.
        Dim endCount As Decimal

        On Error Resume Next

        ' ----- Assume no end point.
        RecordStop.Text = "Not defined."
        If (LabelsX <= 0) Or (LabelsY <= 0) Then Return
        If (IsNumeric(RecordStart.Text) = False) Then Return
        If (IsNumeric(RecordPages.Text) = False) Then Return

        ' ----- Update the display.
        endCount = CDec(RecordStart.Text) +
            (LabelsX * LabelsY * CDec(RecordPages.Text)) - 1
        RecordStop.Text = CStr(endCount)
    End Sub

    Private Function VerifyFields() As Boolean
        ' ----- Get the name of the barcode font.
        If (BarcodeFontName.Length = 0) Then
            BarcodeFontName = GetBarcodeFont()
            If (BarcodeFontName.Length = 0) Then Return False
            Barcode39 = CBool(Val(GetSystemValue("BarcodeCode39")))
        End If

        ' ----- A valid page template is required.
        If (CLng(DirectCast(RecordPage.SelectedItem, ListItemData)) = -1&) Then
            MessageBox.Show("Please select a Page Template.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordPage.Focus()
            Return False
        End If
        If (LabelsX <= 0) Or (LabelsY <= 0) Then
            MessageBox.Show("The selected Page Template will generate no valid labels.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordPage.Focus()
            Return False
        End If

        ' ----- A valid label template is required.
        If (CLng(DirectCast(RecordTemplate.SelectedItem, ListItemData)) = -1&) Then
            MessageBox.Show("Please select a Label Template.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordTemplate.Focus()
            Return False
        End If

        ' ----- The starting value must be valid.
        If (IsNumeric(RecordStart.Text) = False) Then
            MessageBox.Show("Start Value must be numeric.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordStart.Focus()
            Return False
        End If

        ' ----- The pages value must be valid.
        If (IsNumeric(RecordPages.Text) = False) Then
            MessageBox.Show("Pages must be numeric.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordPages.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActPreview_Click(sender As Object, e As EventArgs) Handles ActPreview.Click
        ' ----- The user wants to preview the labels.
        On Error Resume Next

        ' ----- Make sure the user supplied valid data.
        If (VerifyFields() = False) Then Return

        ' ----- Load in all of the page-specific details to be used in printing.
        If (LoadPageDetails() = False) Then Return

        ' ----- Create the preview dialog.
        Me.Cursor = Cursors.WaitCursor
        PageSoFar = 0
        PreviewMode = True
        BarcodeDoc = New Printing.PrintDocument

        ' ----- Display the preview.
        BarcodePreview.Document = BarcodeDoc
        BarcodePreview.ShowDialog()
        BarcodeDoc = Nothing
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub ActPrint_Click(sender As Object, e As EventArgs) Handles ActPrint.Click
        ' ----- The user wants to print the labels.
        Dim whichTarget As Integer

        On Error Resume Next

        ' ----- Make sure the user supplied valid data.
        If (VerifyFields() = False) Then Return

        ' ----- Load in all of the page-specific details to be used in printing.
        If (LoadPageDetails() = False) Then Return

        ' ----- Prompt to print the labels, and then print.
        Me.Cursor = Cursors.WaitCursor
        PageSoFar = 0
        PreviewMode = False
        BarcodeDoc = New Printing.PrintDocument
        BarcodePrintDialog.Document = BarcodeDoc
        If (BarcodePrintDialog.ShowDialog() <> DialogResult.OK) Then
            BarcodeDoc = Nothing
            Me.Cursor = Cursors.Default
            Return
        End If
        BarcodeDoc.Print()
        BarcodeDoc = Nothing
        Me.Cursor = Cursors.Default

        ' ----- Ask the user about the success of the print.
        If (MessageBox.Show("Did the labels print successfully?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) =
            DialogResult.No) Then Return

        ' ----- Update the database with the print values.
        whichTarget = CInt(CLng(DirectCast(RecordType.SelectedItem, ListItemData)))
        If (whichTarget = BarcodeOutput.Item) Then
            ' ----- Item labels.
            SetSystemValue("NextBarcodeItem", CStr(Val(RecordStop.Text) + 1))
        ElseIf (whichTarget = BarcodeOutput.Patron) Then
            ' ----- Patron labels.
            SetSystemValue("NextBarcodePatron", CStr(Val(RecordStop.Text) + 1))
        Else
            ' ----- Miscellaneous labels.
            SetSystemValue("NextBarcodeMisc", CStr(Val(RecordStop.Text) + 1))
        End If

        ' ----- Close the form.
        ActClose.PerformClick()
    End Sub

    Private Function LoadPageDetails() As Boolean
        ' ----- Retrieve the label formatting details from the database.
        '       Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim fromUnit As Char
        Dim newBarcodeImage As BarcodeItemBarcodeImage
        Dim newBarcodeText As BarcodeItemBarcodeNumber
        Dim newStaticText As BarcodeItemText
        Dim newStaticLine As BarcodeItemLine
        Dim newStaticRect As BarcodeItemRect

        Me.Cursor = Cursors.WaitCursor

        ' ----- Load in the page structure details. Store all values in Twips.
        sqlText = "SELECT * FROM BarcodeSheet WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", CLng(DirectCast(RecordPage.SelectedItem, ListItemData)))
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("BarcodePrint.LoadPageDetails.Sheet", ex)
            Return False
        End Try
        oneRow = dbInfo.Rows(0)
        fromUnit = CChar(DBGetText(oneRow!UnitType))
        PageWidth = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(oneRow!PageWidth))))
        PageHeight = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(oneRow!PageHeight))))
        PageMarginTop = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(oneRow!MarginTop))))
        PageMarginBottom = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(oneRow!MarginBottom))))
        PageMarginLeft = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(oneRow!MarginLeft))))
        PageMarginRight = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(oneRow!MarginRight))))
        PageIntraColumn = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(oneRow!IntraColumn))))
        PageIntraRow = CInt(ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(oneRow!IntraRow))))
        dbInfo.Dispose()
        PageLabelWidth = CInt((PageWidth - PageMarginLeft - PageMarginRight -
            ((LabelsX - 1) * PageIntraColumn)) / LabelsX)
        PageLabelHeight = CInt((PageHeight - PageMarginTop - PageMarginBottom -
            ((LabelsY - 1) * PageIntraRow)) / LabelsY)

        ' ----- Get the units to be used for label items.
        sqlText = "SELECT UnitType FROM BarcodeLabel WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", CLng(DirectCast(RecordTemplate.SelectedItem, ListItemData)))
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("BarcodePrint.LoadPageDetails.Units", ex)
            Return False
        End Try
        oneRow = dbInfo.Rows(0)
        fromUnit = CChar(DBGetText(oneRow!UnitType))
        dbInfo.Dispose()

        ' ----- Load in the items that appear on each label.
        LabelElements = New ArrayList
        sqlText = "SELECT * FROM BarcodeLabelItem " &
            "WHERE BarcodeLabel = @RecordID ORDER BY Priority"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", CLng(DirectCast(RecordTemplate.SelectedItem, ListItemData)))
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("BarcodePrint.LoadPageDetails.Items", ex)
            Return False
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Process this object.
            Select Case DBGetText(dbRow!ItemType).ToUpper()
                Case "B"
                    ' ----- Barcode image.
                    newBarcodeImage = New BarcodeItemBarcodeImage
                    newBarcodeImage.Alignment = CType(DBGetInteger(dbRow!Alignment),
                        ContentAlignment)
                    newBarcodeImage.BarcodeColor =
                        Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newBarcodeImage.BarcodeSize = CSng(DBGetDecimal(dbRow!FontSize))
                    newBarcodeImage.Left = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosLeft)))
                    newBarcodeImage.Top = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosTop)))
                    newBarcodeImage.Width = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosWidth)))
                    newBarcodeImage.Height = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosHeight)))
                    newBarcodeImage.RotationAngle = CShort(DBGetInteger(dbRow!Rotation))
                    newBarcodeImage.PadDigits = CByte(DBGetInteger(dbRow!PadDigits))
                    LabelElements.Add(newBarcodeImage)
                Case "L"
                    ' ----- Line.
                    newStaticLine = New BarcodeItemLine
                    newStaticLine.LineColor =
                        Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newStaticLine.X1 = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosLeft)))
                    newStaticLine.X2 = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosWidth)))
                    newStaticLine.Y1 = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosTop)))
                    newStaticLine.Y2 = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosHeight)))
                    LabelElements.Add(newStaticLine)
                Case "N"
                    ' ----- Barcode text.
                    newBarcodeText = New BarcodeItemBarcodeNumber
                    newBarcodeText.Alignment = CType(dbRow!Alignment,
                        ContentAlignment)
                    newBarcodeText.Left = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosLeft)))
                    newBarcodeText.Top = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosTop)))
                    newBarcodeText.Width = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosWidth)))
                    newBarcodeText.Height = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosHeight)))
                    newBarcodeText.RotationAngle = CShort(DBGetInteger(dbRow!Rotation))
                    newBarcodeText.TextColor =
                        Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newBarcodeText.Font = New Font(DBGetText(dbRow!FontName),
                        CSng(DBGetDecimal(dbRow!FontSize)), BuildFontStyle(DBGetText(dbRow!FontStyle)))
                    newBarcodeText.PadDigits = CByte(DBGetInteger(dbRow!PadDigits))
                    LabelElements.Add(newBarcodeText)
                Case "R"
                    ' ----- Rectangle.
                    newStaticRect = New BarcodeItemRect
                    newStaticRect.BorderColor =
                        Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newStaticRect.FillColor =
                        Color.FromArgb(DBGetInteger(dbRow!Color2))
                    newStaticRect.Left =
                        ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(dbRow!PosLeft)))
                    newStaticRect.Top =
                        ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(dbRow!PosTop)))
                    newStaticRect.Width =
                        ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(dbRow!PosWidth)))
                    newStaticRect.Height =
                        ConvertPageUnits(fromUnit, "T"c, CSng(DBGetDecimal(dbRow!PosHeight)))
                    newStaticRect.RotationAngle = CShort(DBGetInteger(dbRow!Rotation))
                    LabelElements.Add(newStaticRect)
                Case "T"
                    ' ----- Static text.
                    newStaticText = New BarcodeItemText
                    newStaticText.DisplayText = DBGetText(dbRow!StaticText)
                    newStaticText.Alignment = CType(dbRow!Alignment,
                        ContentAlignment)
                    newStaticText.Left = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosLeft)))
                    newStaticText.Top = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosTop)))
                    newStaticText.Width = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosWidth)))
                    newStaticText.Height = ConvertPageUnits(fromUnit, "T"c,
                        CSng(DBGetDecimal(dbRow!PosHeight)))
                    newStaticText.RotationAngle = CShort(DBGetInteger(dbRow!Rotation))
                    newStaticText.TextColor =
                        Color.FromArgb(DBGetInteger(dbRow!Color1))
                    newStaticText.Font = New Font(DBGetText(dbRow!FontName),
                        CSng(DBGetDecimal(dbRow!FontSize)), BuildFontStyle(DBGetText(dbRow!FontStyle)))
                    LabelElements.Add(newStaticText)
            End Select
        Next dbRow
        dbInfo.Dispose()

        ' ----- Success.
        Me.Cursor = Cursors.Default
        Return True
    End Function

    Private Sub BarcodeDoc_PrintPage(sender As Object, e As Printing.PrintPageEventArgs) Handles BarcodeDoc.PrintPage
        ' ----- Print one page of labels.
        Dim pixelPen As Pen
        Dim rowScan As Integer
        Dim colScan As Integer
        Dim leftOffset As Integer
        Dim topOffset As Integer
        Dim hardLeft As Integer
        Dim hardTop As Integer
        Dim barcodeNumber As Integer
        Dim holdState As Drawing2D.GraphicsState

        ' ----- Display the outline of labels.
        If (PreviewMode = True) Then
            pixelPen = New Pen(Color.Cyan, 1 / e.Graphics.DpiX)
            For rowScan = 1 To LabelsY
                For colScan = 1 To LabelsX
                    leftOffset = CInt(ConvertPageUnits("T"c, "H"c, PageMarginLeft +
                        ((colScan - 1) * (PageIntraColumn + PageLabelWidth))))
                    topOffset = CInt(ConvertPageUnits("T"c, "H"c, PageMarginTop +
                        ((rowScan - 1) * (PageIntraRow + PageLabelHeight))))
                    e.Graphics.DrawRectangle(Pens.Cyan, leftOffset, topOffset,
                        ConvertPageUnits("T"c, "H"c, PageLabelWidth),
                        ConvertPageUnits("T"c, "H"c, PageLabelHeight))
                Next colScan
            Next rowScan
            pixelPen.Dispose()
        End If

        ' ----- Are we printing to a real printer? If so, we need to take into
        '       account the unprintable area. In preview mode, the VisibleClipBound
        '       has a really huge width and height.
        If (e.PageBounds.Width > e.Graphics.VisibleClipBounds.Width) Or
                (PreviewMode = False) Then
            ' ----- Real print.
            hardLeft = CInt(ConvertPageUnits("H"c, "T"c,
                (e.PageBounds.Width - e.Graphics.VisibleClipBounds.Width) / 2.0F))
            hardTop = CInt(ConvertPageUnits("H"c, "T"c,
                (e.PageBounds.Height - e.Graphics.VisibleClipBounds.Height) / 2.0F))
        Else
            ' ----- Print preview.
            hardLeft = 0
            hardTop = 0
        End If

        ' ----- Determine the starting barcode number on this page.
        barcodeNumber = CInt(Val(RecordStart.Text) + (PageSoFar * (LabelsX * LabelsY)))

        ' ----- Process each label.
        For rowScan = 1 To LabelsY
            For colScan = 1 To LabelsX
                ' ----- Find the upper-left corner of the label.
                leftOffset = PageMarginLeft + ((colScan - 1) *
                    (PageIntraColumn + PageLabelWidth)) - hardLeft
                topOffset = PageMarginTop + ((rowScan - 1) *
                    (PageIntraRow + PageLabelHeight)) - hardTop

                ' ----- Save the current graphics state. Move into twips mode, and
                '       position the origin to the upper-left corner of the label.
                holdState = e.Graphics.Save()
                e.Graphics.ScaleTransform(100.0F / 1440.0F, 100.0F / 1440.0F)
                e.Graphics.TranslateTransform(leftOffset, topOffset)

                ' ----- Print the label.
                PrintOneLabel(e.Graphics, barcodeNumber)
                barcodeNumber += 1

                ' ----- Restore the original graphics state.
                e.Graphics.Restore(holdState)
            Next colScan
        Next rowScan

        ' ----- Move to the next page.
        PageSoFar += 1
        If (PageSoFar >= CInt(RecordPages.Text)) Then
            ' ----- If this was preview mode, the user might want to click on
            '       the "Print" button included on the preview dialog. Since
            '       we print preview screens differently, we need to ensure
            '       the a true print through this button will print a non-
            '       preview set of labels.
            PreviewMode = False
            e.HasMorePages = False
        Else
            e.HasMorePages = True
        End If
    End Sub

    Private Sub PrintOneLabel(g As Graphics, barcodeNumber As Decimal)
        ' ----- Print out one barcode label.
        Dim counter As Integer
        Dim textMessage As String
        Dim useGeneric As BarcodeItemGeneric
        Dim useBarImage As BarcodeItemBarcodeImage
        Dim useBarText As BarcodeItemBarcodeNumber
        Dim useText As BarcodeItemText
        Dim useLine As BarcodeItemLine
        Dim useRect As BarcodeItemRect
        Dim useFont As Font
        Dim holdState As Drawing2D.GraphicsState
        Dim pixelPen As Pen
        Dim pixelBrush As SolidBrush
        Dim textColor As Color
        Dim textAngle As Integer
        Dim textAlign As ContentAlignment
        Dim textFormat As New StringFormat
        Dim X1 As Single
        Dim X2 As Single
        Dim Y1 As Single
        Dim Y2 As Single
        Dim textSize As SizeF
        Dim scanLabel As IEnumerator

        On Error Resume Next

        ' ----- NOTE: The calling procedure has adjusted the graphics display.
        '       All output is measured in twips, and the origin is at the upper-
        '       left corner of the label to be printed.

        ' ----- Process each item in the list.
        scanLabel = LabelElements.GetEnumerator()
        Do While scanLabel.MoveNext()
            ' ----- Save the current state of the graphics area.
            holdState = g.Save()

            ' ----- Different code for each type of drawing object.
            useGeneric = DirectCast(scanLabel.Current, BarcodeItemGeneric)
            Select Case useGeneric.ItemType
                Case "B"
                    ' ----- Barcode image.
                    useBarImage = DirectCast(useGeneric, BarcodeItemBarcodeImage)
                    textMessage = CStr(barcodeNumber)
                    If (textMessage.Length < useBarImage.PadDigits) Then
                        textMessage = New String("0"c, useBarImage.PadDigits -
                            textMessage.Length) & textMessage
                    End If
                    If (Barcode39 = True) Then textMessage = "*" & textMessage & "*"
                    textColor = useBarImage.BarcodeColor
                    textAngle = useBarImage.RotationAngle
                    textAlign = useBarImage.Alignment

                    ' ----- Get the font to use.
                    useFont = New Font(BarcodeFontName,
                        CSng(useBarImage.BarcodeSize * 14.4),
                        FontStyle.Regular)

                    ' ----- Get the coordinates in twips.
                    X1 = useBarImage.Left
                    X2 = useBarImage.Width
                    Y1 = useBarImage.Top
                    Y2 = useBarImage.Height

                    ' ----- The rest is the same as drawing normal text.
                    GoTo DisplayTheText
                Case "L"
                    ' ----- Line.
                    useLine = DirectCast(useGeneric, BarcodeItemLine)

                    ' ----- Create a one-pixel pen to use.
                    pixelPen = New Pen(useLine.LineColor, 1 / g.DpiX)

                    ' ----- Get the coordinates in twips.
                    X1 = useLine.X1
                    X2 = useLine.X2
                    Y1 = useLine.Y1
                    Y2 = useLine.Y2

                    ' ----- Draw the line.
                    g.DrawLine(pixelPen, X1, Y1, X2, Y2)
                    pixelPen.Dispose()
                Case "N"
                    ' ----- Barcode text.
                    useBarText = DirectCast(useGeneric, BarcodeItemBarcodeNumber)
                    textMessage = CStr(barcodeNumber)
                    If (textMessage.Length < useBarText.PadDigits) Then
                        textMessage = New String("0"c, useBarText.PadDigits -
                            textMessage.Length) & textMessage
                    End If
                    textColor = useBarText.TextColor
                    textAngle = useBarText.RotationAngle
                    textAlign = useBarText.Alignment

                    ' ----- Get the font to use.
                    useFont = New Font(useBarText.Font.Name,
                        CSng(useBarText.Font.SizeInPoints * 14.4), useBarText.Font.Style)

                    ' ----- Get the coordinates in twips.
                    X1 = useBarText.Left
                    X2 = useBarText.Width
                    Y1 = useBarText.Top
                    Y2 = useBarText.Height

                    ' ----- The rest is the same as drawing normal text.
                    GoTo DisplayTheText
                Case "R"
                    ' ----- Rectangle.
                    useRect = DirectCast(useGeneric, BarcodeItemRect)

                    ' ----- Create a one-pixel pen to use.
                    pixelPen = New Pen(useRect.BorderColor, 1 / g.DpiX)
                    pixelBrush = New SolidBrush(useRect.FillColor)

                    ' ----- Get the coordinates in twips.
                    X1 = useRect.Left
                    X2 = useRect.Width
                    Y1 = useRect.Top
                    Y2 = useRect.Height

                    ' ----- Rotate the surface if requested.
                    If (useRect.RotationAngle <> 0) Then
                        g.TranslateTransform(X1, Y1)
                        g.RotateTransform(useRect.RotationAngle)
                        X1 = 0.0F
                        Y1 = 0.0F
                    End If

                    ' ----- Draw the rectangle.
                    g.FillRectangle(pixelBrush, X1, Y1, X2, Y2)
                    g.DrawRectangle(pixelPen, X1, Y1, X2, Y2)
                    pixelBrush.Dispose()
                    pixelPen.Dispose()
                Case "T"
                    ' ----- Text.
                    useText = DirectCast(useGeneric, BarcodeItemText)
                    textMessage = useText.DisplayText
                    textColor = useText.TextColor
                    textAngle = useText.RotationAngle
                    textAlign = useText.Alignment

                    ' ----- Get the font to use.
                    useFont = New Font(useText.Font.Name,
                        CSng(useText.Font.SizeInPoints * 14.4), useText.Font.Style)

                    ' ----- Get the coordinates in points.
                    X1 = useText.Left
                    X2 = useText.Width
                    Y1 = useText.Top
                    Y2 = useText.Height

DisplayTheText:
                    ' ----- Rotate around the upper-left corner of the text box
                    '       if needed.
                    If (textAngle <> 0) Then
                        g.TranslateTransform(X1, Y1)
                        g.RotateTransform(textAngle)
                        X1 = 0.0F
                        Y1 = 0.0F
                    End If

                    ' ----- Set the clipping region.
                    g.SetClip(New RectangleF(X1, Y1, X2, Y2))

                    ' ----- Position the text within the bounding box.
                    Select Case textAlign
                        Case ContentAlignment.TopLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomLeft
                            textFormat.Alignment = StringAlignment.Near
                            textFormat.LineAlignment = StringAlignment.Far
                        Case ContentAlignment.TopCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomCenter
                            textFormat.Alignment = StringAlignment.Center
                            textFormat.LineAlignment = StringAlignment.Far
                        Case ContentAlignment.TopRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Near
                        Case ContentAlignment.MiddleRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Center
                        Case ContentAlignment.BottomRight
                            textFormat.Alignment = StringAlignment.Far
                            textFormat.LineAlignment = StringAlignment.Far
                    End Select
                    textFormat.FormatFlags = textFormat.FormatFlags Or
                        StringFormatFlags.NoWrap

                    ' ----- Print the text.
                    g.DrawString(textMessage, useFont,
                        New SolidBrush(textColor),
                        New RectangleF(X1, Y1, X2, Y2), textFormat)
                    useFont.Dispose()
            End Select

            ' ----- Restore the original transformed state of the graphics surface.
            g.Restore(holdState)
        Loop
    End Sub
End Class