﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class Author
    Private ActiveID As Long
    Private StartingOver As Boolean

    Public Overloads Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overloads Function AddRecord(lastText As String, firstText As String) As Long
        ' ----- Add a new record, but use a starting value previously entered
        '       by the user.
        ActiveID = -1&
        PrepareFormFields()
        RecordLastName.Text = lastText
        RecordFirstName.Text = firstText
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageAuthors)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageAuthors)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageAuthors)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the author?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM ItemAuthor WHERE AuthorID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("You cannot delete this record because it is in use.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        Catch ex As Exception
            GeneralError("Author.DeleteRecord.Check", ex)
            Return False
        End Try

        ' ----- Delete the record.
        sqlText = "DELETE FROM Author WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("Author.DeleteRecord.Delete", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items. Prompt the
        '       user first.
        destList.Items.Clear()
        SearchForRecord(destList, exceededMatches)
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim result As String

        ' ----- Initialize.
        result = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT * FROM Author WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Author.FormatRecordName", ex)
            Return "Error"
        End Try
        If (dbInfo.Rows.Count > 0) Then
            ' ----- Format the display name.
            result = dbInfo.Rows(0).FormatAuthorName()
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
        Return result
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use this form to enter authors, editors, performers, and " &
            "others that are primarily responsible for the content of an item " &
            "in the library."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Authors and Names"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim dbRow As DataRow

        ' ----- Clear the existing data.
        StartingOver = True
        RecordLastName.Text = ""
        RecordFirstName.Text = ""
        RecordMiddleName.Text = ""
        RecordSuffix.Text = ""
        RecordBirthYear.Text = ""
        RecordDeathYear.Text = ""
        RecordComments.Text = ""
        ShowMatches.Text = "      None"
        ShowMatches.ImageIndex = MatchNone
        LabelMatches.Visible = True
        ShowMatches.Visible = True
        If (ActiveID = -1&) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM Author WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Author.PrepareFormFields", ex)
            Return
        End Try
        If (dbInfo.Rows.Count > 0) Then
            dbRow = dbInfo.Rows(0)
            RecordLastName.Text = DBGetText(dbRow!LastName)
            RecordFirstName.Text = DBGetText(dbRow!FirstName)
            RecordMiddleName.Text = DBGetText(dbRow!MiddleName)
            RecordSuffix.Text = DBGetText(dbRow!Suffix)
            If (IsDBNull(dbRow!BirthYear) = False) Then _
				RecordBirthYear.Text = DBGetInteger(dbRow!BirthYear).ToString()
            If (IsDBNull(dbRow!DeathYear) = False) Then _
				RecordDeathYear.Text = DBGetInteger(dbRow!DeathYear).ToString()
            RecordComments.Text = DBGetText(dbRow!Comments)
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
        LabelMatches.Visible = False
        ShowMatches.Visible = False
    End Sub

    Private Sub RefreshMatchingAuthors()
        ' ----- See if there are name matches for this author.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim matchCount As Integer
        Dim messageText As String

        ' ----- Only performed for new records.
        If (ActiveID <> -1&) Then Return

        ' ----- Reset the display.
        ShowMatches.Text = "      None"
        ShowMatches.ImageIndex = MatchNone

        ' ----- Ignore if we have insufficient data.
        If (RecordLastName.Text.Trim().Length = 0) Then Return

        ' ----- Retrieve a count of matching records.
        sqlText = "SELECT COUNT(*) AS TheCount FROM Author " &
            "WHERE LastName LIKE @TestLast"
        If (RecordFirstName.Text.Trim().Length <> 0) Then _
            sqlText &= " AND FirstName LIKE @TestFirst"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestLast", RecordLastName.Text.Trim())
        If (RecordFirstName.Text.Trim().Length <> 0) Then _
            sqlRun.AddText("TestFirst", RecordFirstName.Text.Trim().Substring(0, 1) & "%")
        Try
            matchCount = DBGetInteger(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("Author.RefreshMatchingAuthors", ex)
            Return
        End Try
        If (matchCount > 0) Then
            ' ----- There are matches.
            If (matchCount = 1) Then
                messageText = "1 match"
            Else
                messageText = matchCount & " matches"
            End If
            messageText &= ". Click for details."
            ShowMatches.Text = "      " & messageText
            ShowMatches.ImageIndex = MatchPresent
        End If
    End Sub

    Public Overrides Sub SearchForRecord(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Search for authors using user-specified criteria.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim userLimit As String
        Dim limitFirst As String
        Dim limitLast As String
        Dim hasWhere As Boolean
        Dim resultCount As Integer

        ' ----- Prompt the user for the limited author name.
        exceededMatches = False
        userLimit = (New AuthorLimit).PromptUser()
        If (userLimit.Length = 0) Then Return

        ' ----- Use the limits to help prepare the search text.
        limitLast = userLimit.GetSubStr(",", 1).Trim()
        limitFirst = userLimit.GetSubStr(",", 2).Trim()
        If ((limitLast & limitFirst).Length = 0) Then Return
        If (limitLast.Contains("*") = False) Then limitLast &= "*"
        If (limitFirst.Contains("*") = False) Then limitFirst &= "*"
        limitLast = limitLast.Replace("*", "%")
        limitFirst = limitFirst.Replace("*", "%")

        ' ----- Clear any existing values.
        destList.Items.Clear()

        ' ----- Prepare the full SQL statement based on the user's limits.
        hasWhere = False
        sqlText = "SELECT * FROM Author"
        If (limitLast.Length <> 0) Then
            sqlText &= " WHERE LastName LIKE @TestLast"
            hasWhere = True
        End If
        If (limitFirst.Length <> 0) Then
            If (hasWhere) Then sqlText &= " AND" Else sqlText &= " WHERE"
            sqlText &= " FirstName LIKE @TestFirst"
        End If
        sqlText &= " ORDER BY LastName, FirstName"

        ' ----- Retrieve and process the values from the database.
        sqlRun = New DataCommand(sqlText)
        If (limitLast.Length <> 0) Then _
            sqlRun.AddText("TestLast", limitLast)
        If (limitFirst.Length <> 0) Then _
            sqlRun.AddText("TestFirst", limitFirst)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Author.SearchForRecord", ex)
            Return
        End Try

        resultCount = 0
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add this item.
            resultCount += 1
            If (resultCount > SearchMatchLimit) Then
                ' ----- Too many matches found.
                exceededMatches = True
                Exit For
            End If
            destList.Items.Add(New ListItemData(
                dbRow.FormatAuthorName(), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function UsesSearch() As Boolean
        ' ----- Author records are searchable.
        Return True
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save the record to the database.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long

        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO Author (LastName, FirstName, MiddleName, " &
                "Suffix, BirthYear, DeathYear, Comments) OUTPUT INSERTED.ID " &
                "VALUES (@NewLast, @NewFirst, @NewMiddle, @NewSuffix, " &
                "@NewBirth, @NewDeath, @NewComments)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewLast", RecordLastName.Text.Trim())
            sqlRun.AddText("NewFirst", RecordFirstName.Text.Trim())
            sqlRun.AddText("NewMiddle", RecordMiddleName.Text.Trim())
            sqlRun.AddText("NewSuffix", RecordSuffix.Text.Trim())
            sqlRun.AddInteger("NewBirth", RecordBirthYear.Text)
            sqlRun.AddInteger("NewDeath", RecordDeathYear.Text)
            sqlRun.AddText("NewComments", RecordComments.Text.Trim())
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                GeneralError("Author.SaveFormData.Add", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE Author SET LastName = @NewLast, " &
                "FirstName = @NewFirst, MiddleName = @NewMiddle, " &
                "Suffix = @NewSuffix, BirthYear = @NewBirth, " &
                "DeathYear = @NewDeath, Comments = @NewComments " &
                "WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewLast", RecordLastName.Text.Trim())
            sqlRun.AddText("NewFirst", RecordFirstName.Text.Trim())
            sqlRun.AddText("NewMiddle", RecordMiddleName.Text.Trim())
            sqlRun.AddText("NewSuffix", RecordSuffix.Text.Trim())
            sqlRun.AddInteger("NewBirth", RecordBirthYear.Text)
            sqlRun.AddInteger("NewDeath", RecordDeathYear.Text)
            sqlRun.AddText("NewComments", RecordComments.Text.Trim())
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("Author.SaveFormData.Update", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        '       Last name is a required field.
        If (RecordLastName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the last name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordLastName.Focus()
            Return False
        End If

        ' ----- Birth year, if supplied, must be valid.
        If (RecordBirthYear.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordBirthYear.Text) = False) Or
                    (RecordBirthYear.Text.Contains(".") = True) Then
                MessageBox.Show("Year of Birth, if supplied, must be a valid integer.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordBirthYear.Focus()
                Return False
            End If
        End If

        ' ----- Death year, if supplied, must be valid.
        If (RecordDeathYear.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordDeathYear.Text) = False) Or
                    (RecordDeathYear.Text.Contains(".") = True) Then
                MessageBox.Show("Year of Death, if supplied, must be a valid integer.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordDeathYear.Focus()
                Return False
            End If
        End If

        ' ----- Check the matching authors, just in case.
        If (ActiveID = -1&) Then
            RefreshMatchingAuthors()
            If (ShowMatches.ImageIndex = MatchPresent) Then
                If (MessageBox.Show("The name you supplied is similar to one or more names " &
                    "already in the database. Do you still want to save this record?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
                    DialogResult.Yes) Then Return False
            End If
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub Author_Activated(sender As Object, e As EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the first field.
        If (StartingOver) Then RecordLastName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordLastName_Enter(sender As Object, e As EventArgs) Handles RecordLastName.Enter
        ' ----- Highlight the entire text.
        RecordLastName.SelectAll()
    End Sub

    Private Sub RecordFirstName_Enter(sender As Object, e As EventArgs) Handles RecordFirstName.Enter
        ' ----- Highlight the entire text.
        RecordFirstName.SelectAll()
    End Sub

    Private Sub RecordMiddleName_Enter(sender As Object, e As EventArgs) Handles RecordMiddleName.Enter
        ' ----- Highlight the entire text.
        RecordMiddleName.SelectAll()
    End Sub

    Private Sub RecordSuffix_Enter(sender As Object, e As EventArgs) Handles RecordSuffix.Enter
        ' ----- Highlight the entire text.
        RecordSuffix.SelectAll()
    End Sub

    Private Sub RecordBirthYear_Enter(sender As Object, e As EventArgs) Handles RecordBirthYear.Enter
        ' ----- Highlight the entire text.
        RecordBirthYear.SelectAll()
    End Sub

    Private Sub RecordDeathYear_Enter(sender As Object, e As EventArgs) Handles RecordDeathYear.Enter
        ' ----- Highlight the entire text.
        RecordDeathYear.SelectAll()
    End Sub

    Private Sub RecordBirthYear_KeyPress(sender As Object, e As KeyPressEventArgs) Handles RecordBirthYear.KeyPress
        ' ----- Only allow digits, hyphens, and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Or (e.KeyChar = "-") Then Return
        e.Handled = True
    End Sub

    Private Sub RecordDeathYear_KeyPress(sender As Object, e As KeyPressEventArgs) Handles RecordDeathYear.KeyPress
        ' ----- Only allow digits, hyphens, and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Or (e.KeyChar = "-") Then Return
        e.Handled = True
    End Sub

    Private Sub RecordLastName_Leave(sender As Object, e As EventArgs) Handles RecordLastName.Leave
        ' ----- Update the matching display.
        RefreshMatchingAuthors()
    End Sub

    Private Sub RecordFirstName_Leave(sender As Object, e As EventArgs) Handles RecordFirstName.Leave
        ' ----- Update the matching display.
        RefreshMatchingAuthors()
    End Sub

    Private Sub ShowMatches_Click(sender As Object, e As EventArgs) Handles ShowMatches.Click
        ' ----- The user wants more information on the matches.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim nameList As String
        Dim message As String
        Dim matches As Integer
        Dim counter As Integer

        ' ----- Only performed for new records.
        If (ActiveID <> -1&) Then Return

        ' ----- Ignore if there are no matches.
        If (ShowMatches.ImageIndex = MatchNone) Then
            MessageBox.Show("No matches.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
            Return
        End If

        ' ----- Retrieve a count of matching records.
        matches = CInt(Val(ShowMatches.Text))

        ' ----- Retrieve the matching records.
        sqlText = "SELECT * FROM Author WHERE LastName LIKE @TestLast"
        If (RecordFirstName.Text.Trim().Length <> 0) Then _
            sqlText &= " AND FirstName LIKE @TestFirst"
        sqlText &= " ORDER BY LastName, FirstName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestLast", RecordLastName.Text.Trim())
        If (RecordFirstName.Text.Trim().Length <> 0) Then _
            sqlRun.AddText("TestFirst", RecordFirstName.Text.Trim().Substring(0, 1) & "%")
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Author.ShowMatches_Click", ex)
            Return
        End Try

        counter = 0
        nameList = ""
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add this author.
            nameList &= vbCrLf & vbTab & dbRow.FormatAuthorName()

            ' ----- Limit to ten matches.
            counter += 1
            If (counter >= 10) Then Exit For
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Format and display the message.
        message = "The database contains " & matches.ToString() & " match"
        If (matches > 1) Then message &= "es"
        message &= " to the name:" & vbCrLf & vbCrLf & vbTab &
            RecordLastName.Text.Trim()
        If (RecordFirstName.Text.Trim().Length <> 0) Then message &= ", " &
            RecordFirstName.Text.Trim()
        If (matches > 10) Then
            message &= vbCrLf & vbCrLf & "The first ten matches are:"
        Else
            message &= vbCrLf & vbCrLf & "The matches are:"
        End If
        message &= vbCrLf & nameList
        MessageBox.Show(message,
            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    Private Sub LabelMatches_Click(sender As Object, e As EventArgs) Handles LabelMatches.Click
        ' ----- Same as a click on the related data label.
        ShowMatches_Click(sender, e)
    End Sub

    Private Sub Author_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "Author.htm")
    End Sub
End Class
