﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class ChangeUser
    Private Sub AdjustOption()
        ' ----- Adjust the option setting based on the user entry.
        OptionPatron.Checked = CBool(LoginID.Text.Trim().Length = 0)
        OptionAdmin.Checked = Not OptionPatron.Checked
    End Sub

    Private Sub LoginID_Enter(sender As Object, e As EventArgs) Handles LoginID.Enter
        ' ----- Highlight the entire text.
        LoginID.SelectAll()
    End Sub

    Private Sub Password_Enter(sender As Object, e As EventArgs) Handles Password.Enter
        ' ----- Highlight the entire text.
        Password.SelectAll()
    End Sub

    Private Sub LoginID_TextChanged(sender As Object, e As EventArgs) Handles LoginID.TextChanged
        ' ----- Refresh the option.
        AdjustOption()
    End Sub

    Private Sub Password_TextChanged(sender As Object, e As EventArgs) Handles Password.TextChanged
        ' ----- Refresh the option.
        AdjustOption()
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- Accept the user's input.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim dbRow As DataRow
        Dim comparePassword As String

        ' ----- Different code for patron or user mode.
        If (OptionPatron.Checked = True) Then
            ' ----- Patron mode is easy.
            LoggedInUserID = -1&
            LoggedInUserName = ""
            LoggedInGroupID = -1&
            ReprocessSecuritySet()
            Me.DialogResult = DialogResult.OK
            Return
        End If

        ' ----- Check for a valid entry.
        If (LoginID.Text.Trim().Length = 0) Then
            MessageBox.Show(
                "To connect as an administrative user, you must provide a valid login ID.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Check the user name and password.
        sqlText = "SELECT * FROM UserName WHERE UPPER(LoginID) = @TestLogin"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestLogin", LoginID.Text.Trim().ToUpper())
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("ChangeUser.ActOK_Click", ex)
            Return
        End Try

        If (dbInfo.Rows.Count = 0) Then
            MessageBox.Show("Invalid login ID or password.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            dbInfo.Dispose()
            dbInfo = Nothing
            Return
        End If
        dbRow = dbInfo.Rows(0)

        ' ----- Check for an inactive user.
        If (DBGetBool(dbRow!Active) = False) Then
            MessageBox.Show("The administrator has disabled your account.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            dbInfo.Dispose()
            dbInfo = Nothing
            Return
        End If

        ' ----- Check the password.
        If (IsDBNull(dbRow!Password) = True) Then
            comparePassword = EncryptPassword(LoginID.Text.Trim(), "")
        Else
            comparePassword = DBGetText(dbRow!Password)
        End If
        If (EncryptPassword(LoginID.Text.Trim(), Password.Text.Trim()) <>
                comparePassword) Then
            MessageBox.Show("Invalid login ID or password.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            dbInfo.Dispose()
            dbInfo = Nothing
            Return
        End If

        ' ----- Don't allow the login if the program is unlicensed.
        Do While (ExamineLicense().Status <> LicenseStatus.ValidLicense)
            ' ----- Ask the user what to do.
            If (MessageBox.Show("This application is not properly licensed for " &
                    "administrative use. If you have access to a valid license " &
                    "file, you can verify it now. Would you like to locate " &
                    "a valid license file at this time?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
                    DialogResult.Yes) Then
                dbInfo.Dispose()
                dbInfo = Nothing
                Return
            End If

            ' ----- Prompt for an updated license.
            Call LocateLicense.ChangeLicense()
            LocateLicense = Nothing
        Loop

        ' ----- Successful login.
        LoggedInUserID = DBGetLong(dbRow!ID)
        LoggedInUserName = DBGetText(dbRow!LoginID)
        LoggedInGroupID = DBGetLong(dbRow!GroupID)
        dbInfo.Dispose()
        dbInfo = Nothing
        ReprocessSecuritySet()
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub ChangeUser_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "ChangeUser.htm")
    End Sub
End Class