﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class CheckLookup
    Private UseLC As Boolean
    Private SecondItemRow As Integer
    Private CheckInMode As Boolean

    Private Class CheckItemDetail
        ' ----- Used instead of ListItemData, with extra fields.
        Public TitleText As String = ""
        Public AuthorName As String = ""
        Public MultiAuthor As Boolean = False
        Public MediaType As String = ""
        Public CallNumber As String = "-----"
        Public Barcode As String = "-----"
        Public CopyNumber As Integer = -1
        Public ItemCopyID As Long = -1&
    End Class

    Public Function CheckItemByTitle(checkIn As Boolean, searchText As String) As Long
        ' ----- Prompt the user for a title item. Return the item copy
        '       ID on success, or -1 if no item is selected.
        Dim itemDetail As CheckItemDetail

        ' ----- Determine if Dewey or LC is used, and other initialization.
        UseLC = CBool(Val(GetSystemValue("UseLC")))

        ' ----- Perform the first search.
        If (PerformLookup(checkIn, searchText) = False) Then
            MessageBox.Show("Your search resulted in no matches.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
            Return -1&
        End If
        CheckInMode = checkIn

        ' ----- Show the results to the user.
        If (Me.ShowDialog() = DialogResult.Cancel) Then Return -1&

        ' ----- Extract the copy ID number and return it.
        itemDetail = DirectCast(MatchingItems.SelectedItem, CheckItemDetail)
        Return itemDetail.ItemCopyID
    End Function

    Private Function PerformLookup(asCheckIn As Boolean, searchText As String) As Boolean
        ' ----- Perform a check-in or check-out title search. Return True
        '       if matches were found.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim useSearch As String
        Dim matches As Integer
        Dim itemID As Long
        Dim matchItem As CheckItemDetail = Nothing

        ' ----- Prepare the title.
        useSearch = searchText.Trim()
        If (useSearch.Contains("*") = False) Then useSearch &= "*"
        useSearch = useSearch.Replace("*", "%")

        ' ----- Perform the search.
        sqlText = "SELECT AU.LastName, AU.FirstName, AU.MiddleName, AU.Suffix, " &
            "AU.BirthYear, AU.DeathYear, NI.Title, NI.SubTitle, NI.Dewey, " &
            "NI.LC, CMT.FullName AS MediaName, IC.ID, IC.Barcode, IC.CopyNumber " &
            "FROM ItemCopy AS IC " &
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " &
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " &
            "LEFT JOIN ItemAuthor AS IA ON NI.ID = IA.ItemID " &
            "LEFT JOIN Author AS AU ON IA.AuthorID = AU.ID " &
            "WHERE NI.Title LIKE @TestTitle"
        If (asCheckIn) Then sqlText &= " AND IC.ID IN" _
            Else sqlText &= " AND IC.ID NOT IN"
        sqlText &= " (SELECT ItemCopy FROM PatronCopy WHERE Returned = 0)"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestTitle", useSearch)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("ItemLookup.PerformLookup", ex)
            Return False
        End Try
        If (dbInfo.Rows.Count = 0) Then
            ' ----- No matches.
            Return False
        End If

        ' ----- Prepare for item matches.
        MatchingItems.Items.Clear()
        matches = 0
        itemID = -1&
        For Each dbRow As DataRow In dbInfo.Rows
            If (itemID <> DBGetLong(dbRow!ID)) Then
                ' ----- Create a new matching item entry.
                matchItem = New CheckItemDetail
                matchItem.TitleText = DBGetText(dbRow!Title)
                If (IsDBNull(dbRow!Subtitle) = False) Then _
                    matchItem.TitleText &= ": " & DBGetText(dbRow!Subtitle)
                matchItem.AuthorName = dbRow.FormatAuthorName()
                matchItem.MediaType = DBGetText(dbRow!MediaName)
                If (UseLC = True) Then
                    If (IsDBNull(dbRow!LC) = False) Then _
                        matchItem.CallNumber = DBGetText(dbRow!LC).GetSubStr("/", 1)
                Else
                    If (IsDBNull(dbRow!Dewey) = False) Then _
                        matchItem.CallNumber = DBGetText(dbRow!Dewey).GetSubStr("/", 1)
                End If
                If (IsDBNull(dbRow!Barcode) = False) Then _
                    matchItem.Barcode = DBGetText(dbRow!Barcode)
                If (IsDBNull(dbRow!CopyNumber) = False) Then _
                    matchItem.CopyNumber = DBGetInteger(dbRow!CopyNumber)
                matchItem.ItemCopyID = DBGetLong(dbRow!ID)

                ' ----- Ready to display the item.
                MatchingItems.Items.Add(matchItem)

                ' ----- Limit the search to SearchMatchLimit matches.
                matches += 1
                If (matches >= SearchMatchLimit) Then
                    If (dbInfo.Rows.Count > SearchMatchLimit) Then matches += 1
                    Exit For
                End If
            Else
                ' ----- We found another match for the previous item, caused
                '       by the presence of another author record. Modify the
                '       last item by adding "et. al." to the display.
                If (matchItem.MultiAuthor = False) Then
                    matchItem.AuthorName &= ", et. al."
                    matchItem.MultiAuthor = True
                End If
            End If
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Display the results.
        If (matches = 1) Then
            LabelItemsCount.Text = "1 matching item found."
        ElseIf (matches > SearchMatchLimit) Then
            LabelItemsCount.Text = "Limited to first " & SearchMatchLimit &
                " matching items."
        Else
            LabelItemsCount.Text = matches & " matching items found."
        End If
        Return True
    End Function

    Private Sub MatchingItems_DrawItem(sender As Object, e As DrawItemEventArgs) Handles MatchingItems.DrawItem
        ' ----- Draw the matching items on two lines.
        Dim useBrush As Brush
        Dim copyBarcodeText As String
        Dim boldFont As Font
        Dim itemDetail As CheckItemDetail
        Dim ellipsesText As StringFormat

        ' ----- Draw the background of the item.
        If (CBool(CInt(e.State) And CInt(DrawItemState.Selected))) _
            Then useBrush = SystemBrushes.HighlightText _
            Else useBrush = SystemBrushes.WindowText
        e.DrawBackground()

        ' ----- Make a bold version of the font.
        boldFont = New Font(e.Font, FontStyle.Bold)

        ' ----- Extract the details from the list.
        itemDetail = DirectCast(MatchingItems.Items(e.Index), CheckItemDetail)

        ' ----- Format the display strings.
        If (itemDetail.CopyNumber = -1) Then copyBarcodeText = "??" Else _
            copyBarcodeText = itemDetail.CopyNumber.ToString()
        copyBarcodeText &= " / " & itemDetail.Barcode
        ellipsesText = New StringFormat
        ellipsesText.Trimming = StringTrimming.EllipsisCharacter

        ' ----- Draw the text of the item.
        e.Graphics.DrawString(itemDetail.TitleText, boldFont, useBrush,
            New Rectangle(0, e.Bounds.Top, ItemColBarcode.Left -
            MatchingItems.Left - 5, boldFont.Height), ellipsesText)
        e.Graphics.DrawString(copyBarcodeText, e.Font, useBrush,
            New Rectangle(ItemColBarcode.Left - ItemColTitle.Left, e.Bounds.Top,
            ItemColEnd.Left - ItemColBarcode.Left, e.Font.Height), ellipsesText)
        e.Graphics.DrawString(itemDetail.AuthorName, e.Font, useBrush,
            New Rectangle(ItemColAuthor.Left - ItemColTitle.Left,
            e.Bounds.Top + SecondItemRow, ItemColCall.Left - ItemColAuthor.Left - 8,
            e.Font.Height), ellipsesText)
        e.Graphics.DrawString(itemDetail.CallNumber, e.Font, useBrush,
            New Rectangle(ItemColCall.Left - ItemColTitle.Left,
            e.Bounds.Top + SecondItemRow, ItemColEnd.Left - ItemColType.Left - 5,
            e.Font.Height), ellipsesText)
        e.Graphics.DrawString(itemDetail.MediaType, e.Font, useBrush,
            New Rectangle(ItemColType.Left - ItemColTitle.Left,
            e.Bounds.Top + SecondItemRow, ItemColType.Left - ItemColCall.Left - 8,
            e.Font.Height), ellipsesText)

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
        boldFont.Dispose()
    End Sub

    Private Sub CheckLookup_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then
            If (CheckInMode = True) Then
                OnlineHelp(Me, "CheckLookup_In.htm")
            Else
                OnlineHelp(Me, "CheckLookup_Out.htm")
            End If
        End If
    End Sub

    Private Sub CheckLookup_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim formGraphics As Graphics

        ' ----- Set the default height of items in the matching items listbox.
        formGraphics = Me.CreateGraphics()
        MatchingItems.ItemHeight =
            CInt(formGraphics.MeasureString("A" & vbCrLf & "g",
            MatchingItems.Font).Height()) + 3
        SecondItemRow = CInt(formGraphics.MeasureString("Ag",
            MatchingItems.Font).Height()) + 1
        formGraphics.Dispose()

        ' ----- Adjust the form title.
        If (CheckInMode = True) Then
            Me.Text = "Check-In By Title"
        Else
            Me.Text = "Check-Out By Title"
        End If
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- Make sure a list item is selected.
        If (MatchingItems.SelectedIndex = -1) Then
            MessageBox.Show("Please select an item from the list.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Continue with CheckItemByTitle()
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub MatchingItems_DoubleClick(sender As Object, e As EventArgs) Handles MatchingItems.DoubleClick
        ' ----- Same as a click on the OK button.
        ActOK.PerformClick()
    End Sub
End Class
