﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class ListEditRecords
    Private DetailEditor As Library.BaseCodeForm

    Public Sub ManageRecords(UseDetail As Library.BaseCodeForm)
        ' ----- Set up the form for use with this code set.
        Dim exceededMatches As Boolean

        DetailEditor = UseDetail
        RecordsTitle.Text = DetailEditor.GetEditTitle()
        RecordsInfo.Text = DetailEditor.GetEditDescription()
        Me.Text = DetailEditor.GetEditTitle()
        ActAdd.Visible = DetailEditor.CanUserAdd()
        ActEdit.Visible = DetailEditor.CanUserEdit()
        ActDelete.Visible = DetailEditor.CanUserDelete()
        ActLookup.Visible = DetailEditor.UsesSearch()
        DetailEditor.FillListWithRecords(RecordsList, exceededMatches)
        RefreshItemCount(exceededMatches)
        Me.ShowDialog()
    End Sub

    Private Sub RefreshButtons()
        ' ----- Update the buttons as needed.
        ActEdit.Enabled = CBool(RecordsList.SelectedIndex <> -1)
        ActDelete.Enabled = CBool(RecordsList.SelectedIndex <> -1)
    End Sub

    Private Sub RefreshItemCount(exceededMatches As Boolean)
        ' ----- Update the list count display.
        If (exceededMatches) Then
            ResultCount.Text = "Results limited to first " &
                RecordsList.Items.Count & " items"
        ElseIf (RecordsList.Items.Count = 1) Then
            ResultCount.Text = "1 record displayed"
        Else
            ResultCount.Text = RecordsList.Items.Count & " records displayed"
        End If
    End Sub

    Private Sub ActClose_Click(sender As Object, e As EventArgs) Handles ActClose.Click
        ' ----- Close the form.
        DetailEditor.Close()
        DetailEditor = Nothing
        Me.Close()
    End Sub

    Private Sub ActAdd_Click(sender As Object, e As EventArgs) Handles ActAdd.Click
        ' ----- Let the user add a record.
        Dim newID As Long
        Dim newPosition As Integer

        ' ----- Prompt the user.
        newID = DetailEditor.AddRecord()
        If (newID = -1&) Then Return

        ' ----- Add this record to the list.
        newPosition = RecordsList.Items.Add((New Library.ListItemData(
            DetailEditor.FormatRecordName(newID), newID)))
        RecordsList.SelectedIndex = newPosition
        RefreshButtons()
        RefreshItemCount(False)
    End Sub

    Private Sub ActEdit_Click(sender As Object, e As EventArgs) Handles ActEdit.Click
        ' ----- Let the user edit a record.
        Dim newID As Long
        Dim listItem As Library.ListItemData

        ' ----- Retrieve the item to edit.
        If (RecordsList.SelectedIndex = -1) Then Return
        listItem = DirectCast(RecordsList.SelectedItem, Library.ListItemData)

        ' ----- Prompt the user.
        newID = DetailEditor.EditRecord(listItem.ItemData)
        If (newID = -1&) Then Return

        ' ----- Modify the text of this item.
        listItem.ItemText = DetailEditor.FormatRecordName(newID)
        RecordsList.Items(RecordsList.SelectedIndex) = listItem
        RefreshItemCount(False)
    End Sub

    Private Sub ActDelete_Click(sender As Object, e As EventArgs) Handles ActDelete.Click
        ' ----- Let the user edit a record.
        Dim listItem As Library.ListItemData

        ' ----- Retrieve the item to edit.
        If (RecordsList.SelectedIndex = -1) Then Return
        listItem = DirectCast(RecordsList.SelectedItem, Library.ListItemData)

        ' ----- Prompt the user.
        If (DetailEditor.DeleteRecord(listItem.ItemData) = False) Then Return

        ' ----- Remove this item from the display.
        RecordsList.Items.RemoveAt(RecordsList.SelectedIndex)
        RefreshButtons()
        RefreshItemCount(False)
    End Sub

    Private Sub ActLookup_Click(sender As Object, e As EventArgs) Handles ActLookup.Click
        ' ----- Prompt for user limitation of the displayed items.
        Dim exceededMatches As Boolean

        DetailEditor.SearchForRecord(RecordsList, exceededMatches)
        RefreshButtons()
        RefreshItemCount(exceededMatches)
    End Sub

    Private Sub RecordsList_Click(sender As Object, e As EventArgs) Handles RecordsList.Click
        ' ----- Enable controls as needed.
        RefreshButtons()
    End Sub

    Private Sub RecordsList_DoubleClick(sender As Object, e As EventArgs) Handles RecordsList.DoubleClick
        ' ----- Same as the edit button.
        If (ActEdit.Visible = True) Then ActEdit.PerformClick()
    End Sub

    Private Sub RecordsList_SelectedIndexChanged(sender As Object, e As EventArgs) Handles RecordsList.SelectedIndexChanged
        ' ----- Enable controls as needed.
        RefreshButtons()
    End Sub

    Private Sub ListEditRecords_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "ListEditRecords.htm")
    End Sub
End Class