﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class PatronPassword
    Private ActiveID As Long

    Public Function ChangePassword(patronID As Long) As Boolean
        ' ----- Edit an existing record.
        ActiveID = patronID
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return True Else Return False
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- The user wants to save changes.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        sqlText = "UPDATE Patron SET [Password] = @NewPassword " &
            "WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("NewPassword", EncryptPassword("patron",
            RecordPassword.Text.Trim()))
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("PatronPassword.SaveFormData", ex)
            Return False
        Finally
            Me.Cursor = Cursors.Default
        End Try

        ' ----- Success.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        '       Password and confirm must match.
        If (RecordPassword.Text <> RecordConfirm.Text) Then
            MessageBox.Show("The Password and Confirm fields must match.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordPassword.Focus()
            Return False
        End If

        ' ----- A password is required.
        If (RecordPassword.Text.Trim().Length = 0) Then
            MessageBox.Show("The Password field is required.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordPassword.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub RecordPassword_Enter(sender As Object, e As EventArgs) Handles RecordPassword.Enter
        ' ----- Highlight the entire text.
        RecordPassword.Focus()
    End Sub

    Private Sub RecordConfirm_Enter(sender As Object, e As EventArgs) Handles RecordConfirm.Enter
        ' ----- Highlight the entire text.
        RecordConfirm.Focus()
    End Sub

    Private Sub PatronPassword_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "PatronPassword.htm")
    End Sub

    Private Sub PatronPassword_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Show the patron name.
        sqlText = "SELECT FirstName + ' ' + LastName FROM Patron " &
            "WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            RecordName.Text = DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("PatronPassword.Form_Load", ex)
            RecordName.Text = "Unknown"
        End Try

        ' ----- Initialize other fields.
        RecordPassword.Clear()
        RecordConfirm.Clear()
    End Sub
End Class