﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class SeriesAddLocate
    Private ActivePanel As Integer

    Private Const PanelCriteria As Integer = 1
    Private Const PanelResults As Integer = 2

    Private Function FindUserRecords(searchText As String) As Integer
        ' ----- Search for series codes using user-specified criteria.
        Dim sqlText As String
        Dim dbInfo As DataTable
        Dim sqlRun As DataCommand
        Dim matches As Integer

        ' ----- Clear any existing values.
        SearchResults.Items.Clear()
        Me.Cursor = Cursors.WaitCursor

        ' ----- Retrieve the values from the database.
        matches = 0
        sqlText = "SELECT ID, FullName FROM CodeSeries " &
            "WHERE FullName LIKE @SearchText ORDER BY FullName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("SearchText", searchText)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("SeriesAddLocate.FindUserRecords", ex)
            Return -1
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            SearchResults.Items.Add(New ListItemData(DBGetText(dbRow!FullName),
                DBGetLong(dbRow!ID)))
            matches += 1
            If (matches >= SearchMatchLimit) Then Exit For
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
        Me.Cursor = Cursors.Default
        Return matches
    End Function

    Public Function PromptUser() As Long
        ' ----- Prompt the user for a series. Return the matching ID. Return
        '       -1 if no change made, or -2 to clear the active setting.
        ActivePanel = PanelCriteria
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then
            If (CLng(DirectCast(SearchResults.SelectedItem, ListItemData)) = -2&) Then
                ' ----- Clear the current series.
                Return -2&
            ElseIf (CLng(DirectCast(SearchResults.SelectedItem, ListItemData)) = -1&) Then
                ' ----- Prompt for a new series using the original typed name.
                Return (New CodeSeries).AddRecord(SeriesName.Text.Trim())
            Else
                ' ----- The user selected a match.
                Return CLng(DirectCast(SearchResults.SelectedItem, ListItemData))
            End If
        Else
            ' ----- Cancelled by user.
            Return -1&
        End If
    End Function

    Private Sub SeriesName_Enter(sender As Object, e As EventArgs) Handles SeriesName.Enter
        ' ----- Highlight the entire text.
        SeriesName.SelectAll()
    End Sub

    Private Sub ActBack_Click(sender As Object, e As EventArgs) Handles ActBack.Click
        ' ----- Go back to the first panel.
        ActivePanel = PanelCriteria
        SecondPanel.Visible = False
        FirstPanel.Visible = True
        ActBack.Enabled = False
        ActNext.Text = "&Next >"
        SeriesName.Focus()
    End Sub

    Private Sub ActNext_Click(sender As Object, e As EventArgs) Handles ActNext.Click
        ' ----- Take action on the user input.
        Dim searchText As String
        Dim matches As Integer

        ' ----- Different code for each of the panels.
        If (ActivePanel = PanelCriteria) Then
            ' ----- Determine the search string.
            searchText = SeriesName.Text.Trim()
            If (searchText.Length = 0) Then
                MessageBox.Show("Please supply a series name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                SeriesName.Focus()
                Return
            End If
            If (searchText.Contains("*") = False) Then searchText &= "*"
            searchText = searchText.Replace("*", "%")

            ' ----- Search for matches.
            matches = FindUserRecords(searchText)
            If (matches = -1) Then Return

            ' ----- If there are no matches, then either error out, or
            '       add a new record, depending on the user's security.
            '       If there are matches, show them.
            If (matches = 0) Then
                If (SecurityProfile(LibrarySecurity.ManageSeries)) Then
                    ' ----- Prompt to add a new record.
                    SearchResults.ClearSelected()
                    SearchResults.Items.Add(New ListItemData(
                        "<No matches. Add new series.>", -1&))
                    SearchResults.SelectedIndex = 0

                    ' ----- Return to the PromptUser() routine.
                    Me.DialogResult = DialogResult.OK
                Else
                    ' ----- Give an error.
                    MessageBox.Show("No matches.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    Return
                End If
            Else
                ' ----- If the user can add new series, add that option.
                If (SecurityProfile(LibrarySecurity.ManageSeries)) Then _
                    SearchResults.Items.Insert(0, New ListItemData(
                    "<None of these matches. Add new series.>", -1&))
                ActivePanel = PanelResults
                SecondPanel.Visible = True
                FirstPanel.Visible = False
                ActBack.Enabled = True
                ActNext.Text = "Finish"
                SearchResults.Focus()
            End If
        Else
            ' ----- Check for a selected result.
            If (SearchResults.SelectedIndex = -1) Then
                MessageBox.Show("Please select a match from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                SearchResults.Focus()
                Return
            End If

            ' ----- Return to the PromptUser() routine.
            Me.DialogResult = DialogResult.OK
        End If
    End Sub

    Private Sub SearchResults_DoubleClick(sender As Object, e As EventArgs) Handles SearchResults.DoubleClick
        ' ----- Same as the finish button.
        If (SearchResults.SelectedIndex = -1) Then Return
        ActNext.PerformClick()
    End Sub

    Private Sub ActClear_Click(sender As Object, e As EventArgs) Handles ActClear.Click
        ' ----- Clear the current series.
        SearchResults.ClearSelected()
        SearchResults.Items.Add(New ListItemData("Clear item", -2&))
        SearchResults.SelectedIndex = 0

        ' ----- Return to the PromptUser() routine.
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub SeriesAddLocate_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "SeriesAddLocate.htm")
    End Sub

    Private Sub SeriesAddLocate_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        SecondPanel.Location = New Point(FirstPanel.Location.X, FirstPanel.Location.Y)
    End Sub
End Class