﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Module General
    ' ----- Public constants.
    Public Const ProgramTitle As String = "The Library Project"
    Public Const NotAuthorizedMessage As String =
        "You are not authorized to perform this task."
    Public Const UseDBVersion As Integer = 1

    ' ----- Constants for the MatchingImages image list.
    Public Const MatchPresent As Integer = 0
    Public Const MatchNone As Integer = 1

    Public Enum LookupMethods As Integer
        ByTitle = 1
        ByAuthor = 2
        BySubject = 3
        ByKeywordAny = 4
        ByKeywordAll = 5
        ByPublisher = 6
        BySeries = 7
        BySeriesID = 8
        ByBarcode = 9
        ByDatabaseID = 10
        ByAuthorID = 11
        BySubjectID = 12
        ByPublisherID = 13
    End Enum

    ' ----- Security values.
    Public Enum LibrarySecurity As Integer
        ManageAuthors = 1
        ManageAuthorTypes = 2
        ManageCopyStatus = 3
        ManageMediaTypes = 4
        ManageSeries = 5
        ManageGroups = 6
        ManageItems = 7
        ManagePatrons = 8
        ManagePublishers = 9
        ManageValues = 10
        ManageUsers = 11
        ProcessFees = 12
        ManageLocations = 13
        CheckOutItems = 14
        CheckInItems = 15
        AdminFeatures = 16
        DailyProcessing = 17
        RunReports = 18
        PatronOverride = 19
        ManageBarcodeTemplates = 20
        ManageHolidays = 21
        ManagePatronGroups = 22
        ViewAdminPatronMessages = 23
    End Enum
    Public Const MaxLibrarySecurity As LibrarySecurity = LibrarySecurity.ViewAdminPatronMessages

    Public Function CenterText(origText As String, textWidth As Integer) As String
        ' ----- Center a piece of text in a field width. If the text is too
        '       wide, truncate it.
        Dim resultText As String

        resultText = origText.Trim()
        If (resultText.Length >= textWidth) Then
            ' ----- Truncate as needed.
            Return origText.Substring(0, textWidth).Trim()
        Else
            ' ----- Start with extra spaces.
            Return New String(" "c, (textWidth - origText.Length) \ 2) & resultText
        End If
    End Function

    Public Function CountSubStr(mainText As String, subText As String) As Integer
        ' ----- Return a count of the number of times that a subText occurs in
        '       a string (mainText).
        Dim totalTimes As Integer
        Dim startPos As Integer
        Dim foundPos As Integer

        If (mainText Is Nothing) OrElse (mainText.Length = 0) Then Return 0
        If (subText Is Nothing) OrElse (subText.Length = 0) Then Return 0
        totalTimes = 0
        startPos = 0

        ' ----- Keep searching until we don't find it no more!
        Do
            ' ----- Search for the subText.
            foundPos = mainText.IndexOf(subText, startPos)
            If (foundPos = -1) Then Exit Do
            totalTimes += 1

            ' ----- Move to just after the occurrence.
            startPos = foundPos + subText.Length
        Loop

        ' ----- Return the count.
        Return totalTimes
    End Function

    Public Function DigitsOnly(origText As String) As String
        ' ----- Return only the digits found in a string.
        Dim destText As String
        Dim counter As Integer

        ' ----- Examine each character.
        destText = ""
        For counter = 0 To origText.Length - 1
            If (IsNumeric(origText.Substring(counter, 1))) Then _
                destText &= origText.Substring(counter, 1)
        Next counter
        Return destText
    End Function

    Public Sub GeneralError(routineName As String, theError As Exception)
        ' ----- Report an error to the user.
        On Error Resume Next

        MessageBox.Show("The following error occurred at location '" & routineName &
            "':" & vbCrLf & vbCrLf & theError.Message,
            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        My.Application.Log.WriteException(theError)
    End Sub

    Public Function GetSubStr(origString As String, delim As String,
            whichField As Integer) As String
        ' ----- Extracts a delimited string from another larger string.
        Dim stringParts() As String

        ' ----- Handle some errors.
        If (whichField < 0) Then Return ""
        If (origString Is Nothing) OrElse (origString.Length = 0) Then Return ""
        If (delim Is Nothing) OrElse (delim.Length = 0) Then Return ""

        ' ----- Break the string up into delimited parts.
        stringParts = origString.Split({delim}, StringSplitOptions.None)

        ' ----- See if the part we want exists and return it.
        If (whichField > stringParts.Length) Then Return "" _
            Else Return stringParts(whichField - 1)
    End Function

    Public Function LeftAndRightText(origLeft As String, origRight As String,
            textWidth As Integer) As String
        ' ----- Build a string of the indicated width that has both left-justified
        '       and right-justified text.
        Dim leftText As String
        Dim rightText As String

        leftText = origLeft.Trim()
        rightText = origRight.Trim()
        If ((leftText & rightText).Length >= textWidth) Then
            Return (leftText & rightText).Substring(0, textWidth).Trim()
        Else
            Return leftText & New String(" "c,
                textWidth - (leftText & rightText).Length) & rightText
        End If
    End Function
End Module
