﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class CodeMediaType
    Private ActiveID As Long
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageMediaTypes)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageMediaTypes)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageMediaTypes)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the media type?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False


        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM NamedItem WHERE MediaType = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("You cannot delete this record because it is in use.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        Catch ex As Exception
            GeneralError("CodeMediaType.DeleteRecord.Check", ex)
            Return False
        End Try

        ' ----- Delete the record.
        sqlText = "DELETE FROM CodeMediaType WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("CodeMediaType.DeleteRecord.Delete", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, FullName FROM CodeMediaType ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("CodeMediaType.FillListWithRecords", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            destList.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM CodeMediaType WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            Return DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("CodeMediaType.FormatRecordName", ex)
            Return "Error"
        End Try
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use these codes to distinguish between the different types of " &
            "materials available in the library, such as books, CDs, etc."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Media Types"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        RecordCheckoutDays.Text = ""
        RecordRenewDays.Text = ""
        RecordRenewTimes.Text = ""
        RecordDailyFine.Text = ""
        If (ActiveID = -1&) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM CodeMediaType WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("CodeMediaType.PrepareFormFields", ex)
            Return
        End Try

        If (dbInfo.Rows.Count > 0) Then
            oneRow = dbInfo.Rows(0)
            RecordFullName.Text = DBGetText(oneRow!FullName)
            RecordCheckoutDays.Text = DBGetInteger(oneRow!CheckoutDays).ToString()
            RecordRenewDays.Text = DBGetInteger(oneRow!RenewDays).ToString()
            RecordRenewTimes.Text = DBGetInteger(oneRow!RenewTimes).ToString()
            RecordDailyFine.Text = DBGetDecimal(oneRow!DailyFine).ToString("0.00")
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the database.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long

        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO CodeMediaType (FullName, CheckoutDays, " &
                "RenewDays, RenewTimes, DailyFine) OUTPUT INSERTED.ID " &
                "VALUES (@NewName, @NewCheckout, @NewRenew, @NewTimes, @NewFine)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            sqlRun.AddInteger("NewCheckout", RecordCheckoutDays.Text)
            sqlRun.AddInteger("NewRenew", RecordRenewDays.Text)
            sqlRun.AddInteger("NewTimes", RecordRenewTimes.Text)
            sqlRun.AddDecimal("NewFine", RecordDailyFine.Text)
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                GeneralError("CodeMediaType.SaveFormData.Add", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE CodeMediaType SET FullName = @NewName, " &
                "CheckoutDays = @NewCheckout, RenewDays = @NewRenew, " &
                "RenewTimes = @NewTimes, DailyFine = @NewFine " &
                "WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            sqlRun.AddInteger("NewCheckout", RecordCheckoutDays.Text)
            sqlRun.AddInteger("NewRenew", RecordRenewDays.Text)
            sqlRun.AddInteger("NewTimes", RecordRenewTimes.Text)
            sqlRun.AddDecimal("NewFine", RecordDailyFine.Text)
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("CodeMediaType.SaveFormData.Update", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Name is a required field.
        If (RecordFullName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the media type name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Checkout days must be an integer.
        If (IsNumeric(RecordCheckoutDays.Text) = False) Or
                (RecordCheckoutDays.Text.Contains(".") = True) Or
                (Val(RecordCheckoutDays.Text) < 0) Or
                (Val(RecordCheckoutDays.Text) > 99) Then
            MessageBox.Show("Checkout Days must be an integer between 0 and 99.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordCheckoutDays.Focus()
            Return False
        End If

        ' ----- Renewal Days must be an integer.
        If (IsNumeric(RecordRenewDays.Text) = False) Or
                (RecordRenewDays.Text.Contains(".") = True) Or
                (Val(RecordRenewDays.Text) < 0) Or
                (Val(RecordRenewDays.Text) > 99) Then
            MessageBox.Show("Renewal Days must be an integer between 0 and 99.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordRenewDays.Focus()
            Return False
        End If

        ' ----- Renewal Limit must be an integer.
        If (IsNumeric(RecordRenewTimes.Text) = False) Or
                (RecordRenewTimes.Text.Contains(".") = True) Or
                (Val(RecordRenewTimes.Text) < 0) Or
                (Val(RecordRenewTimes.Text) > 99) Then
            MessageBox.Show("Renewal Limit must be an integer between 0 and 99.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordRenewTimes.Focus()
            Return False
        End If

        ' ----- Daily Fine must be a number.
        If (IsNumeric(RecordDailyFine.Text) = False) Then
            MessageBox.Show("Daily Fine must be a valid number.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordDailyFine.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM CodeMediaType " &
            "WHERE UPPER(FullName) = @NewName "
        If (ActiveID <> -1&) Then sqlText &= " AND ID <> @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("NewName", RecordFullName.Text.Trim().ToUpper())
        If (ActiveID <> -1&) Then _
            sqlRun.AddLong("RecordID", ActiveID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                ' ----- Duplicate entry.
                MessageBox.Show("A record already exists with the name '" &
                    RecordFullName.Text.Trim() & "'.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordFullName.Focus()
                Return False
            End If
        Catch ex As Exception
            GeneralError("CodeMediaType.ValidateFormData", ex)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub CodeMediaType_Activated(sender As Object, e As EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordFullName_Enter(sender As Object, e As EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub RecordCheckoutDays_Enter(sender As Object, e As EventArgs) Handles RecordCheckoutDays.Enter
        ' ----- Highlight the entire text.
        RecordCheckoutDays.SelectAll()
    End Sub

    Private Sub RecordRenewDays_Enter(sender As Object, e As EventArgs) Handles RecordRenewDays.Enter
        ' ----- Highlight the entire text.
        RecordRenewDays.SelectAll()
    End Sub

    Private Sub RecordRenewTimes_Enter(sender As Object, e As EventArgs) Handles RecordRenewTimes.Enter
        ' ----- Highlight the entire text.
        RecordRenewTimes.SelectAll()
    End Sub

    Private Sub RecordDailyFine_Enter(sender As Object, e As EventArgs) Handles RecordDailyFine.Enter
        ' ----- Highlight the entire text.
        RecordDailyFine.SelectAll()
    End Sub

    Private Sub RecordCheckoutDays_KeyPress(sender As Object, e As KeyPressEventArgs) Handles RecordCheckoutDays.KeyPress
        ' ----- Only allow digits and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Then Return
        e.Handled = True
    End Sub

    Private Sub RecordRenewDays_KeyPress(sender As Object, e As KeyPressEventArgs) Handles RecordRenewDays.KeyPress
        ' ----- Only allow digits and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Then Return
        e.Handled = True
    End Sub

    Private Sub RecordRenewTimes_KeyPress(sender As Object, e As KeyPressEventArgs) Handles RecordRenewTimes.KeyPress
        ' ----- Only allow digits and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Then Return
        e.Handled = True
    End Sub

    Private Sub RecordDailyFine_KeyPress(sender As Object, e As KeyPressEventArgs) Handles RecordDailyFine.KeyPress
        ' ----- Only allow digits, decimals, and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Or
            (e.KeyChar = ".") Then Return
        e.Handled = True
    End Sub
End Class
