﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class Holiday
    Private ActiveID As Long
    Private StartingOver As Boolean

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageHolidays)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageHolidays)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageHolidays)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the selected holiday?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Delete the record.
        sqlText = "DELETE FROM Holiday WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("Holiday.DeleteRecord", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Clear any existing values.
        destList.Items.Clear()
        exceededMatches = False

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT * FROM Holiday ORDER BY EntryType, EntryDetail"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("Holiday.FillListWithRecords", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            destList.Items.Add(New ListItemData(
                FormatOneEntry(dbRow), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Private Function FormatOneEntry(dbRow As DataRow) As String
        ' ----- Given an active record, format its content for display.
        Dim itemText As String

        ' ----- Different formatting for different entry types.
        itemText = ""
        Select Case DBGetText(dbRow!EntryType).ToUpper()
            Case "A"
                ' ----- Annually.
                itemText = DBGetText(dbRow!EntryDetail) & "/2004"
                If (IsDate(itemText) = True) Then
                    itemText = "Annually - " & CDate(itemText).ToString("MMMM d")
                Else
                    itemText = "Annually - Invalid Entry"
                End If
            Case "E"
                ' ----- Weekly.
                If (CInt(DBGetText(dbRow!EntryDetail)) < FirstDayOfWeek.Sunday) Or
                        (CInt(DBGetText(dbRow!EntryDetail)) > FirstDayOfWeek.Saturday) Then
                    itemText = "Weekly - Invalid Entry"
                Else
                    itemText = "Weekly - Every " & WeekdayName(CInt(DBGetText(dbRow!EntryDetail)),
                        False, FirstDayOfWeek.Sunday)
                End If
            Case "O"
                ' ----- One-time.
                itemText = DBGetText(dbRow!EntryDetail)
                If (IsDate(itemText) = True) Then
                    itemText = CDate(itemText).ToString("yyyy") & " - " &
                        CDate(itemText).ToString("MMMM d, yyyy")
                Else
                    itemText = "???? - Invalid Entry"
                End If
        End Select
        itemText &= " [" & DBGetText(dbRow!FullName) & "]"
        Return itemText
    End Function

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim result As String

        ' ----- Initialize.
        result = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT * FROM Holiday WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Holiday.FormatRecordName", ex)
            Return "Error"
        End Try

        ' ----- Format the holiday for display.
        result = FormatOneEntry(dbInfo.Rows(0))

        ' ----- Finished.
        dbInfo.Dispose()
        dbInfo = Nothing
        Return result
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return a description of this editor.
        Return "Use these codes to establish the different holidays, which can " &
            "not be used for item due dates."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the title-bar text for this editor.
        Return "Holidays"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim counter As Integer
        Dim holdText As String

        ' ----- Load in the list values.
        If (RecordType.Items.Count = 0) Then
            ' ----- Fill in the list of item types.
            RecordType.Items.Add(New ListItemData("<Not Selected>", Asc(" ")))
            RecordType.Items.Add(New ListItemData("Annual Holiday", Asc("A")))
            RecordType.Items.Add(New ListItemData("Weekly Holiday", Asc("E")))
            RecordType.Items.Add(New ListItemData("One-time Holiday", Asc("O")))

            ' ----- Fill in the list of months.
            RecordMonth.Items.Clear()
            For counter = 1 To 12
                RecordMonth.Items.Add(New ListItemData(MonthName(counter), counter))
            Next counter

            ' ----- Fill in the list of days.
            RecordDay.Items.Clear()
            For counter = 1 To 31
                RecordDay.Items.Add(New ListItemData(CStr(counter), counter))
            Next counter

            ' ----- Fill in the list of weekdays.
            RecordWeekday.Items.Clear()
            For counter = 1 To 7
                RecordWeekday.Items.Add(New ListItemData(WeekdayName(counter, False,
                    FirstDayOfWeek.Sunday), counter))
            Next counter
        End If

        ' ----- Clear the existing data.
        StartingOver = True
        RecordName.Text = ""
        RecordType.SelectedIndex = RecordType.Items.IndexOf(CLng(Asc(" ")))
        RecordMonth.SelectedIndex = RecordMonth.Items.IndexOf(1&)
        RecordDay.SelectedIndex = RecordDay.Items.IndexOf(1&)
        RecordWeekday.SelectedIndex =
            RecordWeekday.Items.IndexOf(CLng(FirstDayOfWeek.Sunday))
        RecordOneTime.Value = Today
        If (ActiveID = -1&) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM Holiday WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Holiday.PrepareFormFields", ex)
            Return
        End Try

        If (dbInfo.Rows.Count > 0) Then
            oneRow = dbInfo.Rows(0)
            RecordName.Text = DBGetText(oneRow!FullName)
            RecordType.SelectedIndex = RecordType.Items.IndexOf(
                CLng(Asc(DBGetText(oneRow!EntryType).ToUpper())))
            If (RecordType.SelectedIndex = -1) Then
                ' ----- Something wrong with this record.
                RecordType.SelectedIndex = RecordType.Items.IndexOf(CLng(Asc(" ")))
            Else
                Select Case DBGetText(oneRow!EntryType).ToUpper()
                    Case "A"
                        ' ----- Annually.
                        holdText = DBGetText(oneRow!EntryDetail) & "/2004"
                        If (IsDate(holdText) = True) Then
                            RecordMonth.SelectedIndex =
                                RecordMonth.Items.IndexOf(CLng(CDate(holdText).Month))
                            RecordDay.SelectedIndex =
                                RecordDay.Items.IndexOf(CLng(CDate(holdText).Day))
                        End If
                    Case "E"
                        ' ----- Weekly.
                        holdText = DBGetText(oneRow!EntryDetail)
                        If (Val(holdText) >= FirstDayOfWeek.Sunday) And
                                (Val(holdText) <= FirstDayOfWeek.Saturday) Then
                            RecordWeekday.SelectedIndex =
                                RecordWeekday.Items.IndexOf(CLng(Val(holdText)))
                        End If
                    Case "O"
                        ' ----- One-time.
                        If (IsDate(DBGetText(oneRow!EntryDetail)) = True) Then _
                            RecordOneTime.Value = CDate(DBGetText(oneRow!EntryDetail))
                End Select
            End If
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the record.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim entryType As String
        Dim entryDetail As String
        Dim newID As Long

        Me.Cursor = Cursors.WaitCursor

        ' ----- Get the entry detail content.
        entryType = CStr(Chr(CInt(CLng(DirectCast(RecordType.SelectedItem, ListItemData)))))
        Select Case entryType
            Case "A"
                ' ----- A: Annually.
                entryDetail = CLng(DirectCast(RecordMonth.SelectedItem, ListItemData)) & "/" &
                    CLng(DirectCast(RecordDay.SelectedItem, ListItemData))
            Case "E"
                ' ----- W: Weekly.
                entryDetail = CStr(CLng(DirectCast(RecordWeekday.SelectedItem, ListItemData)))
            Case Else
                ' ----- O: One-time.
                entryDetail = RecordOneTime.Value.ToString("yyyy/MM/dd")
        End Select

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO Holiday(FullName, EntryType, EntryDetail) " &
                "OUTPUT INSERTED.ID VALUES (@NewName, @NewType, @NewDetail)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordName.Text.Trim())
            sqlRun.AddText("NewType", entryType)
            sqlRun.AddText("NewDetail", entryDetail)
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun), -1&)
            Catch ex As Exception
                GeneralError("Holiday.SaveFormData.Add", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE Holiday SET FullName = @NewName, " &
                "EntryType = @NewType, EntryDetail = @NewDetail " &
                "WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordName.Text.Trim())
            sqlRun.AddText("NewType", entryType)
            sqlRun.AddText("NewDetail", entryDetail)
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("Holiday.SaveFormData.Update", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim holdText As String

        ' ----- Name is a required field.
        If (RecordName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the holiday name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordName.Focus()
            Return False
        End If

        ' ----- Make sure a valid type is selected.
        If (Chr(CInt(CLng(DirectCast(RecordType.SelectedItem, ListItemData)))) = " ") Then
            MessageBox.Show("Please select a holiday type.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordName.Focus()
            Return False
        End If

        ' ----- If an annual holiday is used, check for a valid date. I
        '       used 2004 since it has a leap day in it, but the year used
        '       isn't that important.
        If (Chr(CInt(CLng(DirectCast(RecordType.SelectedItem, ListItemData)))) = "A") Then
            holdText = CLng(DirectCast(RecordMonth.SelectedItem, ListItemData)) & "/" &
                    CLng(DirectCast(RecordDay.SelectedItem, ListItemData)) & "/2004"
            If (IsDate(holdText) = False) Then
                MessageBox.Show("Please select a valid day for the selected month.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordDay.Focus()
                Return False
            End If
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub Holiday_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Adjust the elements on the form.
        PanelAnnually.Location = New Point(
            PanelWeekly.Location.X, PanelWeekly.Location.Y)
        PanelOneTime.Location = New Point(
            PanelWeekly.Location.X, PanelWeekly.Location.Y)
    End Sub

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub Holiday_Activated(sender As Object, e As EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordName_Enter(sender As Object, e As EventArgs) Handles RecordName.Enter
        ' ----- Highlight the entire text.
        RecordName.SelectAll()
    End Sub

    Private Sub RecordType_SelectedIndexChanged(sender As Object, e As EventArgs) Handles RecordType.SelectedIndexChanged
        ' ----- Display the correct detail fields.
        If (RecordType.SelectedIndex = -1) Then Return
        Select Case CStr(Chr(CInt(CLng(DirectCast(RecordType.SelectedItem, ListItemData)))))
            Case " "
                ' ----- Not selected.
                PanelAnnually.Visible = False
                PanelWeekly.Visible = False
                PanelOneTime.Visible = False
            Case "A"
                ' ----- Annually.
                PanelAnnually.Visible = True
                PanelWeekly.Visible = False
                PanelOneTime.Visible = False
            Case "E"
                ' ----- Weekly.
                PanelAnnually.Visible = False
                PanelWeekly.Visible = True
                PanelOneTime.Visible = False
            Case "O"
                ' ----- One-time.
                PanelAnnually.Visible = False
                PanelWeekly.Visible = False
                PanelOneTime.Visible = True
        End Select
    End Sub
End Class
