﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class HolidaySet
    Private HolidayType As String
    Private HolidayDetail As String
    Private IsValid As Boolean

    Public Sub New(entryType As String, entryDetail As String)
        ' ----- Create a new holiday entry instance.
        HolidayType = ""
        If (entryType.Trim().Length > 1) Then _
            HolidayType = entryType.ToUpper().Trim().Substring(0, 1)
        HolidayDetail = entryDetail

        ' ----- See if the details are valid.
        IsValid = True
        Select Case HolidayType
            Case "A"
                ' ----- The detail should be in mm/dd format.
                IsValid = IsDate(entryDetail & "/2004")
            Case "E"
                ' ----- The detail is a number from 1 to 7.
                If (Val(entryDetail) < 1) Or (Val(entryDetail) > 7) Then IsValid = False
            Case "O"
                ' ----- The detail should be a valid date.
                IsValid = IsDate(entryDetail)
            Case Else
                ' ---- Invalid. This should never happen.
                IsValid = False
        End Select
    End Sub

    Public Function IsHoliday(whatDate As Date) As Boolean
        ' ----- Given a date, see if it matches the entry type in this instance.
        Dim buildDate As String

        ' ----- If this record is invalid, then it is never a holiday match.
        If (IsValid = False) Then Return False

        Select Case HolidayType
            Case "A"
                ' ----- Annual.
                buildDate = HolidayDetail & "/" & whatDate.Year
                If (IsDate(buildDate)) Then
                    Return CBool(CDate(buildDate) = whatDate)
                Else
                    ' ----- Must be 2/29 on a non-leap-year.
                    Return False
                End If
            Case "E"
                ' ----- Day of the week.
                Return CBool(Val(HolidayDetail) = CInt(whatDate.DayOfWeek))
            Case "O"
                ' ----- See if this is an exact one-time match.
                Return CBool(CDate(HolidayDetail) = whatDate)
        End Select
        Return False
    End Function
End Class
