﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class ItemAuthorEdit
    Private ItemID As Long
    Private AuthorID As Long
    Private OrigTypeID As Long

    Public Function EditRecord(useItemID As Long, useAuthorID As Long,
            useItemName As String, useAuthorName As String,
            useOrigNameType As Long) As Long
        ' ----- Edit the name type of an existing item/author relationship.
        '       If the user changes the name type, return the name type ID.
        '       Otherwise, return -1.
        ItemID = useItemID
        AuthorID = useAuthorID
        OrigTypeID = useOrigNameType
        ItemName.Text = useItemName
        AuthorName.Text = useAuthorName
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return CLng(DirectCast(NameType.SelectedItem, ListItemData)) Else Return -1&
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the database.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        Me.Cursor = Cursors.WaitCursor

        ' ----- Update the record in the database.
        sqlText = "UPDATE ItemAuthor SET AuthorType = @NewType " &
            "WHERE ItemID = @WhichItem  AND AuthorID = @WhichAuthor"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("NewType", DBComboParam(NameType))
        sqlRun.AddLong("WhichItem", ItemID)
        sqlRun.AddLong("WhichAuthor", AuthorID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("ItemAuthorEdit.SaveFormData", ex)
            Return False
        Finally
            Me.Cursor = Cursors.Default
        End Try

        ' ----- Success.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- The name type must be selected.
        If (CLng(DirectCast(NameType.SelectedItem, ListItemData)) = -1&) Then
            MessageBox.Show("Please select a name type.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            NameType.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub ItemAuthorEdit_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim dbInfo As DataTable

        ' ----- Load in the list of name types.
        NameType.Items.Add(New ListItemData("<Select a Name Type>", -1&))
        NameType.SelectedIndex = 0
        sqlText = "SELECT * FROM CodeAuthorType ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("ItemAuthorEdit.ItemAuthorEdit_Load", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            NameType.Items.Add(New ListItemData(DBGetText(dbRow!FullName),
                DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Set the existing name type.
        NameType.SelectedIndex = NameType.Items.IndexOf(OrigTypeID)
        If (NameType.SelectedIndex = -1) Then _
            NameType.SelectedIndex = NameType.Items.IndexOf(-1&)
    End Sub
End Class