﻿' ----- The Library Project: License Generator
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Module General
    Public Const ProgramTitle As String = "Library License Generator"
    Public Const ApplicationProductName As String = "Library Project"

    ' ----- The filenames for the two keys are static.
    Public Const PublicKeyFilename As String = "LibraryPublicKey.xml"
    Public Const PrivateKeyFilename As String = "LibraryPrivateKey.xml"

    ' ----- Indicates the status of the directory containing the
    '       public and private keys.
    Public Enum KeyFolderStatus
        NoFolder
        InvalidFolder
        MissingBothFiles
        MissingPrivateFile
        MissingPublicFile
        ValidFolder
    End Enum

    Public Function AttemptCreateFolder(whichFolder As String) As Boolean
        ' ----- Ask the user to create a folder, and create it.
        '       Return True on success.
        If (whichFolder.Trim().Length = 0) Then Return False
        If (System.IO.Directory.Exists(whichFolder) = True) Then Return True

        ' ----- Prompt the user.
        If (MessageBox.Show("The directory" & vbCrLf & vbCrLf & vbTab & whichFolder &
            vbCrLf & vbCrLf & "does not exist. Do you wish to create it?",
            ProgramTitle, MessageBoxButton.YesNo, MessageBoxImage.Question) <>
            MessageBoxResult.Yes) Then Return False

        ' ----- Create the directory.
        Try
            System.IO.Directory.CreateDirectory(whichFolder)
        Catch ex As Exception
            MessageBox.Show("Could not create the directory due to error: " & ex.Message,
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Public Function GetFolderStatus(whichFolder As String) As KeyFolderStatus
        ' ----- Given a folder, return the status.
        Dim usePath As String
        Dim privateExists As Boolean
        Dim publicExists As Boolean

        ' ----- Was anything even supplied?
        If (whichFolder.Trim().Length = 0) Then Return KeyFolderStatus.NoFolder

        ' ----- Is this a valid directory.
        If (System.IO.Directory.Exists(whichFolder) = False) Then _
            Return KeyFolderStatus.InvalidFolder

        ' ----- Check for the files.
        usePath = System.IO.Path.Combine(whichFolder, PublicKeyFilename)
        publicExists = System.IO.File.Exists(usePath)
        usePath = System.IO.Path.Combine(whichFolder, PrivateKeyFilename)
        privateExists = System.IO.File.Exists(usePath)

        ' ----- Return the file status.
        If (publicExists = False) And (privateExists = False) Then
            Return KeyFolderStatus.MissingBothFiles
        ElseIf (publicExists = False) Then
            Return KeyFolderStatus.MissingPublicFile
        ElseIf (privateExists = False) Then
            Return KeyFolderStatus.MissingPrivateFile
        Else
            Return KeyFolderStatus.ValidFolder
        End If
    End Function

    Public Function GetFolderStatusText(whichStatus As KeyFolderStatus) As String
        ' ----- Return a description of the current status.
        Select Case whichStatus
            Case KeyFolderStatus.NoFolder
                Return "No directory specified."
            Case KeyFolderStatus.InvalidFolder
                Return "Invalid directory specified."
            Case KeyFolderStatus.MissingPublicFile
                Return "The directory lacks a public key file."
            Case KeyFolderStatus.MissingPrivateFile
                Return "The directory lacks a private key file."
            Case KeyFolderStatus.MissingBothFiles
                Return "The directory lacks the two key files."
            Case KeyFolderStatus.ValidFolder
                Return "The keys are ready to use."
            Case Else
                Return "Unknown."
        End Select
    End Function
End Module
