﻿' ----- The Library Project: License Generator
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Imports System.Security.Cryptography

Public Class KeyLocationWindow
    Public Function GetKeyFolder(ByRef folderPath As String) As Boolean
        ' ----- Prompt the user for the key folder.
        KeyDirectory.Text = folderPath
        RefreshFolderStatus(GetFolderStatus(folderPath))
        If (Me.ShowDialog() = True) Then
            ' ----- Return the new user selection.
            folderPath = KeyDirectory.Text
            Return True
        Else
            Return False
        End If
    End Function

    Private Sub RefreshFolderStatus(whichStatus As KeyFolderStatus)
        ' ----- Display a description of the current status.
        If (whichStatus = KeyFolderStatus.ValidFolder) Then
            StatusDetailColor.Fill = Brushes.Green
        Else
            StatusDetailColor.Fill = Brushes.Red
        End If
        Me.StatusDetailText.Content = GetFolderStatusText(whichStatus)
    End Sub

    Private Sub ActBrowse_Click(sender As Object, e As RoutedEventArgs) Handles ActBrowse.Click
        ' ----- Let the user browse for the folder through a system dialog.
        Dim browseKeyFolder As New System.Windows.Forms.FolderBrowserDialog

        If (browseKeyFolder.ShowDialog() = MessageBoxResult.OK) Then
            KeyDirectory.Text = browseKeyFolder.SelectedPath
            ActConfirm.RaiseEvent(New RoutedEventArgs(Controls.Primitives.ButtonBase.ClickEvent))
        End If
    End Sub

    Private Sub ActConfirm_Click(sender As Object, e As RoutedEventArgs) Handles ActConfirm.Click
        ' ----- Update the status.
        RefreshFolderStatus(GetFolderStatus(KeyDirectory.Text))
    End Sub

    Private Sub ActGenerate_Click(sender As Object, e As RoutedEventArgs) Handles ActGenerate.Click
        ' ----- Generate and save the keys.
        Dim twoPartKey As RSA
        Dim currentStatus As KeyFolderStatus
        Dim publicFile As String
        Dim privateFile As String

        ' ----- Determine the directory and file status.
        currentStatus = GetFolderStatus(KeyDirectory.Text)
        If (currentStatus = KeyFolderStatus.NoFolder) Then
            MessageBox.Show("Please supply a directory path.",
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            KeyDirectory.Focus()
            Return
        End If

        ' ----- Make sure the directory exists.
        If (currentStatus = KeyFolderStatus.InvalidFolder) Then
            If (AttemptCreateFolder(KeyDirectory.Text) = False) Then Return
            currentStatus = KeyFolderStatus.MissingBothFiles
            RefreshFolderStatus(currentStatus)
        End If

        ' ----- Warn if either file exists.
        If (currentStatus = KeyFolderStatus.ValidFolder) Then
            If (MessageBox.Show("One or both of the key files already exists. " &
                "Do you really want to overwrite the existing file(s)? " &
                "This action cannot be undone.",
                ProgramTitle, MessageBoxButton.YesNo, MessageBoxImage.Question,
                MessageBoxResult.No) <> MessageBoxResult.Yes) Then Return
        End If

        ' ----- Delete the existing files.
        Try
            publicFile = System.IO.Path.Combine(KeyDirectory.Text, PublicKeyFilename)
            privateFile = System.IO.Path.Combine(KeyDirectory.Text, PrivateKeyFilename)
        Catch ex As Exception
            MessageBox.Show("Error occurred while locating files:" &
                vbCrLf & vbCrLf & Err.Description,
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            Return
        End Try
        Try
            System.IO.File.Delete(publicFile)
            System.IO.File.Delete(privateFile)
        Catch
            ' ----- Ignore errors.
        End Try
        RefreshFolderStatus(KeyFolderStatus.MissingBothFiles)

        ' ----- Generate the keys.
        twoPartKey = New RSACryptoServiceProvider
        twoPartKey = RSA.Create()

        ' ----- Save the public key.
        Try
            System.IO.File.WriteAllText(publicFile, twoPartKey.ToXmlString(False))
        Catch ex As Exception
            MessageBox.Show("Error occurred while generating public key:" &
                vbCrLf & vbCrLf & Err.Description,
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            Return
        End Try
        RefreshFolderStatus(KeyFolderStatus.MissingPrivateFile)

        ' ----- Save the private key.
        Try
            System.IO.File.WriteAllText(privateFile, twoPartKey.ToXmlString(True))
        Catch ex As Exception
            MessageBox.Show("Error occurred while generating private key:" &
                vbCrLf & vbCrLf & Err.Description,
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            Return
        End Try
        RefreshFolderStatus(KeyFolderStatus.ValidFolder)

        MessageBox.Show("Key files generated successfully.",
            ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Information)
    End Sub

    Private Sub ActOK_Click(sender As Object, e As RoutedEventArgs) Handles ActOK.Click
        ' ----- Return the key folder to the calling form.
        Me.DialogResult = True
    End Sub

    Private Sub ActCancel_Click(sender As Object, e As RoutedEventArgs) Handles ActCancel.Click
        ' ----- Close the form without confirming location.
        Me.DialogResult = False
    End Sub
End Class
