﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class NamedItem
    Private ActiveID As Long
    Private StartingOver As Boolean
    Private PublisherID As Long
    Private SeriesID As Long
    Private SessionSaved As Boolean

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Or (SessionSaved = True) _
            Then Return ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManageItems)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManageItems)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManageItems)
    End Function

    Private Function DeleteOneKeyword(keywordID As Long) As Boolean
        ' ----- Remove a keyword from this record. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Delete the record.
        sqlText = "DELETE FROM ItemKeyword WHERE ItemID = @RecordID" &
            " AND KeywordID = @KeyID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        sqlRun.AddLong("KeyID", keywordID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.DeleteOneKeyword.Delete", ex)
            Return False
        End Try

        ' ----- If no books are using this keyword, remove it.
        sqlText = "SELECT COUNT(*) FROM ItemKeyword WHERE KeywordID = @KeyID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("KeyID", keywordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) = 0) Then
                sqlText = "DELETE FROM Keyword WHERE ID = @KeyID"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddLong("KeyID", keywordID)
                ExecuteSQL(sqlRun)
            End If
        Catch ex As Exception
            GeneralError("NamedItem.DeleteOneKeyword.Books", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim counter As Integer
        Dim transInfo As LibraryTransaction = Nothing

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the item?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE ItemID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("You cannot delete this record because it has copies.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        Catch ex As Exception
            GeneralError("NamedItem.DeleteRecord.Check", ex)
            Return False
        End Try

        ' ----- Delete each of the keywords.
        For counter = 0 To KeywordList.Items.Count - 1
            DeleteOneKeyword(CLng(DirectCast(KeywordList.Items(counter), ListItemData)))
        Next counter

        ' ----- Delete the records.
        Try
            transInfo = TransactionBegin()

            sqlText = "DELETE FROM ItemAuthor WHERE ItemID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            sqlText = "DELETE FROM NamedItem WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            GeneralError("NamedItem.DeleteRecord.Delete", ex)
            Return False
        End Try

        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Or (SessionSaved = True) _
            Then Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items. Prompt the
        '       user first.
        destList.Items.Clear()
        SearchForRecord(destList, exceededMatches)
    End Sub

    Private Function FormatItemName(dbRow As DataRow) As String
        ' ----- Given an item record, return the formatted name.
        Dim nameText As String

        On Error Resume Next

        ' ----- Format the name.
        nameText = DBGetText(dbRow!Title)
        If (IsDBNull(dbRow!Subtitle) = False) Then _
            nameText &= ": " & DBGetText(dbRow!Subtitle)
        If (IsDBNull(dbRow!Edition) = False) Then _
            nameText &= ", " & DBGetText(dbRow!Edition)
        nameText &= " [" & DBGetText(dbRow!MediaName) & "]"

        ' ----- Finished.
        Return nameText
    End Function

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim result As String

        ' ----- Initialize.
        result = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT NI.ID, NI.Title, NI.Subtitle, NI.Edition, CMT.FullName " &
            "AS MediaName FROM NamedItem AS NI INNER JOIN CodeMediaType AS CMT " &
            "ON NI.MediaType = CMT.ID WHERE NI.ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.FormatRecordName", ex)
            Return "Error"
        End Try
        If (dbInfo.Rows.Count > 0) Then result = FormatItemName(dbInfo.Rows(0))
        dbInfo.Dispose()
        dbInfo = Nothing
        Return result
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use this list to identify the items found in the library. " &
            "Each item may include more than one copy."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Named Items"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim newItem As ListViewItem
        Dim holdText As String
        Dim counter As Integer

        ' ----- Load in the list of security groups.
        RecordMediaType.Items.Clear()
        RecordMediaType.Items.Add(New ListItemData("<Not Selected>", -1&))
        sqlText = "SELECT * FROM CodeMediaType ORDER BY FullName"
        Try
            dbInfo = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("NamedItem.PrepareFormFields.Groups", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            RecordMediaType.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Clear the existing data.
        StartingOver = True
        RecordID.Text = "Not Available"
        RecordTitle.Text = ""
        RecordSubtitle.Text = ""
        RecordDescription.Text = ""
        RecordCopyright.Text = ""
        RecordEdition.Text = ""
        RecordPublisher.Text = "Not Available"
        PublisherID = -1&
        RecordDewey.Text = ""
        RecordLC.Text = ""
        RecordLCCN.Text = ""
        RecordISxN.Text = ""
        RecordMediaType.SelectedIndex = RecordMediaType.Items.IndexOf(-1&)
        RecordOutOfPrint.Checked = False
        RecordSeries.Text = "No Series"
        SeriesID = -1&
        AuthorsList.Items.Clear()
        ActAuthorProperties.Enabled = False
        ActAuthorDelete.Enabled = False
        ActAuthorMoveUp.Enabled = False
        ActAuthorMoveDown.Enabled = False
        SubjectList.Items.Clear()
        ActSubjectDelete.Enabled = False
        ActSubjectProperties.Enabled = False
        KeywordList.Items.Clear()
        ActKeywordDelete.Enabled = False
        ActKeywordProperties.Enabled = False
        CopyList.Items.Clear()
        ActCopyEdit.Enabled = False
        ActCopyDelete.Enabled = False
        ActCancel.Text = "Cancel"
        TabNamedItem.SelectedTab = TabGeneral
        SessionSaved = False
        If (ActiveID = -1&) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT NI.*, PB.FullName AS PubName, CS.FullName As SeriesName " &
            "FROM NamedItem AS NI LEFT JOIN Publisher AS PB ON NI.Publisher = PB.ID " &
            "LEFT JOIN CodeSeries AS CS ON NI.Series = CS.ID " &
            "WHERE NI.ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.PrepareFormFields.Retrieve", ex)
            Return
        End Try
        If (dbInfo.Rows.Count > 0) Then
            oneRow = dbInfo.Rows(0)
            RecordID.Text = DBGetLong(oneRow!ID).ToString()
            RecordTitle.Text = DBGetText(oneRow!Title)
            RecordSubtitle.Text = DBGetText(oneRow!Subtitle)
            RecordDescription.Text = DBGetText(oneRow!Description)
            RecordCopyright.Text = DBGetText(oneRow!Copyright)
            RecordEdition.Text = DBGetText(oneRow!Edition)
            If (IsDBNull(oneRow!PubName) = False) Then _
                RecordPublisher.Text = DBGetText(oneRow!PubName)
            If (IsDBNull(oneRow!Publisher) = False) Then _
                PublisherID = DBGetLong(oneRow!Publisher)
            RecordDewey.Text = DBGetText(oneRow!Dewey)
            RecordLC.Text = DBGetText(oneRow!LC)
            RecordLCCN.Text = DBGetText(oneRow!LCCN)
            RecordISxN.Text = DBGetText(oneRow!ISxN)
            RecordMediaType.SelectedIndex = RecordMediaType.Items.IndexOf(
                DBGetLong(oneRow!MediaType))
            RecordOutOfPrint.Checked = DBGetBool(oneRow!OutOfPrint)
            If (IsDBNull(oneRow!SeriesName) = False) Then _
                RecordSeries.Text = DBGetText(oneRow!SeriesName)
            If (IsDBNull(oneRow!Series) = False) Then _
                SeriesID = DBGetLong(oneRow!Series)
        End If
        dbInfo.Dispose()

        ' ----- Load in the authors for this item.
        sqlText = "SELECT AU.*, CT.FullName AS TypeName, CT.ID AS TypeID " &
            "FROM ItemAuthor AS IA INNER JOIN Author AS AU ON IA.AuthorID = AU.ID " &
            "INNER JOIN CodeAuthorType AS CT ON IA.AuthorType = CT.ID " &
            "WHERE IA.ItemID = @RecordID ORDER BY IA.Sequence"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.PrepareFormFields.Authors", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            newItem = AuthorsList.Items.Add(dbRow.FormatAuthorName())
            newItem.SubItems.Add(DBGetText(dbRow!TypeName))
            newItem.Tag = DBGetLong(dbRow!ID).ToString() & "," &
                DBGetLong(dbRow!TypeID).ToString()
        Next dbRow
        dbInfo.Dispose()

        ' ----- Load in the subjects for this item.
        sqlText = "SELECT SB.* FROM ItemSubject AS IJ INNER JOIN Subject AS SB " &
            "ON IJ.SubjectID = SB.ID WHERE IJ.ItemID = @RecordID " &
            "ORDER BY SB.FullName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.PrepareFormFields.Subjects", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            SubjectList.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()

        ' ----- Load in the keywords for this item.
        sqlText = "SELECT KW.* FROM ItemKeyword AS IK INNER JOIN Keyword AS KW " &
            "ON IK.KeywordID = KW.ID WHERE IK.ItemID = @RecordID " &
            "ORDER BY KW.FullName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.PrepareFormFields.Keywords", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            KeywordList.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()

        ' ----- Load in the copies for this item.
        sqlText = "SELECT ID, CopyNumber, Barcode FROM ItemCopy WHERE ItemID = @RecordID " &
            "ORDER BY CopyNumber"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.PrepareFormFields.Copies", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            newItem = CopyList.Items.Add(DBGetInteger(dbRow!CopyNumber).ToString())
            newItem.SubItems.Add("")  ' Add disposition later
            holdText = DBGetText(dbRow!Barcode)
            If (holdText.Length = 0) Then holdText = "None"
            newItem.SubItems.Add(holdText)
            newItem.Tag = DBGetLong(dbRow!ID).ToString()
        Next dbRow
        dbInfo.Dispose()

        ' ----- Add the disposition for each copy. We can't do it in the
        '       earlier loop since that would cause two OleDB readers
        '       to be opened at once, and that doesn't work.
        For counter = 0 To CopyList.Items.Count - 1
            newItem = CopyList.Items(counter)
            newItem.SubItems(1).Text = GetCopyDisposition(CLng(newItem.Tag))
        Next counter

        ' ----- Finished.
        dbInfo = Nothing
    End Sub

    Private Function SaveFormData() As Boolean
        ' ----- Save the changes to the database.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long

        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO NamedItem (Title, Subtitle, Description, " &
                "Copyright, Edition, Publisher, Dewey, LC, LCCN, ISxN, MediaType, " &
                "OutOfPrint, Series) OUTPUT INSERTED.ID VALUES (@NewTitle, @NewSubtitle, " &
                "@NewDesc, @NewCopyright, @NewEdition, @NewPub, @NewDewey, @NewLC, " &
                "@NewLCCN, @NewISXN, @NewMedia, @NewPrint, @NewSeries)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewTitle", RecordTitle.Text.Trim())
            sqlRun.AddText("NewSubtitle", RecordSubtitle.Text.Trim())
            sqlRun.AddText("NewDesc", RecordDescription.Text.Trim())
            sqlRun.AddInteger("NewCopyright", RecordCopyright.Text.Trim())
            sqlRun.AddText("NewEdition", RecordEdition.Text.Trim())
            sqlRun.AddID("NewPub", PublisherID)
            sqlRun.AddText("NewDewey", RecordDewey.Text.Trim())
            sqlRun.AddText("NewLC", RecordLC.Text.Trim())
            sqlRun.AddText("NewLCCN", RecordLCCN.Text.Trim())
            sqlRun.AddText("NewISXN", RecordISxN.Text.Trim())
            sqlRun.AddLong("NewMedia", DBComboParam(RecordMediaType))
            sqlRun.AddBool("NewPrint", RecordOutOfPrint.Checked)
            sqlRun.AddID("NewSeries", SeriesID)
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                GeneralError("NamedItem.SaveFormData.Add", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE NamedItem SET Title = @NewTitle, " &
                "Subtitle = @NewSubtitle, Description = @NewDesc, " &
                "Copyright = @NewCopyright, Edition = @NewEdition, " &
                "Publisher = @NewPub, Dewey = @NewDewey, " &
                "LC = @NewLC, LCCN = @NewLCCN, ISxN = @NewISXN, " &
                "MediaType = @NewMedia, OutOfPrint = @NewPrint, " &
                "Series = @NewSeries WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewTitle", RecordTitle.Text.Trim())
            sqlRun.AddText("NewSubtitle", RecordSubtitle.Text.Trim())
            sqlRun.AddText("NewDesc", RecordDescription.Text.Trim())
            sqlRun.AddInteger("NewCopyright", RecordCopyright.Text.Trim())
            sqlRun.AddText("NewEdition", RecordEdition.Text.Trim())
            sqlRun.AddID("NewPub", PublisherID)
            sqlRun.AddText("NewDewey", RecordDewey.Text.Trim())
            sqlRun.AddText("NewLC", RecordLC.Text.Trim())
            sqlRun.AddText("NewLCCN", RecordLCCN.Text.Trim())
            sqlRun.AddText("NewISXN", RecordISxN.Text.Trim())
            sqlRun.AddLong("NewMedia", DBComboParam(RecordMediaType))
            sqlRun.AddBool("NewPrint", RecordOutOfPrint.Checked)
            sqlRun.AddID("NewSeries", SeriesID)
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("NamedItem.SaveFormData.Update", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Success.
        Me.Cursor = Cursors.Default
        ActCancel.Text = "Close"
        SessionSaved = True
        Return True
    End Function

    Public Overrides Sub SearchForRecord(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Search for items using user-specified criteria.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim userLimit As String
        Dim resultCount As Integer

        ' ----- Prompt the user for the limited item title.
        exceededMatches = False
        userLimit = (New ItemLimit).PromptUser()
        If (userLimit.Length = 0) Then Return
        If (userLimit.Contains("*") = False) Then userLimit &= "*"

        ' ----- Clear any existing values.
        destList.Items.Clear()
        resultCount = 0

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT NI.ID, NI.Title, NI.Subtitle, NI.Edition, CMT.FullName " &
            "AS MediaName FROM NamedItem AS NI INNER JOIN CodeMediaType AS CMT " &
            "ON NI.MediaType = CMT.ID WHERE NI.Title LIKE @TestTitle " &
            "ORDER BY NI.Title, NI.Subtitle, NI.Edition"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestTitle", userLimit.Trim().Replace("*", "%"))
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.SearchForRecord", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            resultCount += 1
            If (resultCount > SearchMatchLimit) Then
                exceededMatches = True
                Exit For
            End If
            destList.Items.Add(New ListItemData(
                FormatItemName(dbRow), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function UsesSearch() As Boolean
        ' ----- NamedItem records are not searchable.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        '       Title is a required field.
        If (RecordTitle.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the Title.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            TabNamedItem.SelectedTab = TabGeneral
            RecordTitle.Focus()
            Return False
        End If

        ' ----- Media Type is a required field.
        If (CLng(DirectCast(RecordMediaType.SelectedItem, ListItemData)) = -1&) Then
            MessageBox.Show("Please supply the Media Type.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            TabNamedItem.SelectedTab = TabClassification
            RecordMediaType.Focus()
            Return False
        End If

        ' ----- Copyright Year, if supplied, must be numeric.
        If (RecordCopyright.Text.Trim().Length <> 0) Then
            If (IsNumeric(RecordCopyright.Text) = False) Or
                    (RecordCopyright.Text.Contains(".") = True) Then
                MessageBox.Show("Copyright Year, if supplied, must be a valid integer.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                TabNamedItem.SelectedTab = TabGeneral
                RecordCopyright.Focus()
                Return False
            End If
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub NamedItem_Activated(sender As Object, e As EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then
            TabNamedItem.SelectedTab = TabGeneral
            RecordTitle.Focus()
        End If
        StartingOver = False
    End Sub

    Private Sub NamedItem_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "NamedItem.htm")
    End Sub

    Private Sub NamedItem_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Adjust the author list columns.
        ColAuthorName.Width = LabelColNameType.Left - LabelColAuthor.Left
        ColAuthorNameType.Width = (AuthorsList.Left + AuthorsList.Width) -
            LabelColNameType.Left - 12

        ' ----- Adjust the copy list columns.
        ColCopyNumber.Width = LabelColCopyStatus.Left - LabelColCopy.Left
        ColCopyStatus.Width = LabelColCopyBarcode.Left - LabelColCopyStatus.Left
        ColCopyBarcode.Width = (CopyList.Left + CopyList.Width) -
            LabelColCopyBarcode.Left - 12
    End Sub

    Private Sub RecordCopyright_Enter(sender As Object, e As EventArgs) Handles RecordCopyright.Enter
        ' ----- Highlight the entire text.
        RecordCopyright.SelectAll()
    End Sub

    Private Sub RecordDewey_Enter(sender As Object, e As EventArgs) Handles RecordDewey.Enter
        ' ----- Highlight the entire text.
        RecordDewey.SelectAll()
    End Sub

    Private Sub RecordEdition_Enter(sender As Object, e As EventArgs) Handles RecordEdition.Enter
        ' ----- Highlight the entire text.
        RecordEdition.SelectAll()
    End Sub

    Private Sub RecordISxN_Enter(sender As Object, e As EventArgs) Handles RecordISxN.Enter
        ' ----- Highlight the entire text.
        RecordISxN.SelectAll()
    End Sub

    Private Sub RecordLC_Enter(sender As Object, e As EventArgs) Handles RecordLC.Enter
        ' ----- Highlight the entire text.
        RecordLC.SelectAll()
    End Sub

    Private Sub RecordLCCN_Enter(sender As Object, e As EventArgs) Handles RecordLCCN.Enter
        ' ----- Highlight the entire text.
        RecordLCCN.SelectAll()
    End Sub

    Private Sub RecordSubtitle_Enter(sender As Object, e As EventArgs) Handles RecordSubtitle.Enter
        ' ----- Highlight the entire text.
        RecordSubtitle.SelectAll()
    End Sub

    Private Sub RecordTitle_Enter(sender As Object, e As EventArgs) Handles RecordTitle.Enter
        ' ----- Highlight the entire text.
        RecordTitle.SelectAll()
    End Sub

    Private Sub RecordCopyright_KeyPress(sender As Object, e As KeyPressEventArgs) Handles RecordCopyright.KeyPress
        ' ----- Only allow digits and backspaces.
        If (e.KeyChar = vbBack) Or (IsNumeric(e.KeyChar)) Then Return
        e.Handled = True
    End Sub

    Private Sub ActSelectPublisher_Click(sender As Object, e As EventArgs) Handles ActSelectPublisher.Click
        ' ----- Get a new publisher.
        Dim newPublisher As Long
        Dim displayText As String

        ' ----- Prompt the user.
        newPublisher = (New PublisherAddLocate).PromptUser()
        If (newPublisher = -1&) Then Return

        ' ----- Check to clear the publisher.
        If (newPublisher = -2&) Then
            RecordPublisher.Text = "Not Available"
            PublisherID = -1&
            Return
        End If

        ' ----- Save and display the publisher.
        PublisherID = newPublisher
        displayText = (New Publisher).FormatRecordName(PublisherID)
        RecordPublisher.Text = displayText
    End Sub

    Private Sub ActSelectSeries_Click(sender As Object, e As EventArgs) Handles ActSelectSeries.Click
        ' ----- Get a new series.
        Dim newSeries As Long
        Dim displayText As String

        ' ----- Prompt the user.
        newSeries = (New SeriesAddLocate).PromptUser()
        If (newSeries = -1&) Then Return

        ' ----- Check to clear the series.
        If (newSeries = -2&) Then
            RecordSeries.Text = "No Series"
            SeriesID = -1&
            Return
        End If

        ' ----- Save and display the series.
        SeriesID = newSeries
        displayText = (New CodeSeries).FormatRecordName(SeriesID)
        RecordSeries.Text = displayText
    End Sub

    Private Sub ActKeywordDelete_Click(sender As Object, e As EventArgs) Handles ActKeywordDelete.Click
        ' ----- Delete the selected keyword.
        If (KeywordList.SelectedIndex = -1) Then Return

        ' ----- Confirm with the user.
        If (MessageBox.Show("Really delete the keyword '" &
            KeywordList.Text & "' from this item?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return

        ' ----- Proceed with the removal.
        If (DeleteOneKeyword(CLng(DirectCast(KeywordList.SelectedItem, ListItemData))) =
            False) Then Return

        ' ----- Remove this item from the display.
        KeywordList.Items.Remove(KeywordList.SelectedItem)
        KeywordList.SelectedIndex = -1
        ActKeywordDelete.Enabled = False
        ActKeywordProperties.Enabled = False
        ActCancel.Text = "Close"
        SessionSaved = True
    End Sub

    Private Sub ActKeywordAdd_Click(sender As Object, e As EventArgs) Handles ActKeywordAdd.Click
        ' ----- Let the user add a new keyword.
        Dim keywordID As Long
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- The record must be saved first.
        If (ActiveID = -1&) Then
            ' ----- Confirm with the user.
            If (MessageBox.Show("The item must be saved to the database before keywords " &
                "can be added. Would you like to save the record now?",
                ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
                DialogResult.Yes) Then Return

            ' ----- Verify and save the data.
            If (ValidateFormData() = False) Then Return
            If (SaveFormData() = False) Then Return
        End If

        ' ----- Prompt the user to add a keyword.
        keywordID = (New KeywordAdd).AddRecord()
        If (keywordID = -1&) Then Return

        ' ----- See if the record already exists.
        If (KeywordList.Items.IndexOf(keywordID) <> -1) Then Return

        ' ----- Add the keyword to the database record.
        sqlText = "INSERT INTO ItemKeyword (ItemID, KeywordID) " &
            "VALUES (@RecordID, @KeyID)"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        sqlRun.AddLong("KeyID", keywordID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.ActKeywordAdd_Click.Add", ex)
            Return
        End Try
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Add this item to the display.
        sqlText = "SELECT FullName FROM Keyword WHERE ID = @KeyID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("KeyID", keywordID)
        Try
            KeywordList.Items.Add(New ListItemData(
                DBGetText(ExecuteSQLReturn(sqlRun)), keywordID))
        Catch ex As Exception
            GeneralError("NamedItem.ActKeywordAdd_Click.Name", ex)
            Return
        End Try
    End Sub

    Private Sub KeywordList_SelectedIndexChanged(sender As Object, e As EventArgs) Handles KeywordList.SelectedIndexChanged
        ' ----- Enable buttons as needed.
        ActKeywordDelete.Enabled = (KeywordList.SelectedIndex <> -1)
        ActKeywordProperties.Enabled = ActKeywordDelete.Enabled
    End Sub

    Private Sub KeywordList_Click(sender As Object, e As EventArgs) Handles KeywordList.Click
        ' ----- Enable buttons as needed.
        ActKeywordDelete.Enabled = (KeywordList.SelectedIndex <> -1)
        ActKeywordProperties.Enabled = ActKeywordDelete.Enabled
    End Sub

    Private Sub SubjectList_SelectedIndexChanged(sender As Object, e As EventArgs) Handles SubjectList.SelectedIndexChanged
        ' ----- Enable buttons as needed.
        ActSubjectDelete.Enabled = (SubjectList.SelectedIndex <> -1)
        ActSubjectProperties.Enabled = ActSubjectDelete.Enabled
    End Sub

    Private Sub SubjectList_Click(sender As Object, e As EventArgs) Handles SubjectList.Click
        ' ----- Enable buttons as needed.
        ActSubjectDelete.Enabled = (SubjectList.SelectedIndex <> -1)
        ActSubjectProperties.Enabled = ActSubjectDelete.Enabled
    End Sub

    Private Sub ActSubjectDelete_Click(sender As Object, e As EventArgs) Handles ActSubjectDelete.Click
        ' ----- Delete the selected subject.
        If (SubjectList.SelectedIndex = -1) Then Return

        ' ----- Confirm with the user.
        If (MessageBox.Show("Really delete the subject '" &
            SubjectList.Text & "' from this item?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return

        ' ----- Proceed with the removal.
        If (DeleteOneSubject(CLng(DirectCast(SubjectList.SelectedItem, ListItemData))) =
            False) Then Return

        ' ----- Remove this item from the display.
        SubjectList.Items.Remove(SubjectList.SelectedItem)
        SubjectList.SelectedIndex = -1
        ActSubjectDelete.Enabled = False
        ActSubjectProperties.Enabled = False
        ActCancel.Text = "Close"
        SessionSaved = True
    End Sub

    Private Function DeleteOneSubject(subjectID As Long) As Boolean
        ' ----- Remove a subject from this record. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Delete the record.
        sqlText = "DELETE FROM ItemSubject WHERE ItemID = @RecordID " &
            "AND SubjectID = @SubID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        sqlRun.AddLong("SubID", subjectID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.DeleteOneSubject.Delete", ex)
            Return False
        End Try

        ' ----- If no books are using this subject, remove it.
        sqlText = "SELECT COUNT(*) FROM ItemSubject WHERE SubjectID = @SubID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("SubID", subjectID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) = 0) Then
                sqlText = "DELETE FROM Subject WHERE ID = @SubID"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddLong("SubID", subjectID)
                ExecuteSQL(sqlRun)
            End If
        Catch ex As Exception
            GeneralError("NamedItem.DeleteOneSubject.Subject", ex)
            Return False
        End Try
        Return True
    End Function

    Private Sub ActKeywordProperties_Click(sender As Object, e As EventArgs) Handles ActKeywordProperties.Click
        ' ----- Display some information about the selected keyword.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim inUse As Integer
        Dim message As String

        ' ----- Ignore if no item selected.
        If (KeywordList.SelectedIndex = -1) Then Return

        ' ----- Collect some statistics.
        sqlText = "SELECT COUNT(*) FROM ItemKeyword WHERE KeywordID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", CLng(DirectCast(KeywordList.SelectedItem, ListItemData)))
        Try
            inUse = DBGetInteger(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("NamedItem.ActKeywordProperties_Click", ex)
            Return
        End Try

        ' ----- Display an informational message.
        message = "Keyword: " & KeywordList.Text & vbCrLf & vbCrLf &
            "This keyword is in use by " & inUse & " library item"
        If (inUse = 1) Then message &= "." Else message &= "s."
        MessageBox.Show(message,
            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    Private Sub ActSubjectAdd_Click(sender As Object, e As EventArgs) Handles ActSubjectAdd.Click
        ' ----- Let the user add a new subject.
        Dim subjectID As Long
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- The record must be saved first.
        If (ActiveID = -1&) Then
            ' ----- Confirm with the user.
            If (MessageBox.Show("The item must be saved to the database before subjects " &
                "can be added. Would you like to save the record now?",
                ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
                DialogResult.Yes) Then Return

            ' ----- Verify and save the data.
            If (ValidateFormData() = False) Then Return
            If (SaveFormData() = False) Then Return
        End If

        ' ----- Prompt the user to add a subject.
        subjectID = (New SubjectAdd).AddRecord()
        If (subjectID = -1&) Then Return

        ' ----- See if the record already exists.
        If (SubjectList.Items.IndexOf(subjectID) <> -1) Then Return

        ' ----- Add the subject to the database record.
        sqlText = "INSERT INTO ItemSubject (ItemID, SubjectID) " &
            "VALUES (@RecordID, @SubID)"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        sqlRun.AddLong("SubID", subjectID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.ActSubjectAdd_Click.Add", ex)
            Return
        End Try
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Add this item to the display.
        sqlText = "SELECT FullName FROM Subject WHERE ID = @SubID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("SubID", subjectID)
        Try
            SubjectList.Items.Add(New ListItemData(
                DBGetText(ExecuteSQLReturn(sqlRun)), subjectID))
        Catch ex As Exception
            GeneralError("NamedItem.ActSubjectAdd_Click.Name", ex)
        End Try
    End Sub

    Private Sub ActSubjectProperties_Click(sender As Object, e As EventArgs) Handles ActSubjectProperties.Click
        ' ----- Display some information about the selected subject.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim inUse As Integer
        Dim message As String

        ' ----- Ignore if no item selected.
        If (SubjectList.SelectedIndex = -1) Then Return

        ' ----- Collect some statistics.
        sqlText = "SELECT COUNT(*) FROM ItemSubject WHERE SubjectID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", CLng(DirectCast(SubjectList.SelectedItem, ListItemData)))
        Try
            inUse = DBGetInteger(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("NamedItem.ActSubjectProperties_Click", ex)
            Return
        End Try

        ' ----- Display an informational message.
        message = "Subject: " & SubjectList.Text & vbCrLf & vbCrLf &
            "This subject is in use by " & inUse & " library item"
        If (inUse = 1) Then message &= "." Else message &= "s."
        MessageBox.Show(message,
            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    Private Sub SubjectList_DoubleClick(sender As Object, e As EventArgs) Handles SubjectList.DoubleClick
        ' ----- Same as the properties button.
        If (ActSubjectProperties.Enabled = True) Then ActSubjectProperties.PerformClick()
    End Sub

    Private Sub KeywordList_DoubleClick(sender As Object, e As EventArgs) Handles KeywordList.DoubleClick
        ' ----- Same as the properties button.
        If (ActKeywordProperties.Enabled = True) Then ActKeywordProperties.PerformClick()
    End Sub

    Private Sub ActAuthorAdd_Click(sender As Object, e As EventArgs) Handles ActAuthorAdd.Click
        ' ----- Let the user add a new author.
        Dim authorID As Long
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim authorText As String
        Dim nameTypeID As Long
        Dim typeText As String
        Dim sequenceID As Integer
        Dim newItem As ListViewItem

        ' ----- The record must be saved first.
        If (ActiveID = -1&) Then
            ' ----- Confirm with the user.
            If (MessageBox.Show("The item must be saved to the database before authors " &
                "or names can be added. Would you like to save the record now?",
                ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
                DialogResult.Yes) Then Return

            ' ----- Verify and save the data.
            If (ValidateFormData() = False) Then Return
            If (SaveFormData() = False) Then Return
        End If

        ' ----- Prompt the user to add a author.
        authorID = (New AuthorAddLocate).PromptUser(nameTypeID)
        If (authorID = -1&) Then Return

        ' ----- Jump out early if the record already exists.
        sqlText = "SELECT COUNT(*) FROM ItemAuthor " &
            "WHERE ItemID = @RecordID AND AuthorID = @AuthID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        sqlRun.AddLong("AuthID", authorID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then Return
        Catch ex As Exception
            GeneralError("NamedItem.ActSubjectAdd_Click.Check", ex)
            Return
        End Try
        Me.Cursor = Cursors.WaitCursor

        ' ----- Get the next sequence number.
        sequenceID = 1
        sqlText = "SELECT MAX(Sequence) AS TheMax " &
            "FROM ItemAuthor WHERE ItemID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("NamedItem.ActSubjectAdd_Click.Sequence", ex)
            Return
        End Try
        If (dbInfo.Rows.Count > 0) Then
            If (IsDBNull(dbInfo.Rows(0) !TheMax) = False) Then _
                sequenceID = DBGetInteger(dbInfo.Rows(0) !TheMax) + 1
        End If
        dbInfo.Dispose()

        ' ----- Add the author to the database record.
        sqlText = "INSERT INTO ItemAuthor (ItemID, AuthorID, Sequence, AuthorType) " &
            "VALUES (@NewItem, @NewAuthor, @NewSequence, @NewType)"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("NewItem", ActiveID)
        sqlRun.AddLong("NewAuthor", authorID)
        sqlRun.AddInteger("NewSequence", sequenceID)
        sqlRun.AddLong("NewType", nameTypeID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("NamedItem.ActSubjectAdd_Click.Add", ex)
            Return
        End Try
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Get the text of the name type.
        sqlText = "SELECT FullName FROM CodeAuthorType WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", nameTypeID)
        Try
            typeText = DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("NamedItem.ActSubjectAdd_Click.Type", ex)
            Return
        End Try

        ' ----- Get the author name.
        sqlText = "SELECT * FROM Author WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", authorID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("NamedItem.ActSubjectAdd_Click.Author", ex)
            Return
        End Try
        authorText = FormatAuthorName(dbInfo.Rows(0))
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Add this item to the display.
        newItem = AuthorsList.Items.Add(authorText)
        newItem.SubItems.Add(typeText)
        newItem.Tag = authorID.ToString() & "," & nameTypeID.ToString()
        UpdateAuthorButtons()
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub ActAuthorDelete_Click(sender As Object, e As EventArgs) Handles ActAuthorDelete.Click
        ' ----- Allow the user to delete an author entry.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Make sure an item is selected.
        If (AuthorsList.SelectedItems.Count = 0) Then Return

        ' ----- Confirm with the user.
        If (MessageBox.Show("Really disassociate the author '" &
            AuthorsList.SelectedItems(0).Text & "' from the library item?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return

        ' ----- Remove the selection from the database.
        sqlText = "DELETE FROM ItemAuthor WHERE ItemID = @RecordID " &
            "AND AuthorID = @AuthID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        sqlRun.AddLong("AuthID", CStr(AuthorsList.SelectedItems(0).Tag).GetSubStr(",", 1))
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.ActAuthorDelete_Click", ex)
            Return
        End Try
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Remove the item from the display.
        AuthorsList.Items.Remove(AuthorsList.SelectedItems(0))
        UpdateAuthorButtons()
    End Sub

    Private Sub ActAuthorMoveUp_Click(sender As Object, e As EventArgs) Handles ActAuthorMoveUp.Click
        ' ----- Move the selected author up in priority.
        If (AuthorsList.SelectedItems.Count = 0) Then Return
        If (AuthorsList.SelectedItems(0).Index = 0) Then Return

        SwitchTwoAuthors(AuthorsList.SelectedItems(0).Index,
            AuthorsList.SelectedItems(0).Index - 1)
    End Sub

    Private Sub ActAuthorMoveDown_Click(sender As Object, e As EventArgs) Handles ActAuthorMoveDown.Click
        ' ----- Move the selected author down in priority.
        If (AuthorsList.SelectedItems.Count = 0) Then Return
        If (AuthorsList.SelectedItems(0).Index = (AuthorsList.Items.Count - 1)) _
            Then Return

        SwitchTwoAuthors(AuthorsList.SelectedItems(0).Index,
            AuthorsList.SelectedItems(0).Index + 1)
    End Sub

    Private Sub SwitchTwoAuthors(activeIndex As Integer, inactiveIndex As Integer)
        ' ----- Given the indexes of two items in the AuthorsList control,
        '       switch them in the database and on-screen.
        Dim activeAuthor As Long
        Dim inactiveAuthor As Long
        Dim activeSequence As Integer
        Dim inactiveSequence As Integer
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim activeItem As ListViewItem
        Dim inactiveItem As ListViewItem
        Dim holdText As String
        Dim transInfo As LibraryTransaction = Nothing

        ' ----- Get the author IDs from the two items.
        activeItem = AuthorsList.Items(activeIndex)
        inactiveItem = AuthorsList.Items(inactiveIndex)
        activeAuthor = CLng(CStr(activeItem.Tag).GetSubStr(",", 1))
        inactiveAuthor = CLng(CStr(inactiveItem.Tag).GetSubStr(",", 1))

        ' ----- Get the sequence numbers of the two records.
        activeSequence = -1
        inactiveSequence = -1
        sqlText = "SELECT AuthorID, Sequence FROM ItemAuthor " &
            "WHERE ItemID = @RecordID AND AuthorID IN (@Author1, @Author2)"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        sqlRun.AddLong("Author1", activeAuthor)
        sqlRun.AddLong("Author2", inactiveAuthor)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.SwitchTwoAuthors.Sequence", ex)
            Return
        End Try
        For Each dbRow As DataRow In dbInfo.Rows
            If (DBGetLong(dbRow!AuthorID) = activeAuthor) Then
                activeSequence = DBGetInteger(dbRow!Sequence)
            ElseIf (DBGetLong(dbRow!AuthorID) = inactiveAuthor) Then
                inactiveSequence = DBGetInteger(dbRow!Sequence)
            End If
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
        If (activeSequence = -1) Or (inactiveSequence = -1) Then
            MessageBox.Show("Error occurred while trying to access the author or name records.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Update the database records.
        Me.Cursor = Cursors.WaitCursor
        Try
            transInfo = TransactionBegin()

            sqlText = "UPDATE ItemAuthor SET Sequence = @NewSequence " &
                "WHERE ItemID = @RecordID AND AuthorID = @AuthID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddInteger("NewSequence", inactiveSequence)
            sqlRun.AddLong("RecordID", ActiveID)
            sqlRun.AddLong("AuthID", activeAuthor)
            ExecuteSQL(sqlRun, transInfo)

            sqlRun = New DataCommand(sqlText)
            sqlRun.AddInteger("NewSequence", activeSequence)
            sqlRun.AddLong("RecordID", ActiveID)
            sqlRun.AddLong("AuthID", inactiveAuthor)
            ExecuteSQL(sqlRun, transInfo)

            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            GeneralError("NamedItem.SwitchTwoAuthors.Swap", ex)
            Return
        Finally
            Me.Cursor = Cursors.Default
        End Try

        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Reverse the elements on the display.
        holdText = activeItem.Text
        activeItem.Text = inactiveItem.Text
        inactiveItem.Text = holdText

        holdText = activeItem.SubItems(1).Text
        activeItem.SubItems(1).Text = inactiveItem.SubItems(1).Text
        inactiveItem.SubItems(1).Text = holdText

        holdText = CStr(activeItem.Tag)
        activeItem.Tag = inactiveItem.Tag
        inactiveItem.Tag = holdText

        ' ----- Highlight the correct item.
        AuthorsList.SelectedItems.Clear()
        inactiveItem.Selected = True
        UpdateAuthorButtons()
    End Sub

    Private Sub ActAuthorProperties_Click(sender As Object, e As EventArgs) Handles ActAuthorProperties.Click
        ' ----- Allow the use to change the name type.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newNameType As Long
        Dim oldNameType As Long
        Dim authorID As Long
        Dim holdText As String

        ' ----- Check for invalid data.
        If (AuthorsList.SelectedItems.Count = 0) Then Return

        ' ----- Get the IDs of the author and author type.
        holdText = CStr(AuthorsList.SelectedItems(0).Tag)
        authorID = CLng(holdText.GetSubStr(",", 1))
        oldNameType = CLng(holdText.GetSubStr(",", 2))

        ' ----- Prompt the user.
        newNameType = (New ItemAuthorEdit).EditRecord(ActiveID, authorID,
            BuildItemName(), AuthorsList.SelectedItems(0).Text, oldNameType)
        If (newNameType = -1&) Then Return
        ActCancel.Text = "Close"
        SessionSaved = True
        If (newNameType = oldNameType) Then Return

        ' ----- Update the internal value of the display item.
        AuthorsList.SelectedItems(0).Tag = authorID & "," & newNameType

        ' ----- Get the text for the new name type and update the display.
        sqlText = "SELECT FullName FROM CodeAuthorType WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", newNameType)
        Try
            AuthorsList.SelectedItems(0).SubItems(1).Text = DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("NamedItem.ActAuthorProperties_Click", ex)
        End Try
    End Sub

    Private Sub UpdateAuthorButtons()
        ' ----- Update the author panel buttons based on selection.
        If (AuthorsList.SelectedItems.Count = 0) Then
            ' ----- No selection.
            ActAuthorDelete.Enabled = False
            ActAuthorProperties.Enabled = False
            ActAuthorMoveUp.Enabled = False
            ActAuthorMoveDown.Enabled = False
        Else
            ' ----- Selection.
            ActAuthorDelete.Enabled = True
            ActAuthorProperties.Enabled = True
            ActAuthorMoveUp.Enabled = (AuthorsList.SelectedItems(0).Index > 0)
            ActAuthorMoveDown.Enabled = (AuthorsList.SelectedItems(0).Index <
                (AuthorsList.Items.Count - 1))
        End If
    End Sub

    Private Sub AuthorsList_SelectedIndexChanged(sender As Object, e As EventArgs) Handles AuthorsList.SelectedIndexChanged
        ' ----- Refresh the buttons.
        UpdateAuthorButtons()
    End Sub

    Private Sub AuthorsList_Click(sender As Object, e As EventArgs) Handles AuthorsList.Click
        ' ----- Refresh the buttons.
        UpdateAuthorButtons()
    End Sub

    Private Sub ActCopyAdd_Click(sender As Object, e As EventArgs) Handles ActCopyAdd.Click
        ' ----- Let the user add a new author.
        Dim copyID As Long
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim barcode As String
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim copyNumber As String
        Dim newItem As ListViewItem

        ' ----- The record must be saved first.
        If (ActiveID = -1&) Then
            ' ----- Confirm with the user.
            If (MessageBox.Show("The item must be saved to the database before copies " &
                "can be added. Would you like to save the record now?",
                ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
                DialogResult.Yes) Then Return

            ' ----- Verify and save the data.
            If (ValidateFormData() = False) Then Return
            If (SaveFormData() = False) Then Return
        End If

        ' ----- Prompt the user to add a subject.
        copyID = (New ItemCopy).AddCopy(ActiveID, BuildItemName())
        If (copyID = -1&) Then Return
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Get some information about the copy.
        sqlText = "SELECT CopyNumber, Barcode FROM ItemCopy WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", copyID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.ActCopyAdd_Click", ex)
            Return
        End Try
        oneRow = dbInfo.Rows(0)
        copyNumber = DBGetInteger(oneRow!CopyNumber).ToString()
        barcode = DBGetText(oneRow!Barcode)
        dbInfo.Dispose()
        dbInfo = Nothing
        If (barcode.Length = 0) Then barcode = "None"

        ' ----- Add this item to the display.
        newItem = CopyList.Items.Add(copyNumber)
        newItem.SubItems.Add(GetCopyDisposition(copyID))
        newItem.SubItems.Add(barcode)
        newItem.Tag = copyID.ToString()
        UpdateCopyButtons()
    End Sub

    Private Function BuildItemName() As String
        ' ----- Build the text of the item name used by some subordinate
        '       forms.
        Dim itemName As String

        ' ----- The name includes title, subtitle, edition, and media type.
        itemName = RecordTitle.Text
        If (RecordSubtitle.Text.Trim().Length <> 0) Then itemName &= ": " & RecordSubtitle.Text
        If (RecordEdition.Text.Trim().Length <> 0) Then itemName &= ", " & RecordEdition.Text
        If (CLng(DirectCast(RecordMediaType.SelectedItem, ListItemData)) <> -1&) Then _
            itemName &= " [" & RecordMediaType.Text & "]"
        Return itemName
    End Function

    Private Sub ActCopyEdit_Click(sender As Object, e As EventArgs) Handles ActCopyEdit.Click
        ' ----- Let the user edit an existing author.
        Dim copyID As Long
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim barcode As String
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim copyNumber As String
        Dim newItem As ListViewItem

        ' ----- Check for invalid data.
        If (CopyList.SelectedItems.Count = 0) Then Return
        newItem = CopyList.SelectedItems(0)
        copyID = CLng(newItem.Tag)

        ' ----- Prompt the user to add a subject.
        copyID = (New ItemCopy).EditCopy(ActiveID, copyID, BuildItemName())
        If (copyID = -1&) Then Return
        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Get some information about the copy.
        sqlText = "SELECT CopyNumber, Barcode FROM ItemCopy WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", copyID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("NamedItem.ActCopyEdit_Click", ex)
            Return
        End Try
        oneRow = dbInfo.Rows(0)
        copyNumber = DBGetInteger(oneRow!CopyNumber).ToString()
        barcode = DBGetText(oneRow!Barcode)
        dbInfo.Dispose()
        dbInfo = Nothing
        If (barcode.Length = 0) Then barcode = "None"

        ' ----- Update this item in the display.
        newItem.SubItems(0).Text = copyNumber
        newItem.SubItems(1).Text = GetCopyDisposition(copyID)
        newItem.SubItems(2).Text = barcode
        UpdateCopyButtons()
    End Sub

    Private Sub ActCopyDelete_Click(sender As Object, e As EventArgs) Handles ActCopyDelete.Click
        ' ----- Allow the user to delete an item copy.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim holdText As String
        Dim copyID As Long
        Dim transInfo As LibraryTransaction = Nothing

        ' ----- Make sure an item is selected.
        If (CopyList.SelectedItems.Count = 0) Then Return
        copyID = CLng(CopyList.SelectedItems(0).Tag)

        ' ----- Make sure this item is not checked out.
        holdText = CopyList.SelectedItems(0).SubItems(1).Text
        If (holdText = "Checked Out") Or (holdText = "Overdue") Then
            MessageBox.Show("You cannot delete this item because it is still checked out.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Confirm with the user.
        If (MessageBox.Show("Deleting an item copy from the library is a permanent " &
            "process; it cannot be undone without re-entering the item." &
            vbCrLf & vbCrLf & "Also, any patron history associated with this " &
            "item copy will be lost. This includes any unpaid fines associated " &
            "with the item copy." & vbCrLf & vbCrLf & "Modifying the item copy " &
            "and setting it to 'Not Available' is often a better choice." &
            vbCrLf & vbCrLf & "Really delete copy '" &
            CopyList.SelectedItems(0).Text & "' from the database?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return

        ' ----- Remove the selection from the database.
        Try
            transInfo = TransactionBegin()

            sqlText = "DELETE FROM PatronPayment WHERE PatronCopy IN " &
                "(SELECT ID FROM PatronCopy WHERE ItemCopy = @RecordID)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", copyID)
            ExecuteSQL(sqlRun, transInfo)

            sqlText = "DELETE FROM PatronCopy WHERE ItemCopy = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", copyID)
            ExecuteSQL(sqlRun, transInfo)

            sqlText = "DELETE FROM ItemCopy WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", copyID)
            ExecuteSQL(sqlRun, transInfo)

            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            GeneralError("NamedItem.ActCopyDelete_Click", ex)
            Return
        End Try

        ActCancel.Text = "Close"
        SessionSaved = True

        ' ----- Remove the item from the display.
        CopyList.Items.Remove(CopyList.SelectedItems(0))
        UpdateCopyButtons()
    End Sub

    Private Sub UpdateCopyButtons()
        ' ----- Update the author panel buttons based on selection.
        If (CopyList.SelectedItems.Count = 0) Then
            ' ----- No selection.
            ActCopyEdit.Enabled = False
            ActCopyDelete.Enabled = False
        Else
            ' ----- Selection.
            ActCopyEdit.Enabled = True
            ActCopyDelete.Enabled = True
        End If
    End Sub

    Private Sub CopyList_SelectedIndexChanged(sender As Object, e As EventArgs) Handles CopyList.SelectedIndexChanged
        ' ----- Refresh the buttons.
        UpdateCopyButtons()
    End Sub

    Private Sub CopyList_Click(sender As Object, e As EventArgs) Handles CopyList.Click
        ' ----- Refresh the buttons.
        UpdateCopyButtons()
    End Sub

    Private Sub AuthorsList_DoubleClick(sender As Object, e As EventArgs) Handles AuthorsList.DoubleClick
        ' ----- Same as a click on the edit button.
        If (ActAuthorProperties.Enabled = True) Then ActAuthorProperties.PerformClick()
    End Sub

    Private Sub CopyList_DoubleClick(sender As Object, e As EventArgs) Handles CopyList.DoubleClick
        ' ----- Same as a click on the edit button.
        If (ActCopyEdit.Enabled = True) Then ActCopyEdit.PerformClick()
    End Sub

    Private Sub RecordTitle_TextChanged(sender As Object, e As EventArgs) Handles RecordTitle.TextChanged
        ' ----- Update the large display title when the title changes.
        If (RecordTitle.Text.Trim().Length = 0) Then
            MainTitle.Text = "Untitled"
        Else
            MainTitle.Text = RecordTitle.Text.Trim()
        End If
    End Sub
End Class
