﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class PatronRecord
    Private ActivePatronID As Long
    Private SecondItemRow As Integer
    Private AllowPatronEdit As Boolean
    Private SessionPayments As Generic.List(Of PaymentItem)

    Public Sub ViewPatronRecord(patronID As Long, allowEdit As Boolean)
        ' ----- Provide access to the patron record.
        ActivePatronID = patronID
        AllowPatronEdit = allowEdit
        Me.ShowDialog()
    End Sub

    Private Sub RefreshPatronFines(forcePatronCopy As Long)
        ' ----- Refresh the list of fines due by and paid by the patron.
        '       Always include the forcePatronCopy item, unless it is -1.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim fineAmount As Decimal
        Dim paidAmount As Decimal
        Dim balanceAmount As Decimal
        Dim totalBalance As Decimal
        Dim boldFont As Font
        Dim titleText As String
        Dim copyText As String
        Dim newEntry As PatronDetailItem
        Dim canvas As Drawing.Graphics

        ' ----- Create graphic objects used for measuring.
        boldFont = New Font(Fines.Font, FontStyle.Bold)
        canvas = Fines.CreateGraphics()

        ' ----- Clear the current list.
        Fines.Items.Clear()
        totalBalance = 0@
        BalanceDue.Text = (0@).ToString("C")
        Me.Cursor = Cursors.WaitCursor

        ' ----- Retrieve the fine records for the patron.
        sqlText = "SELECT PC.ID, NI.Title, NI.Subtitle, PC.Fine, PC.Paid, " &
            "IC.CopyNumber FROM PatronCopy AS PC " &
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " &
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " &
            "WHERE (PC.Patron = " & ActivePatronID &
            " AND (PC.Fine <> 0 OR PC.Paid <> 0))"
        If (forcePatronCopy <> -1&) Then sqlText &= " OR (PC.ID = @RecordID)"
        sqlText &= " ORDER BY NI.Title"
        sqlRun = New DataCommand(sqlText)
        If (forcePatronCopy <> -1&) Then _
            sqlRun.AddLong("RecordID", forcePatronCopy)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("PatronRecord.RefreshPatronFines.Retrieve", ex)
            canvas.Dispose()
            boldFont.Dispose()
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Collect the values.
            fineAmount = DBGetDecimal(dbRow!Fine)
            paidAmount = DBGetDecimal(dbRow!Paid)
            balanceAmount = fineAmount - paidAmount
            If (DBGetLong(dbRow!ID) <> forcePatronCopy) Then
                If (balanceAmount = 0@) And (ShowPaidFines.Checked = False) Then _
                    Continue For
            End If
            totalBalance += balanceAmount

            ' ----- Format the display items.
            copyText = " [Copy #" & DBGetInteger(dbRow!CopyNumber).ToString() & "]"
            titleText = DBGetText(dbRow!Title)
            If (IsDBNull(dbRow!Subtitle) = False) Then _
                titleText &= ": " & DBGetText(dbRow!Subtitle)
            titleText = FitTextToWidth(titleText,
                CInt(FineColEnd.Left - FineColTitle.Left -
                canvas.MeasureString(copyText, boldFont).Width()),
                canvas, boldFont)

            ' ----- Prepare a new list item.
            newEntry = New PatronDetailItem
            newEntry.DetailID = DBGetLong(dbRow!ID)
            newEntry.TitleText = titleText & copyText
            newEntry.FineAmount = fineAmount
            newEntry.PaidAmount = paidAmount
            newEntry.BalanceDue = balanceAmount
            Fines.Items.Add(newEntry)
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- Show the total balance.
        BalanceDue.Text = totalBalance.ToString("C")
        ActFinePay.Enabled = False

        ' ----- Dispense with the graphics elements.
        canvas.Dispose()
        boldFont.Dispose()
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub ActRenewItemsOut_Click(sender As Object, e As EventArgs) Handles ActRenewItemsOut.Click
        ' ----- Try to renew a checked-out item.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim checkOutDate As Date
        Dim dueDate As Date
        Dim renewsSoFar As Integer
        Dim maxRenew As Integer
        Dim renewDays As Integer
        Dim itemDetail As PatronDetailItem
        Dim transInfo As LibraryTransaction = Nothing

        ' ----- Make sure an item is selected.
        If (ItemsOut.SelectedIndex = -1) Then Return
        itemDetail = DirectCast(ItemsOut.SelectedItem, PatronDetailItem)

        ' ----- Make sure this is not a reference item.
        sqlText = "SELECT IC.Available, IC.Reference FROM PatronCopy AS PC " &
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " &
            "WHERE PC.ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", itemDetail.DetailID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("PatronRecord.ActRenewItemsOut_Click.Check", ex)
            Return
        End Try
        oneRow = dbInfo.Rows(0)
        If (DBGetBool(oneRow!Available) = False) Then
            dbInfo.Dispose()
            dbInfo = Nothing
            MessageBox.Show("You cannot renew this item because it is no " &
                "longer available for circulation or checkout.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If
        If (DBGetBool(oneRow!Reference) = True) Then
            dbInfo.Dispose()
            dbInfo = Nothing
            MessageBox.Show("You cannot renew this item because it is a reference item.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If
        dbInfo.Dispose()

        ' ----- Get the status of the checked out item.
        sqlText = "SELECT CheckOut, DueDate, Renewal FROM PatronCopy WHERE ID = " &
            itemDetail.DetailID
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", itemDetail.DetailID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("PatronRecord.ActRenewItemsOut_Click.Status", ex)
            Return
        End Try
        oneRow = dbInfo.Rows(0)
        checkOutDate = DBGetDate(oneRow!CheckOut)
        dueDate = DBGetDate(oneRow!DueDate)
        renewsSoFar = DBGetInteger(oneRow!Renewal)
        dbInfo.Dispose()

        ' ----- Determine the limits on this type of item.
        sqlText = "SELECT * FROM CodeMediaType WHERE ID = (SELECT NI.MediaType " &
            "FROM PatronCopy AS PC INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " &
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " &
            "WHERE PC.ID = @RecordID)"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", itemDetail.DetailID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("PatronRecord.ActRenewItemsOut_Click.Limit", ex)
            Return
        End Try
        oneRow = dbInfo.Rows(0)
        maxRenew = DBGetInteger(oneRow!RenewTimes)
        renewDays = DBGetInteger(oneRow!RenewDays)
        dbInfo.Dispose()
        dbInfo = Nothing

        ' ----- See if this item has been renewed too often.
        If (renewsSoFar >= maxRenew) Then
            If (maxRenew = 0) Then
                MessageBox.Show("You cannot renew this item because this type of " &
                    "item does not permit renewals.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Else
                MessageBox.Show("You cannot renew this item because you have already renewed it " &
                    If(renewsSoFar = 1, "1 time", renewsSoFar & " times") &
                    ", which is the limit for this type of item.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End If
            Return
        End If
        Me.Cursor = Cursors.WaitCursor

        ' ----- Renew this item by adjusting the due date. While there is room,
        '       keep renewing the item until the item is not overdue, if needed.
        Do While True
            dueDate = dueDate.AddDays(renewDays)
            renewsSoFar += 1
            If (renewsSoFar >= maxRenew) Then Exit Do
            If (dueDate >= Today) Then Exit Do
        Loop

        ' ----- Don't allow the due date to be on a holiday.
        Do While (IsHolidayDate(dueDate) = True)
            dueDate = dueDate.AddDays(1)
        Loop

        Try
            transInfo = TransactionBegin()

            ' ----- Update the record.
            sqlText = "UPDATE PatronCopy SET DueDate = @TargetDate, " &
                "Renewal = @RenewalPos WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddDate("TargetDate", dueDate)
            sqlRun.AddInteger("RenewalPos", renewsSoFar)
            sqlRun.AddLong("RecordID", itemDetail.DetailID)
            ExecuteSQL(sqlRun, transInfo)

            ' ----- Update the patron record.
            sqlText = "UPDATE Patron SET LastActivity = GETDATE() " &
                "WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", ActivePatronID)
            ExecuteSQL(sqlRun, transInfo)

            TransactionCommit(transInfo)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            TransactionRollback(transInfo)
            GeneralError("PatronRecord.ActRenewItemsOut_Click.Update", ex)
            Return
        End Try

        ' ----- Update the display.
        itemDetail.DueDate = dueDate
        ItemsOut.Invalidate(ItemsOut.GetItemRectangle(ItemsOut.SelectedIndex))
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub PatronRecord_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' ----- Prepare the form.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim holdText As String
        Dim canvas As Drawing.Graphics
        Dim newEntry As PatronDetailItem

        ' ----- Adjust the line item height of the fines list.
        canvas = Fines.CreateGraphics()
        Fines.ItemHeight = CInt(canvas.MeasureString("A" & vbCrLf & "g",
            Fines.Font).Height()) + 3
        SecondItemRow = CInt(canvas.MeasureString("Ag", Fines.Font).Height()) + 1
        canvas.Dispose()

        ' ----- Load in the patron name and barcode.
        sqlText = "SELECT LastName, FirstName, Active, Barcode FROM Patron " &
            "WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActivePatronID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("PatronRecord.PatronRecord_Load.Patron", ex)
            dbInfo = Nothing
        End Try
        If (dbInfo IsNot Nothing) Then
            oneRow = dbInfo.Rows(0)
            PatronName.Text = DBGetText(oneRow!FirstName) & " " & DBGetText(oneRow!LastName)
            If (DBGetBool(oneRow!Active) = False) Then PatronName.Text &= " [Inactive]"
            If (IsDBNull(oneRow!Barcode) = True) Then PatronBarcode.Text = "None" Else _
                PatronBarcode.Text = DBGetText(oneRow!Barcode)
            dbInfo.Dispose()
        End If

        ' ----- Load in the list of items checked out.
        sqlText = "SELECT PC.ID, NI.Title, NI.Subtitle, PC.DueDate " &
            "FROM PatronCopy AS PC INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " &
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " &
            "WHERE PC.Patron = @RecordID " &
            "AND PC.Returned = 0 AND PC.Missing = 0 ORDER BY PC.DueDate"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActivePatronID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("PatronRecord.PatronRecord_Load.Items", ex)
            dbInfo = Nothing
        End Try
        If (dbInfo IsNot Nothing) Then
            For Each dbRow As DataRow In dbInfo.Rows
                ' ----- Add this checked-out or overdue book to the list.
                holdText = DBGetText(dbRow!Title)
                If (IsDBNull(dbRow!Subtitle) = False) Then _
                    holdText &= ": " & DBGetText(dbRow!Subtitle)

                newEntry = New PatronDetailItem
                newEntry.DetailID = DBGetLong(dbRow!ID)
                newEntry.TitleText = holdText
                newEntry.DueDate = DBGetDate(dbRow!DueDate)
                ItemsOut.Items.Add(newEntry)
            Next dbRow
            dbInfo.Dispose()
        End If

        ' ***** Insert Code Snippet #9 here.

        ' ----- Load in the list of patron fines.
        RefreshPatronFines(-1&)

        ' ----- Prepare a list of payments for printing.
        SessionPayments = New Generic.List(Of PaymentItem)

        ' ----- If the user has authorization, show the Edit Patron button.
        ActPasswordPatron.Location = New Point(
            ActEditPatron.Location.X, ActEditPatron.Location.Y)
        If (SecurityProfile(LibrarySecurity.ManagePatrons) = True) Then
            ' ----- If this form was called ultimately from the edit patron form,
            '       then hide the Edit Patron button.
            ActEditPatron.Visible = AllowPatronEdit
            ActPasswordPatron.Visible = Not AllowPatronEdit
        Else
            ' ----- This user is not authorized to edit patron records.
            ActEditPatron.Visible = False
            ActPasswordPatron.Visible = True
        End If

        ' ----- Only allow authorized users to adjust fines.
        ActFinePay.Visible = SecurityProfile(LibrarySecurity.ProcessFees)
    End Sub

    Private Sub ActEditPatron_Click(sender As Object, e As EventArgs) Handles ActEditPatron.Click
        ' ----- Let the user edit the patron record.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow

        ' ----- Display the patron form.
        If ((New Patron).EditRecordLimited(ActivePatronID) <> -1&) Then
            ' ----- Update the patron's name and barcode number.
            sqlText = "SELECT LastName, FirstName, Active, Barcode FROM Patron " &
                "WHERE ID = " & ActivePatronID
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", ActivePatronID)
            Try
                dbInfo = CreateDataTable(sqlRun)
            Catch ex As Exception
                GeneralError("PatronRecord.ActEditPatron_Click", ex)
                Return
            End Try

            oneRow = dbInfo.Rows(0)
            PatronName.Text = DBGetText(oneRow!FirstName) & " " & DBGetText(oneRow!LastName)
            If (DBGetBool(oneRow!Active) = False) Then PatronName.Text &= " [Inactive]"
            If (IsDBNull(oneRow!Barcode) = True) Then _
                PatronBarcode.Text = "None" Else _
                PatronBarcode.Text = DBGetText(oneRow!Barcode)
            dbInfo.Dispose()
        End If
    End Sub

    Private Sub ItemsOut_SelectedIndexChanged(sender As Object, e As EventArgs) Handles ItemsOut.SelectedIndexChanged
        ' ----- Enable the Renew button if needed.
        ActRenewItemsOut.Enabled = CBool(ItemsOut.SelectedIndex <> -1)
    End Sub

    Private Sub ItemsOut_Click(sender As Object, e As EventArgs) Handles ItemsOut.Click
        ' ----- Enable the Renew button if needed.
        ActRenewItemsOut.Enabled = CBool(ItemsOut.SelectedIndex <> -1)
    End Sub

    Private Sub ItemsOut_DrawItem(sender As Object, e As DrawItemEventArgs) Handles ItemsOut.DrawItem
        ' ----- Specially format the checked out item.
        Dim useBrush As Brush
        Dim dueDate As String
        Dim itemDetail As PatronDetailItem
        Dim ellipsesText As StringFormat

        ' ----- Draw the background of the item.
        e.DrawBackground()
        If (e.Index = -1) Then Return

        ' ----- Extract the detail from the list item.
        itemDetail = DirectCast(ItemsOut.Items(e.Index), PatronDetailItem)
        dueDate = itemDetail.DueDate.ToString("MMM d, yyyy")

        ' ----- Print the title in the right color.
        If ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Then
            useBrush = SystemBrushes.HighlightText
        Else
            useBrush = SystemBrushes.WindowText
        End If
        ellipsesText = New StringFormat
        ellipsesText.Trimming = StringTrimming.EllipsisCharacter
        e.Graphics.DrawString(itemDetail.TitleText, e.Font, useBrush,
            New Rectangle(0, e.Bounds.Top,
            ItemsOutColDue.Left - ItemsOutColTitle.Left - 5, e.Font.Height),
            ellipsesText)

        ' ----- Print the due date in the right color.
        If ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Then
            useBrush = SystemBrushes.HighlightText
        ElseIf (CDate(dueDate) < Today) Then
            ' ----- Overdue.
            useBrush = Brushes.Red
        Else
            useBrush = SystemBrushes.WindowText
        End If
        e.Graphics.DrawString(dueDate, e.Font, useBrush,
            ItemsOutColDue.Left - ItemsOutColTitle.Left, e.Bounds.Top)

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
    End Sub

    Private Sub ShowPaidFines_CheckedChanged(sender As Object, e As EventArgs) Handles ShowPaidFines.CheckedChanged
        ' ----- Refresh the list of fines based on the new setting.
        RefreshPatronFines(-1&)
    End Sub

    Private Sub Fines_SelectedIndexChanged(sender As Object, e As EventArgs) Handles Fines.SelectedIndexChanged
        ' ----- Enable the buttons if needed.
        ActFinePay.Enabled = CBool(Fines.SelectedIndex <> -1)
    End Sub

    Private Sub Fines_Click(sender As Object, e As EventArgs) Handles Fines.Click
        ' ----- Enable the buttons if needed.
        ActFinePay.Enabled = CBool(Fines.SelectedIndex <> -1)
    End Sub

    Private Sub Fines_DrawItem(sender As Object, e As DrawItemEventArgs) Handles Fines.DrawItem
        ' ----- Draw the matching items on two lines.
        Dim useBrush As Drawing.Brush
        Dim useNotice As Drawing.Brush
        Dim titleText As String
        Dim fineText As String
        Dim paidText As String
        Dim balanceText As String
        Dim boldFont As Drawing.Font
        Dim itemDetail As PatronDetailItem

        ' ----- Draw the background of the item.
        If ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Then
            useBrush = SystemBrushes.HighlightText
            useNotice = SystemBrushes.HighlightText
        Else
            useBrush = SystemBrushes.WindowText
            useNotice = Brushes.Red
        End If
        e.DrawBackground()

        ' ----- Make a bold version of the font.
        boldFont = New Font(e.Font, FontStyle.Bold)

        ' ----- Extract the details from the list item.
        itemDetail = DirectCast(Fines.Items(e.Index), PatronDetailItem)
        titleText = itemDetail.TitleText
        fineText = itemDetail.FineAmount.ToString("C")
        paidText = itemDetail.PaidAmount.ToString("C")
        balanceText = itemDetail.BalanceDue.ToString("C")
        If (itemDetail.BalanceDue = 0@) Then useNotice = useBrush

        ' ----- Draw the text of the item.
        e.Graphics.DrawString(titleText, boldFont, useBrush, 0, e.Bounds.Top)
        e.Graphics.DrawString(fineText, e.Font, useBrush, FineColFine.Left -
            FineColTitle.Left, e.Bounds.Top + SecondItemRow)
        e.Graphics.DrawString(paidText, e.Font, useBrush, FineColPaid.Left -
            FineColTitle.Left, e.Bounds.Top + SecondItemRow)
        e.Graphics.DrawString(balanceText, e.Font, useNotice, FineColBalance.Left -
            FineColTitle.Left, e.Bounds.Top + SecondItemRow)

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
        boldFont.Dispose()
    End Sub

    Private Sub ActFinePay_Click(sender As Object, e As EventArgs) Handles ActFinePay.Click
        ' ----- Prompt the user to adjust the fines on an item.
        Dim patronCopyID As Long

        ' ----- An item must be selected.
        If (Fines.SelectedIndex = -1) Then
            MessageBox.Show("Please select a fine from the list.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If
        patronCopyID = DirectCast(Fines.SelectedItem, PatronDetailItem).DetailID

        ' ----- Prompt the user to make changes.
        Call (New PatronPayment).ManagePayments(patronCopyID, SessionPayments)

        ' ----- Refresh the fines list.
        ActFinePay.Enabled = False
        RefreshPatronFines(patronCopyID)
        Fines.SelectedIndex = Fines.Items.IndexOf(patronCopyID)
    End Sub

    Private Sub ActPasswordPatron_Click(sender As Object, e As EventArgs) Handles ActPasswordPatron.Click
        ' ----- Let the user change their password.
        If ((New PatronPassword).ChangePassword(ActivePatronID) = True) Then
            MessageBox.Show("Password changed successfully.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
    End Sub

    Private Sub ActClose_Click(sender As Object, e As EventArgs) Handles ActClose.Click
        ' ----- Print out a ticket if needed.
        If (SessionPayments.Count > 0) Then _
           PrintPaymentTicket(ActivePatronID, SessionPayments)
        SessionPayments.Clear()
        SessionPayments = Nothing

        ' ----- Close the form.
        Me.DialogResult = DialogResult.Cancel
    End Sub

    Private Sub ActBalanceTicket_Click(sender As Object, e As EventArgs) Handles ActBalanceTicket.Click
        ' ----- Print a ticket of all balances.
        PrintBalanceTicket(ActivePatronID, Fines)
    End Sub
End Class
