﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class Publisher
    Private ActiveID As Long
    Private StartingOver As Boolean

    Public Overloads Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overloads Function AddRecord(pubText As String) As Long
        ' ----- Add a new record, but use a starting value previously entered
        '       by the user.
        ActiveID = -1&
        PrepareFormFields()
        RecordFullName.Text = pubText
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then Return _
            ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManagePublishers)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManagePublishers)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManagePublishers)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the publisher?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM NamedItem WHERE Publisher = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("You cannot delete this record because it is in use.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        Catch ex As Exception
            GeneralError("Publisher.DeleteRecord.Check", ex)
            Return False
        End Try

        ' ----- Delete the record.
        sqlText = "DELETE FROM Publisher WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            ExecuteSQL(sqlRun)
        Catch ex As Exception
            GeneralError("Publisher.DeleteRecord.Delete", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record.
        ActiveID = recordID
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items. Prompt the
        '       user first.
        destList.Items.Clear()
        SearchForRecord(destList, exceededMatches)
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT FullName FROM Publisher WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            Return DBGetText(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("Publisher.FormatRecordName", ex)
            Return "Error"
        End Try
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "This form lets you defined the list of publishers that distribute " &
            "items found in the library."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Publishers"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim inUse As Integer

        ' ----- Clear the existing data.
        StartingOver = True
        RecordFullName.Text = ""
        RecordWeb.Text = ""
        RecordStatus.Text = "New publisher record"
        If (ActiveID = -1&) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM Publisher WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Publisher.PrepareFormFields.Retrieve", ex)
            Return
        End Try
        If (dbInfo.Rows.Count > 0) Then
            oneRow = dbInfo.Rows(0)
            RecordFullName.Text = DBGetText(oneRow!FullName)
            RecordWeb.Text = DBGetText(oneRow!WebSite)
        End If
        dbInfo.Dispose()

        ' ----- Find out how many records link to this publisher.
        sqlText = "SELECT COUNT(*) FROM NamedItem WHERE Publisher = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            inUse = DBGetInteger(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("Publisher.PrepareFormFields.Link", ex)
            inUse = 0
        End Try
        RecordStatus.Text = "Linked to " & inUse.ToString() & " library item"
        If (inUse <> 1) Then RecordStatus.Text &= "s"
    End Sub

    Public Overrides Sub SearchForRecord(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Search for publishers using user-specified criteria.
        Dim sqlText As String
        Dim dbInfo As DataTable
        Dim sqlRun As DataCommand
        Dim limitPattern As String
        Dim resultCount As Integer

        ' ----- Prompt the user for the limited publisher name.
        exceededMatches = False
        limitPattern = (New PublisherLimit).PromptUser()
        If (limitPattern.Length = 0) Then Return
        If (limitPattern.Contains("*") = False) Then limitPattern &= "*"

        ' ----- Clear any existing values.
        destList.Items.Clear()
        resultCount = 0

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, FullName FROM Publisher " &
            "WHERE FullName LIKE @TestName ORDER BY FullName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestName", limitPattern.Trim().Replace("*", "%"))
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Publisher.SearchForRecord", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            resultCount += 1
            If (resultCount > SearchMatchLimit) Then
                exceededMatches = True
                Exit For
            End If
            destList.Items.Add(New ListItemData(
                DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function UsesSearch() As Boolean
        ' ----- Publisher records are searchable.
        Return True
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save changes to the record.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long

        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO Publisher (FullName, WebSite) " &
                "OUTPUT INSERTED.ID VALUES (@NewName, @NewSite)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            sqlRun.AddText("NewSite", RecordWeb.Text.Trim())
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                GeneralError("Publisher.SaveFormData.Add", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE Publisher SET FullName = @NewName, " &
                "WebSite = @NewSite WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewName", RecordFullName.Text.Trim())
            sqlRun.AddText("NewSite", RecordWeb.Text.Trim())
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("Publisher.SaveFormData.Update", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Name is a required field.
        If (RecordFullName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the publisher name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordFullName.Focus()
            Return False
        End If

        ' ----- Make sure the name is not already in use.
        sqlText = "SELECT COUNT(*) FROM Publisher WHERE UPPER(FullName) = @TestName"
        If (ActiveID <> -1&) Then sqlText &= " AND ID <> @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestName", RecordFullName.Text.Trim().ToUpper())
        If (ActiveID <> -1&) Then _
            sqlRun.AddLong("RecordID", ActiveID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                ' ----- Duplicate entry.
                MessageBox.Show("A record already exists with the name '" &
                    RecordFullName.Text.Trim() & "'.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                RecordFullName.Focus()
                Return False
            End If
        Catch ex As Exception
            GeneralError("Publisher.ValidateFormData", ex)
            Return False
        End Try

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub Publisher_Activated(sender As Object, e As EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordFullName.Focus()
        StartingOver = False
    End Sub

    Private Sub RecordFullName_Enter(sender As Object, e As EventArgs) Handles RecordFullName.Enter
        ' ----- Highlight the entire text.
        RecordFullName.SelectAll()
    End Sub

    Private Sub RecordWeb_Enter(sender As Object, e As EventArgs) Handles RecordWeb.Enter
        ' ----- Highlight the entire text.
        RecordWeb.SelectAll()
    End Sub

    Private Sub ShowWeb_Click(sender As Object, e As EventArgs) Handles ShowWeb.Click
        ' ***** Insert Code Snippet #7 here.

    End Sub

    Private Sub RecordWeb_TextChanged(sender As Object, e As EventArgs) Handles RecordWeb.TextChanged
        ' ----- Enable the web site button as needed.
        ShowWeb.Enabled = CBool(RecordWeb.Text.Trim().Length <> 0)
    End Sub
End Class