﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class ReportSelect
    Private Sub LoadReportGroup(groupNode As XElement, indentLevel As Integer)
        ' ----- Add the groups and items at this level,
        '       and recurse as needed.
        Dim detailNode As XElement
        Dim reportEntry As ReportItem

        ' ----- Process each item or group.
        For Each scanNode As XElement In groupNode.Nodes
            ' ----- Build a content item for the list.
            reportEntry = New ReportItem
            reportEntry.Indent = indentLevel

            ' ----- Get the display name.
            detailNode = scanNode.<displayText>
            If (detailNode Is Nothing) Then Continue For
            reportEntry.DisplayText = detailNode.Value

            If (scanNode.Name = "reportGroup") Then
                ' ----- Start a new display group.
                reportEntry.ItemType = ReportItemEnum.GroupLabel
                AllReports.Items.Add(reportEntry)

                ' ----- Recurse to child items.
                LoadReportGroup(scanNode, indentLevel + 1)
            ElseIf (scanNode.Name = "reportItem") Then
                ' ----- This is an item. Record its location.
                detailNode = scanNode.<reportPath>
                If (detailNode IsNot Nothing) Then _
                   reportEntry.ReportPath = detailNode.Value

                ' ----- Get any command-line arguments.
                detailNode = scanNode.<reportArgs>
                If (detailNode IsNot Nothing) Then _
                   reportEntry.ReportArgs = detailNode.Value

                ' ----- Get any item-specific flags.
                detailNode = scanNode.<reportFlags>
                If (detailNode IsNot Nothing) Then
                    ' ---- "U" adds "-u loginid" to the command.
                    If (detailNode.Value.ToUpper().Contains("U") = True) And
                       (LoggedInUserName.Length <> 0) Then reportEntry.ReportArgs =
                       (reportEntry.ReportArgs & " -u " & LoggedInUserName).Trim()
                End If

                ' ----- Store the full description.
                detailNode = scanNode.<description>
                If (detailNode IsNot Nothing) Then _
                   reportEntry.Description = detailNode.Value

                ' ----- So, what type of entry is it?
                If (scanNode.@type = "built-in") Then
                    ' ----- Built-in program. Check for valid ID.
                    If (IsNumeric(reportEntry.ReportPath) = False) Or
                       (Val(reportEntry.ReportPath) < 1) Or
                       (Val(reportEntry.ReportPath) >
                       CInt(ReportItemEnum.BuiltInStatistics)) Then _
                       Continue For
                    reportEntry.ItemType = CType(CInt(
                       reportEntry.ReportPath), ReportItemEnum)
                    AllReports.Items.Add(reportEntry)
                ElseIf (scanNode.@type = "program") Then
                    ' ----- EXE program-based report.
                    If (reportEntry.ReportPath.Length = 0) Then Continue For
                    reportEntry.ItemType = ReportItemEnum.ExeProgram
                    AllReports.Items.Add(reportEntry)
                ElseIf (scanNode.@type = "url") Then
                    ' ----- URL-based report.
                    If (reportEntry.ReportPath.Length = 0) Then Continue For
                    reportEntry.ItemType = ReportItemEnum.UrlProgram
                    AllReports.Items.Add(reportEntry)
                End If
            End If
        Next scanNode
    End Sub

    Private Sub RefreshReportList()
        ' ----- Load in the list of available reports.
        Dim configFile As String
        Dim configData As XDocument
        Dim reportEntry As ReportItem
        Dim counter As Integer

        ' ----- Clear the existing list.
        AllReports.Items.Clear()

        ' ----- Get the location of the configuration file.
        configFile = My.Settings.ReportConfig & ""

        ' ----- Load the configuration file.
        If (configFile.Length <> 0) Then
            If (System.IO.File.Exists(configFile)) Then
                Try
                    ' ----- Load in the file.
                    configData = XDocument.Load(configFile)

                    ' ----- Process the configuration file.
                    LoadReportGroup(configData.FirstNode, 0)
                Catch ex As Exception
                    GeneralError("ReportSelect.RefreshReportList", ex)
                End Try
            End If
        End If

        ' ----- If the configuration file resulted in no reports
        '       appearing in the list, add the default reports.
        If (AllReports.Items.Count = 0) Then
            For counter = 1 To CInt(ReportItemEnum.BuiltInStatistics)
                ' ----- Build the report entry.
                reportEntry = New ReportItem
                reportEntry.Indent = 0
                reportEntry.ItemType = CType(counter, ReportItemEnum)
                Select Case reportEntry.ItemType
                    Case ReportItemEnum.BuiltInCheckedOut
                        reportEntry.DisplayText = "Items Checked Out"
                    Case ReportItemEnum.BuiltInOverdue
                        reportEntry.DisplayText = "Items Overdue"
                    Case ReportItemEnum.BuiltInMissing
                        reportEntry.DisplayText = "Items Missing"
                    Case ReportItemEnum.BuiltInFinesOwed
                        reportEntry.DisplayText = "Patron Fines Owed"
                    Case ReportItemEnum.BuiltInStatistics
                        reportEntry.DisplayText = "Database Statistics"
                End Select

                ' ----- Add the report entry to the list.
                AllReports.Items.Add(reportEntry)
            Next counter
        End If
    End Sub

    Private Sub AllReports_SelectedIndexChanged(sender As Object, e As EventArgs) Handles AllReports.SelectedIndexChanged
        ' ----- Display a description of the report, if available.
        Dim reportEntry As Library.ReportItem

        ' ----- Clear any previous description.
        FullDescription.Text = "No report selected."
        If (AllReports.SelectedIndex <> -1) Then
            ' ----- Locate the content and display it.
            reportEntry = DirectCast(AllReports.SelectedItem, Library.ReportItem)
            FullDescription.Text = reportEntry.Description
        End If
    End Sub

    Private Sub ReportSelect_Load(sender As Object, e As EventArgs) Handles Me.Load
        ' ----- Display the list of reports.
        RefreshReportList()
    End Sub

    Private Sub ActRun_Click(sender As Object, e As EventArgs) Handles ActRun.Click
        ' ----- Run the selected report.
        Dim reportEntry As Library.ReportItem

        ' ----- Make sure a report is selected.
        If (AllReports.SelectedIndex = -1) Then
            MessageBox.Show("Please select a report from the list.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Different code for each type of entry.
        reportEntry = DirectCast(AllReports.SelectedItem, Library.ReportItem)
        Me.Cursor = Cursors.WaitCursor
        Select Case reportEntry.ItemType
            Case ReportItemEnum.GroupLabel
                ' ----- No report for group entries.
                MessageBox.Show("Please select a report from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Case ReportItemEnum.BuiltInCheckedOut
                ' ----- Items Checked Out
                ' TODO: Write BasicReportCheckedOut()
            Case ReportItemEnum.BuiltInOverdue
                ' ----- Items Overdue
                ' TODO: Write BasicReportOverdue()
            Case ReportItemEnum.BuiltInMissing
                ' ----- Items Missing
                ' TODO: Write BasicReportMissing()
            Case ReportItemEnum.BuiltInFinesOwed
                ' ----- Fines Owed by Patrons
                ' TODO: Write BasicReportFines()
            Case ReportItemEnum.BuiltInStatistics
                ' ----- Library Database Statistics
                ' TODO: Write BasicReportStatistics()
            Case ReportItemEnum.ExeProgram
                ' ----- Start a program.
                Try
                    Process.Start("""" & reportEntry.ReportPath & """ " &
                        reportEntry.ReportArgs)
                Catch ex As Exception
                    GeneralError("ReportSelect.ActRun_Click.Exe", ex)
                End Try
            Case ReportItemEnum.UrlProgram
                ' ----- Start a URL.
                Process.Start(reportEntry.ReportPath)
        End Select
        Me.Cursor = Cursors.Default
    End Sub
End Class
